class A {
  /**
  Get the line description around the given position.
  */
  lineAt(t) {
    if (t < 0 || t > this.length)
      throw new RangeError(`Invalid position ${t} in document of length ${this.length}`);
    return this.lineInner(t, !1, 1, 0);
  }
  /**
  Get the description for the given (1-based) line number.
  */
  line(t) {
    if (t < 1 || t > this.lines)
      throw new RangeError(`Invalid line number ${t} in ${this.lines}-line document`);
    return this.lineInner(t, !0, 1, 0);
  }
  /**
  Replace a range of the text with the given content.
  */
  replace(t, e, i) {
    [t, e] = At(this, t, e);
    let n = [];
    return this.decompose(
      0,
      t,
      n,
      2
      /* Open.To */
    ), i.length && i.decompose(
      0,
      i.length,
      n,
      3
      /* Open.To */
    ), this.decompose(
      e,
      this.length,
      n,
      1
      /* Open.From */
    ), _.from(n, this.length - (e - t) + i.length);
  }
  /**
  Append another document to this one.
  */
  append(t) {
    return this.replace(this.length, this.length, t);
  }
  /**
  Retrieve the text between the given points.
  */
  slice(t, e = this.length) {
    [t, e] = At(this, t, e);
    let i = [];
    return this.decompose(t, e, i, 0), _.from(i, e - t);
  }
  /**
  Test whether this text is equal to another instance.
  */
  eq(t) {
    if (t == this)
      return !0;
    if (t.length != this.length || t.lines != this.lines)
      return !1;
    let e = this.scanIdentical(t, 1), i = this.length - this.scanIdentical(t, -1), n = new Wt(this), s = new Wt(t);
    for (let o = e, l = e; ; ) {
      if (n.next(o), s.next(o), o = 0, n.lineBreak != s.lineBreak || n.done != s.done || n.value != s.value)
        return !1;
      if (l += n.value.length, n.done || l >= i)
        return !0;
    }
  }
  /**
  Iterate over the text. When `dir` is `-1`, iteration happens
  from end to start. This will return lines and the breaks between
  them as separate strings.
  */
  iter(t = 1) {
    return new Wt(this, t);
  }
  /**
  Iterate over a range of the text. When `from` > `to`, the
  iterator will run in reverse.
  */
  iterRange(t, e = this.length) {
    return new yn(this, t, e);
  }
  /**
  Return a cursor that iterates over the given range of lines,
  _without_ returning the line breaks between, and yielding empty
  strings for empty lines.
  
  When `from` and `to` are given, they should be 1-based line numbers.
  */
  iterLines(t, e) {
    let i;
    if (t == null)
      i = this.iter();
    else {
      e == null && (e = this.lines + 1);
      let n = this.line(t).from;
      i = this.iterRange(n, Math.max(n, e == this.lines + 1 ? this.length : e <= 1 ? 0 : this.line(e - 1).to));
    }
    return new wn(i);
  }
  /**
  Return the document as a string, using newline characters to
  separate lines.
  */
  toString() {
    return this.sliceString(0);
  }
  /**
  Convert the document to an array of lines (which can be
  deserialized again via [`Text.of`](https://codemirror.net/6/docs/ref/#state.Text^of)).
  */
  toJSON() {
    let t = [];
    return this.flatten(t), t;
  }
  /**
  @internal
  */
  constructor() {
  }
  /**
  Create a `Text` instance for the given array of lines.
  */
  static of(t) {
    if (t.length == 0)
      throw new RangeError("A document must have at least one line");
    return t.length == 1 && !t[0] ? A.empty : t.length <= 32 ? new P(t) : _.from(P.split(t, []));
  }
}
class P extends A {
  constructor(t, e = Rs(t)) {
    super(), this.text = t, this.length = e;
  }
  get lines() {
    return this.text.length;
  }
  get children() {
    return null;
  }
  lineInner(t, e, i, n) {
    for (let s = 0; ; s++) {
      let o = this.text[s], l = n + o.length;
      if ((e ? i : l) >= t)
        return new Es(n, l, i, o);
      n = l + 1, i++;
    }
  }
  decompose(t, e, i, n) {
    let s = t <= 0 && e >= this.length ? this : new P(bi(this.text, t, e), Math.min(e, this.length) - Math.max(0, t));
    if (n & 1) {
      let o = i.pop(), l = ae(s.text, o.text.slice(), 0, s.length);
      if (l.length <= 32)
        i.push(new P(l, o.length + s.length));
      else {
        let h = l.length >> 1;
        i.push(new P(l.slice(0, h)), new P(l.slice(h)));
      }
    } else
      i.push(s);
  }
  replace(t, e, i) {
    if (!(i instanceof P))
      return super.replace(t, e, i);
    [t, e] = At(this, t, e);
    let n = ae(this.text, ae(i.text, bi(this.text, 0, t)), e), s = this.length + i.length - (e - t);
    return n.length <= 32 ? new P(n, s) : _.from(P.split(n, []), s);
  }
  sliceString(t, e = this.length, i = `
`) {
    [t, e] = At(this, t, e);
    let n = "";
    for (let s = 0, o = 0; s <= e && o < this.text.length; o++) {
      let l = this.text[o], h = s + l.length;
      s > t && o && (n += i), t < h && e > s && (n += l.slice(Math.max(0, t - s), e - s)), s = h + 1;
    }
    return n;
  }
  flatten(t) {
    for (let e of this.text)
      t.push(e);
  }
  scanIdentical() {
    return 0;
  }
  static split(t, e) {
    let i = [], n = -1;
    for (let s of t)
      i.push(s), n += s.length + 1, i.length == 32 && (e.push(new P(i, n)), i = [], n = -1);
    return n > -1 && e.push(new P(i, n)), e;
  }
}
class _ extends A {
  constructor(t, e) {
    super(), this.children = t, this.length = e, this.lines = 0;
    for (let i of t)
      this.lines += i.lines;
  }
  lineInner(t, e, i, n) {
    for (let s = 0; ; s++) {
      let o = this.children[s], l = n + o.length, h = i + o.lines - 1;
      if ((e ? h : l) >= t)
        return o.lineInner(t, e, i, n);
      n = l + 1, i = h + 1;
    }
  }
  decompose(t, e, i, n) {
    for (let s = 0, o = 0; o <= e && s < this.children.length; s++) {
      let l = this.children[s], h = o + l.length;
      if (t <= h && e >= o) {
        let f = n & ((o <= t ? 1 : 0) | (h >= e ? 2 : 0));
        o >= t && h <= e && !f ? i.push(l) : l.decompose(t - o, e - o, i, f);
      }
      o = h + 1;
    }
  }
  replace(t, e, i) {
    if ([t, e] = At(this, t, e), i.lines < this.lines)
      for (let n = 0, s = 0; n < this.children.length; n++) {
        let o = this.children[n], l = s + o.length;
        if (t >= s && e <= l) {
          let h = o.replace(t - s, e - s, i), f = this.lines - o.lines + h.lines;
          if (h.lines < f >> 4 && h.lines > f >> 6) {
            let a = this.children.slice();
            return a[n] = h, new _(a, this.length - (e - t) + i.length);
          }
          return super.replace(s, l, h);
        }
        s = l + 1;
      }
    return super.replace(t, e, i);
  }
  sliceString(t, e = this.length, i = `
`) {
    [t, e] = At(this, t, e);
    let n = "";
    for (let s = 0, o = 0; s < this.children.length && o <= e; s++) {
      let l = this.children[s], h = o + l.length;
      o > t && s && (n += i), t < h && e > o && (n += l.sliceString(t - o, e - o, i)), o = h + 1;
    }
    return n;
  }
  flatten(t) {
    for (let e of this.children)
      e.flatten(t);
  }
  scanIdentical(t, e) {
    if (!(t instanceof _))
      return 0;
    let i = 0, [n, s, o, l] = e > 0 ? [0, 0, this.children.length, t.children.length] : [this.children.length - 1, t.children.length - 1, -1, -1];
    for (; ; n += e, s += e) {
      if (n == o || s == l)
        return i;
      let h = this.children[n], f = t.children[s];
      if (h != f)
        return i + h.scanIdentical(f, e);
      i += h.length + 1;
    }
  }
  static from(t, e = t.reduce((i, n) => i + n.length + 1, -1)) {
    let i = 0;
    for (let d of t)
      i += d.lines;
    if (i < 32) {
      let d = [];
      for (let g of t)
        g.flatten(d);
      return new P(d, e);
    }
    let n = Math.max(
      32,
      i >> 5
      /* Tree.BranchShift */
    ), s = n << 1, o = n >> 1, l = [], h = 0, f = -1, a = [];
    function c(d) {
      let g;
      if (d.lines > s && d instanceof _)
        for (let p of d.children)
          c(p);
      else d.lines > o && (h > o || !h) ? (u(), l.push(d)) : d instanceof P && h && (g = a[a.length - 1]) instanceof P && d.lines + g.lines <= 32 ? (h += d.lines, f += d.length + 1, a[a.length - 1] = new P(g.text.concat(d.text), g.length + 1 + d.length)) : (h + d.lines > n && u(), h += d.lines, f += d.length + 1, a.push(d));
    }
    function u() {
      h != 0 && (l.push(a.length == 1 ? a[0] : _.from(a, f)), f = -1, h = a.length = 0);
    }
    for (let d of t)
      c(d);
    return u(), l.length == 1 ? l[0] : new _(l, e);
  }
}
A.empty = /* @__PURE__ */ new P([""], 0);
function Rs(r) {
  let t = -1;
  for (let e of r)
    t += e.length + 1;
  return t;
}
function ae(r, t, e = 0, i = 1e9) {
  for (let n = 0, s = 0, o = !0; s < r.length && n <= i; s++) {
    let l = r[s], h = n + l.length;
    h >= e && (h > i && (l = l.slice(0, i - n)), n < e && (l = l.slice(e - n)), o ? (t[t.length - 1] += l, o = !1) : t.push(l)), n = h + 1;
  }
  return t;
}
function bi(r, t, e) {
  return ae(r, [""], t, e);
}
class Wt {
  constructor(t, e = 1) {
    this.dir = e, this.done = !1, this.lineBreak = !1, this.value = "", this.nodes = [t], this.offsets = [e > 0 ? 1 : (t instanceof P ? t.text.length : t.children.length) << 1];
  }
  nextInner(t, e) {
    for (this.done = this.lineBreak = !1; ; ) {
      let i = this.nodes.length - 1, n = this.nodes[i], s = this.offsets[i], o = s >> 1, l = n instanceof P ? n.text.length : n.children.length;
      if (o == (e > 0 ? l : 0)) {
        if (i == 0)
          return this.done = !0, this.value = "", this;
        e > 0 && this.offsets[i - 1]++, this.nodes.pop(), this.offsets.pop();
      } else if ((s & 1) == (e > 0 ? 0 : 1)) {
        if (this.offsets[i] += e, t == 0)
          return this.lineBreak = !0, this.value = `
`, this;
        t--;
      } else if (n instanceof P) {
        let h = n.text[o + (e < 0 ? -1 : 0)];
        if (this.offsets[i] += e, h.length > Math.max(0, t))
          return this.value = t == 0 ? h : e > 0 ? h.slice(t) : h.slice(0, h.length - t), this;
        t -= h.length;
      } else {
        let h = n.children[o + (e < 0 ? -1 : 0)];
        t > h.length ? (t -= h.length, this.offsets[i] += e) : (e < 0 && this.offsets[i]--, this.nodes.push(h), this.offsets.push(e > 0 ? 1 : (h instanceof P ? h.text.length : h.children.length) << 1));
      }
    }
  }
  next(t = 0) {
    return t < 0 && (this.nextInner(-t, -this.dir), t = this.value.length), this.nextInner(t, this.dir);
  }
}
class yn {
  constructor(t, e, i) {
    this.value = "", this.done = !1, this.cursor = new Wt(t, e > i ? -1 : 1), this.pos = e > i ? t.length : 0, this.from = Math.min(e, i), this.to = Math.max(e, i);
  }
  nextInner(t, e) {
    if (e < 0 ? this.pos <= this.from : this.pos >= this.to)
      return this.value = "", this.done = !0, this;
    t += Math.max(0, e < 0 ? this.pos - this.to : this.from - this.pos);
    let i = e < 0 ? this.pos - this.from : this.to - this.pos;
    t > i && (t = i), i -= t;
    let { value: n } = this.cursor.next(t);
    return this.pos += (n.length + t) * e, this.value = n.length <= i ? n : e < 0 ? n.slice(n.length - i) : n.slice(0, i), this.done = !this.value, this;
  }
  next(t = 0) {
    return t < 0 ? t = Math.max(t, this.from - this.pos) : t > 0 && (t = Math.min(t, this.to - this.pos)), this.nextInner(t, this.cursor.dir);
  }
  get lineBreak() {
    return this.cursor.lineBreak && this.value != "";
  }
}
class wn {
  constructor(t) {
    this.inner = t, this.afterBreak = !0, this.value = "", this.done = !1;
  }
  next(t = 0) {
    let { done: e, lineBreak: i, value: n } = this.inner.next(t);
    return e && this.afterBreak ? (this.value = "", this.afterBreak = !1) : e ? (this.done = !0, this.value = "") : i ? this.afterBreak ? this.value = "" : (this.afterBreak = !0, this.next()) : (this.value = n, this.afterBreak = !1), this;
  }
  get lineBreak() {
    return !1;
  }
}
typeof Symbol < "u" && (A.prototype[Symbol.iterator] = function() {
  return this.iter();
}, Wt.prototype[Symbol.iterator] = yn.prototype[Symbol.iterator] = wn.prototype[Symbol.iterator] = function() {
  return this;
});
class Es {
  /**
  @internal
  */
  constructor(t, e, i, n) {
    this.from = t, this.to = e, this.number = i, this.text = n;
  }
  /**
  The length of the line (not including any line break after it).
  */
  get length() {
    return this.to - this.from;
  }
}
function At(r, t, e) {
  return t = Math.max(0, Math.min(r.length, t)), [t, Math.max(t, Math.min(r.length, e))];
}
let St = /* @__PURE__ */ "lc,34,7n,7,7b,19,,,,2,,2,,,20,b,1c,l,g,,2t,7,2,6,2,2,,4,z,,u,r,2j,b,1m,9,9,,o,4,,9,,3,,5,17,3,3b,f,,w,1j,,,,4,8,4,,3,7,a,2,t,,1m,,,,2,4,8,,9,,a,2,q,,2,2,1l,,4,2,4,2,2,3,3,,u,2,3,,b,2,1l,,4,5,,2,4,,k,2,m,6,,,1m,,,2,,4,8,,7,3,a,2,u,,1n,,,,c,,9,,14,,3,,1l,3,5,3,,4,7,2,b,2,t,,1m,,2,,2,,3,,5,2,7,2,b,2,s,2,1l,2,,,2,4,8,,9,,a,2,t,,20,,4,,2,3,,,8,,29,,2,7,c,8,2q,,2,9,b,6,22,2,r,,,,,,1j,e,,5,,2,5,b,,10,9,,2u,4,,6,,2,2,2,p,2,4,3,g,4,d,,2,2,6,,f,,jj,3,qa,3,t,3,t,2,u,2,1s,2,,7,8,,2,b,9,,19,3,3b,2,y,,3a,3,4,2,9,,6,3,63,2,2,,1m,,,7,,,,,2,8,6,a,2,,1c,h,1r,4,1c,7,,,5,,14,9,c,2,w,4,2,2,,3,1k,,,2,3,,,3,1m,8,2,2,48,3,,d,,7,4,,6,,3,2,5i,1m,,5,ek,,5f,x,2da,3,3x,,2o,w,fe,6,2x,2,n9w,4,,a,w,2,28,2,7k,,3,,4,,p,2,5,,47,2,q,i,d,,12,8,p,b,1a,3,1c,,2,4,2,2,13,,1v,6,2,2,2,2,c,,8,,1b,,1f,,,3,2,2,5,2,,,16,2,8,,6m,,2,,4,,fn4,,kh,g,g,g,a6,2,gt,,6a,,45,5,1ae,3,,2,5,4,14,3,4,,4l,2,fx,4,ar,2,49,b,4w,,1i,f,1k,3,1d,4,2,2,1x,3,10,5,,8,1q,,c,2,1g,9,a,4,2,,2n,3,2,,,2,6,,4g,,3,8,l,2,1l,2,,,,,m,,e,7,3,5,5f,8,2,3,,,n,,29,,2,6,,,2,,,2,,2,6j,,2,4,6,2,,2,r,2,2d,8,2,,,2,2y,,,,2,6,,,2t,3,2,4,,5,77,9,,2,6t,,a,2,,,4,,40,4,2,2,4,,w,a,14,6,2,4,8,,9,6,2,3,1a,d,,2,ba,7,,6,,,2a,m,2,7,,2,,2,3e,6,3,,,2,,7,,,20,2,3,,,,9n,2,f0b,5,1n,7,t4,,1r,4,29,,f5k,2,43q,,,3,4,5,8,8,2,7,u,4,44,3,1iz,1j,4,1e,8,,e,,m,5,,f,11s,7,,h,2,7,,2,,5,79,7,c5,4,15s,7,31,7,240,5,gx7k,2o,3k,6o".split(",").map((r) => r ? parseInt(r, 36) : 1);
for (let r = 1; r < St.length; r++)
  St[r] += St[r - 1];
function Ps(r) {
  for (let t = 1; t < St.length; t += 2)
    if (St[t] > r)
      return St[t - 1] <= r;
  return !1;
}
function Si(r) {
  return r >= 127462 && r <= 127487;
}
const xi = 8205;
function it(r, t, e = !0, i = !0) {
  return (e ? bn : Bs)(r, t, i);
}
function bn(r, t, e) {
  if (t == r.length)
    return t;
  t && Sn(r.charCodeAt(t)) && xn(r.charCodeAt(t - 1)) && t--;
  let i = Ae(r, t);
  for (t += vi(i); t < r.length; ) {
    let n = Ae(r, t);
    if (i == xi || n == xi || e && Ps(n))
      t += vi(n), i = n;
    else if (Si(n)) {
      let s = 0, o = t - 2;
      for (; o >= 0 && Si(Ae(r, o)); )
        s++, o -= 2;
      if (s % 2 == 0)
        break;
      t += 2;
    } else
      break;
  }
  return t;
}
function Bs(r, t, e) {
  for (; t > 0; ) {
    let i = bn(r, t - 2, e);
    if (i < t)
      return i;
    t--;
  }
  return 0;
}
function Sn(r) {
  return r >= 56320 && r < 57344;
}
function xn(r) {
  return r >= 55296 && r < 56320;
}
function Ae(r, t) {
  let e = r.charCodeAt(t);
  if (!xn(e) || t + 1 == r.length)
    return e;
  let i = r.charCodeAt(t + 1);
  return Sn(i) ? (e - 55296 << 10) + (i - 56320) + 65536 : e;
}
function vi(r) {
  return r < 65536 ? 1 : 2;
}
const Be = /\r\n?|\n/;
var j = /* @__PURE__ */ function(r) {
  return r[r.Simple = 0] = "Simple", r[r.TrackDel = 1] = "TrackDel", r[r.TrackBefore = 2] = "TrackBefore", r[r.TrackAfter = 3] = "TrackAfter", r;
}(j || (j = {}));
class rt {
  // Sections are encoded as pairs of integers. The first is the
  // length in the current document, and the second is -1 for
  // unaffected sections, and the length of the replacement content
  // otherwise. So an insertion would be (0, n>0), a deletion (n>0,
  // 0), and a replacement two positive numbers.
  /**
  @internal
  */
  constructor(t) {
    this.sections = t;
  }
  /**
  The length of the document before the change.
  */
  get length() {
    let t = 0;
    for (let e = 0; e < this.sections.length; e += 2)
      t += this.sections[e];
    return t;
  }
  /**
  The length of the document after the change.
  */
  get newLength() {
    let t = 0;
    for (let e = 0; e < this.sections.length; e += 2) {
      let i = this.sections[e + 1];
      t += i < 0 ? this.sections[e] : i;
    }
    return t;
  }
  /**
  False when there are actual changes in this set.
  */
  get empty() {
    return this.sections.length == 0 || this.sections.length == 2 && this.sections[1] < 0;
  }
  /**
  Iterate over the unchanged parts left by these changes. `posA`
  provides the position of the range in the old document, `posB`
  the new position in the changed document.
  */
  iterGaps(t) {
    for (let e = 0, i = 0, n = 0; e < this.sections.length; ) {
      let s = this.sections[e++], o = this.sections[e++];
      o < 0 ? (t(i, n, s), n += s) : n += o, i += s;
    }
  }
  /**
  Iterate over the ranges changed by these changes. (See
  [`ChangeSet.iterChanges`](https://codemirror.net/6/docs/ref/#state.ChangeSet.iterChanges) for a
  variant that also provides you with the inserted text.)
  `fromA`/`toA` provides the extent of the change in the starting
  document, `fromB`/`toB` the extent of the replacement in the
  changed document.
  
  When `individual` is true, adjacent changes (which are kept
  separate for [position mapping](https://codemirror.net/6/docs/ref/#state.ChangeDesc.mapPos)) are
  reported separately.
  */
  iterChangedRanges(t, e = !1) {
    Le(this, t, e);
  }
  /**
  Get a description of the inverted form of these changes.
  */
  get invertedDesc() {
    let t = [];
    for (let e = 0; e < this.sections.length; ) {
      let i = this.sections[e++], n = this.sections[e++];
      n < 0 ? t.push(i, n) : t.push(n, i);
    }
    return new rt(t);
  }
  /**
  Compute the combined effect of applying another set of changes
  after this one. The length of the document after this set should
  match the length before `other`.
  */
  composeDesc(t) {
    return this.empty ? t : t.empty ? this : vn(this, t);
  }
  /**
  Map this description, which should start with the same document
  as `other`, over another set of changes, so that it can be
  applied after it. When `before` is true, map as if the changes
  in `other` happened before the ones in `this`.
  */
  mapDesc(t, e = !1) {
    return t.empty ? this : Fe(this, t, e);
  }
  mapPos(t, e = -1, i = j.Simple) {
    let n = 0, s = 0;
    for (let o = 0; o < this.sections.length; ) {
      let l = this.sections[o++], h = this.sections[o++], f = n + l;
      if (h < 0) {
        if (f > t)
          return s + (t - n);
        s += l;
      } else {
        if (i != j.Simple && f >= t && (i == j.TrackDel && n < t && f > t || i == j.TrackBefore && n < t || i == j.TrackAfter && f > t))
          return null;
        if (f > t || f == t && e < 0 && !l)
          return t == n || e < 0 ? s : s + h;
        s += h;
      }
      n = f;
    }
    if (t > n)
      throw new RangeError(`Position ${t} is out of range for changeset of length ${n}`);
    return s;
  }
  /**
  Check whether these changes touch a given range. When one of the
  changes entirely covers the range, the string `"cover"` is
  returned.
  */
  touchesRange(t, e = t) {
    for (let i = 0, n = 0; i < this.sections.length && n <= e; ) {
      let s = this.sections[i++], o = this.sections[i++], l = n + s;
      if (o >= 0 && n <= e && l >= t)
        return n < t && l > e ? "cover" : !0;
      n = l;
    }
    return !1;
  }
  /**
  @internal
  */
  toString() {
    let t = "";
    for (let e = 0; e < this.sections.length; ) {
      let i = this.sections[e++], n = this.sections[e++];
      t += (t ? " " : "") + i + (n >= 0 ? ":" + n : "");
    }
    return t;
  }
  /**
  Serialize this change desc to a JSON-representable value.
  */
  toJSON() {
    return this.sections;
  }
  /**
  Create a change desc from its JSON representation (as produced
  by [`toJSON`](https://codemirror.net/6/docs/ref/#state.ChangeDesc.toJSON).
  */
  static fromJSON(t) {
    if (!Array.isArray(t) || t.length % 2 || t.some((e) => typeof e != "number"))
      throw new RangeError("Invalid JSON representation of ChangeDesc");
    return new rt(t);
  }
  /**
  @internal
  */
  static create(t) {
    return new rt(t);
  }
}
class L extends rt {
  constructor(t, e) {
    super(t), this.inserted = e;
  }
  /**
  Apply the changes to a document, returning the modified
  document.
  */
  apply(t) {
    if (this.length != t.length)
      throw new RangeError("Applying change set to a document with the wrong length");
    return Le(this, (e, i, n, s, o) => t = t.replace(n, n + (i - e), o), !1), t;
  }
  mapDesc(t, e = !1) {
    return Fe(this, t, e, !0);
  }
  /**
  Given the document as it existed _before_ the changes, return a
  change set that represents the inverse of this set, which could
  be used to go from the document created by the changes back to
  the document as it existed before the changes.
  */
  invert(t) {
    let e = this.sections.slice(), i = [];
    for (let n = 0, s = 0; n < e.length; n += 2) {
      let o = e[n], l = e[n + 1];
      if (l >= 0) {
        e[n] = l, e[n + 1] = o;
        let h = n >> 1;
        for (; i.length < h; )
          i.push(A.empty);
        i.push(o ? t.slice(s, s + o) : A.empty);
      }
      s += o;
    }
    return new L(e, i);
  }
  /**
  Combine two subsequent change sets into a single set. `other`
  must start in the document produced by `this`. If `this` goes
  `docA` → `docB` and `other` represents `docB` → `docC`, the
  returned value will represent the change `docA` → `docC`.
  */
  compose(t) {
    return this.empty ? t : t.empty ? this : vn(this, t, !0);
  }
  /**
  Given another change set starting in the same document, maps this
  change set over the other, producing a new change set that can be
  applied to the document produced by applying `other`. When
  `before` is `true`, order changes as if `this` comes before
  `other`, otherwise (the default) treat `other` as coming first.
  
  Given two changes `A` and `B`, `A.compose(B.map(A))` and
  `B.compose(A.map(B, true))` will produce the same document. This
  provides a basic form of [operational
  transformation](https://en.wikipedia.org/wiki/Operational_transformation),
  and can be used for collaborative editing.
  */
  map(t, e = !1) {
    return t.empty ? this : Fe(this, t, e, !0);
  }
  /**
  Iterate over the changed ranges in the document, calling `f` for
  each, with the range in the original document (`fromA`-`toA`)
  and the range that replaces it in the new document
  (`fromB`-`toB`).
  
  When `individual` is true, adjacent changes are reported
  separately.
  */
  iterChanges(t, e = !1) {
    Le(this, t, e);
  }
  /**
  Get a [change description](https://codemirror.net/6/docs/ref/#state.ChangeDesc) for this change
  set.
  */
  get desc() {
    return rt.create(this.sections);
  }
  /**
  @internal
  */
  filter(t) {
    let e = [], i = [], n = [], s = new $t(this);
    t: for (let o = 0, l = 0; ; ) {
      let h = o == t.length ? 1e9 : t[o++];
      for (; l < h || l == h && s.len == 0; ) {
        if (s.done)
          break t;
        let a = Math.min(s.len, h - l);
        H(n, a, -1);
        let c = s.ins == -1 ? -1 : s.off == 0 ? s.ins : 0;
        H(e, a, c), c > 0 && ft(i, e, s.text), s.forward(a), l += a;
      }
      let f = t[o++];
      for (; l < f; ) {
        if (s.done)
          break t;
        let a = Math.min(s.len, f - l);
        H(e, a, -1), H(n, a, s.ins == -1 ? -1 : s.off == 0 ? s.ins : 0), s.forward(a), l += a;
      }
    }
    return {
      changes: new L(e, i),
      filtered: rt.create(n)
    };
  }
  /**
  Serialize this change set to a JSON-representable value.
  */
  toJSON() {
    let t = [];
    for (let e = 0; e < this.sections.length; e += 2) {
      let i = this.sections[e], n = this.sections[e + 1];
      n < 0 ? t.push(i) : n == 0 ? t.push([i]) : t.push([i].concat(this.inserted[e >> 1].toJSON()));
    }
    return t;
  }
  /**
  Create a change set for the given changes, for a document of the
  given length, using `lineSep` as line separator.
  */
  static of(t, e, i) {
    let n = [], s = [], o = 0, l = null;
    function h(a = !1) {
      if (!a && !n.length)
        return;
      o < e && H(n, e - o, -1);
      let c = new L(n, s);
      l = l ? l.compose(c.map(l)) : c, n = [], s = [], o = 0;
    }
    function f(a) {
      if (Array.isArray(a))
        for (let c of a)
          f(c);
      else if (a instanceof L) {
        if (a.length != e)
          throw new RangeError(`Mismatched change set length (got ${a.length}, expected ${e})`);
        h(), l = l ? l.compose(a.map(l)) : a;
      } else {
        let { from: c, to: u = c, insert: d } = a;
        if (c > u || c < 0 || u > e)
          throw new RangeError(`Invalid change range ${c} to ${u} (in doc of length ${e})`);
        let g = d ? typeof d == "string" ? A.of(d.split(i || Be)) : d : A.empty, p = g.length;
        if (c == u && p == 0)
          return;
        c < o && h(), c > o && H(n, c - o, -1), H(n, u - c, p), ft(s, n, g), o = u;
      }
    }
    return f(t), h(!l), l;
  }
  /**
  Create an empty changeset of the given length.
  */
  static empty(t) {
    return new L(t ? [t, -1] : [], []);
  }
  /**
  Create a changeset from its JSON representation (as produced by
  [`toJSON`](https://codemirror.net/6/docs/ref/#state.ChangeSet.toJSON).
  */
  static fromJSON(t) {
    if (!Array.isArray(t))
      throw new RangeError("Invalid JSON representation of ChangeSet");
    let e = [], i = [];
    for (let n = 0; n < t.length; n++) {
      let s = t[n];
      if (typeof s == "number")
        e.push(s, -1);
      else {
        if (!Array.isArray(s) || typeof s[0] != "number" || s.some((o, l) => l && typeof o != "string"))
          throw new RangeError("Invalid JSON representation of ChangeSet");
        if (s.length == 1)
          e.push(s[0], 0);
        else {
          for (; i.length < n; )
            i.push(A.empty);
          i[n] = A.of(s.slice(1)), e.push(s[0], i[n].length);
        }
      }
    }
    return new L(e, i);
  }
  /**
  @internal
  */
  static createSet(t, e) {
    return new L(t, e);
  }
}
function H(r, t, e, i = !1) {
  if (t == 0 && e <= 0)
    return;
  let n = r.length - 2;
  n >= 0 && e <= 0 && e == r[n + 1] ? r[n] += t : t == 0 && r[n] == 0 ? r[n + 1] += e : i ? (r[n] += t, r[n + 1] += e) : r.push(t, e);
}
function ft(r, t, e) {
  if (e.length == 0)
    return;
  let i = t.length - 2 >> 1;
  if (i < r.length)
    r[r.length - 1] = r[r.length - 1].append(e);
  else {
    for (; r.length < i; )
      r.push(A.empty);
    r.push(e);
  }
}
function Le(r, t, e) {
  let i = r.inserted;
  for (let n = 0, s = 0, o = 0; o < r.sections.length; ) {
    let l = r.sections[o++], h = r.sections[o++];
    if (h < 0)
      n += l, s += l;
    else {
      let f = n, a = s, c = A.empty;
      for (; f += l, a += h, h && i && (c = c.append(i[o - 2 >> 1])), !(e || o == r.sections.length || r.sections[o + 1] < 0); )
        l = r.sections[o++], h = r.sections[o++];
      t(n, f, s, a, c), n = f, s = a;
    }
  }
}
function Fe(r, t, e, i = !1) {
  let n = [], s = i ? [] : null, o = new $t(r), l = new $t(t);
  for (let h = -1; ; )
    if (o.ins == -1 && l.ins == -1) {
      let f = Math.min(o.len, l.len);
      H(n, f, -1), o.forward(f), l.forward(f);
    } else if (l.ins >= 0 && (o.ins < 0 || h == o.i || o.off == 0 && (l.len < o.len || l.len == o.len && !e))) {
      let f = l.len;
      for (H(n, l.ins, -1); f; ) {
        let a = Math.min(o.len, f);
        o.ins >= 0 && h < o.i && o.len <= a && (H(n, 0, o.ins), s && ft(s, n, o.text), h = o.i), o.forward(a), f -= a;
      }
      l.next();
    } else if (o.ins >= 0) {
      let f = 0, a = o.len;
      for (; a; )
        if (l.ins == -1) {
          let c = Math.min(a, l.len);
          f += c, a -= c, l.forward(c);
        } else if (l.ins == 0 && l.len < a)
          a -= l.len, l.next();
        else
          break;
      H(n, f, h < o.i ? o.ins : 0), s && h < o.i && ft(s, n, o.text), h = o.i, o.forward(o.len - a);
    } else {
      if (o.done && l.done)
        return s ? L.createSet(n, s) : rt.create(n);
      throw new Error("Mismatched change set lengths");
    }
}
function vn(r, t, e = !1) {
  let i = [], n = e ? [] : null, s = new $t(r), o = new $t(t);
  for (let l = !1; ; ) {
    if (s.done && o.done)
      return n ? L.createSet(i, n) : rt.create(i);
    if (s.ins == 0)
      H(i, s.len, 0, l), s.next();
    else if (o.len == 0 && !o.done)
      H(i, 0, o.ins, l), n && ft(n, i, o.text), o.next();
    else {
      if (s.done || o.done)
        throw new Error("Mismatched change set lengths");
      {
        let h = Math.min(s.len2, o.len), f = i.length;
        if (s.ins == -1) {
          let a = o.ins == -1 ? -1 : o.off ? 0 : o.ins;
          H(i, h, a, l), n && a && ft(n, i, o.text);
        } else o.ins == -1 ? (H(i, s.off ? 0 : s.len, h, l), n && ft(n, i, s.textBit(h))) : (H(i, s.off ? 0 : s.len, o.off ? 0 : o.ins, l), n && !o.off && ft(n, i, o.text));
        l = (s.ins > h || o.ins >= 0 && o.len > h) && (l || i.length > f), s.forward2(h), o.forward(h);
      }
    }
  }
}
class $t {
  constructor(t) {
    this.set = t, this.i = 0, this.next();
  }
  next() {
    let { sections: t } = this.set;
    this.i < t.length ? (this.len = t[this.i++], this.ins = t[this.i++]) : (this.len = 0, this.ins = -2), this.off = 0;
  }
  get done() {
    return this.ins == -2;
  }
  get len2() {
    return this.ins < 0 ? this.len : this.ins;
  }
  get text() {
    let { inserted: t } = this.set, e = this.i - 2 >> 1;
    return e >= t.length ? A.empty : t[e];
  }
  textBit(t) {
    let { inserted: e } = this.set, i = this.i - 2 >> 1;
    return i >= e.length && !t ? A.empty : e[i].slice(this.off, t == null ? void 0 : this.off + t);
  }
  forward(t) {
    t == this.len ? this.next() : (this.len -= t, this.off += t);
  }
  forward2(t) {
    this.ins == -1 ? this.forward(t) : t == this.ins ? this.next() : (this.ins -= t, this.off += t);
  }
}
class gt {
  constructor(t, e, i) {
    this.from = t, this.to = e, this.flags = i;
  }
  /**
  The anchor of the range—the side that doesn't move when you
  extend it.
  */
  get anchor() {
    return this.flags & 32 ? this.to : this.from;
  }
  /**
  The head of the range, which is moved when the range is
  [extended](https://codemirror.net/6/docs/ref/#state.SelectionRange.extend).
  */
  get head() {
    return this.flags & 32 ? this.from : this.to;
  }
  /**
  True when `anchor` and `head` are at the same position.
  */
  get empty() {
    return this.from == this.to;
  }
  /**
  If this is a cursor that is explicitly associated with the
  character on one of its sides, this returns the side. -1 means
  the character before its position, 1 the character after, and 0
  means no association.
  */
  get assoc() {
    return this.flags & 8 ? -1 : this.flags & 16 ? 1 : 0;
  }
  /**
  The bidirectional text level associated with this cursor, if
  any.
  */
  get bidiLevel() {
    let t = this.flags & 7;
    return t == 7 ? null : t;
  }
  /**
  The goal column (stored vertical offset) associated with a
  cursor. This is used to preserve the vertical position when
  [moving](https://codemirror.net/6/docs/ref/#view.EditorView.moveVertically) across
  lines of different length.
  */
  get goalColumn() {
    let t = this.flags >> 6;
    return t == 16777215 ? void 0 : t;
  }
  /**
  Map this range through a change, producing a valid range in the
  updated document.
  */
  map(t, e = -1) {
    let i, n;
    return this.empty ? i = n = t.mapPos(this.from, e) : (i = t.mapPos(this.from, 1), n = t.mapPos(this.to, -1)), i == this.from && n == this.to ? this : new gt(i, n, this.flags);
  }
  /**
  Extend this range to cover at least `from` to `to`.
  */
  extend(t, e = t) {
    if (t <= this.anchor && e >= this.anchor)
      return b.range(t, e);
    let i = Math.abs(t - this.anchor) > Math.abs(e - this.anchor) ? t : e;
    return b.range(this.anchor, i);
  }
  /**
  Compare this range to another range.
  */
  eq(t, e = !1) {
    return this.anchor == t.anchor && this.head == t.head && (!e || !this.empty || this.assoc == t.assoc);
  }
  /**
  Return a JSON-serializable object representing the range.
  */
  toJSON() {
    return { anchor: this.anchor, head: this.head };
  }
  /**
  Convert a JSON representation of a range to a `SelectionRange`
  instance.
  */
  static fromJSON(t) {
    if (!t || typeof t.anchor != "number" || typeof t.head != "number")
      throw new RangeError("Invalid JSON representation for SelectionRange");
    return b.range(t.anchor, t.head);
  }
  /**
  @internal
  */
  static create(t, e, i) {
    return new gt(t, e, i);
  }
}
class b {
  constructor(t, e) {
    this.ranges = t, this.mainIndex = e;
  }
  /**
  Map a selection through a change. Used to adjust the selection
  position for changes.
  */
  map(t, e = -1) {
    return t.empty ? this : b.create(this.ranges.map((i) => i.map(t, e)), this.mainIndex);
  }
  /**
  Compare this selection to another selection. By default, ranges
  are compared only by position. When `includeAssoc` is true,
  cursor ranges must also have the same
  [`assoc`](https://codemirror.net/6/docs/ref/#state.SelectionRange.assoc) value.
  */
  eq(t, e = !1) {
    if (this.ranges.length != t.ranges.length || this.mainIndex != t.mainIndex)
      return !1;
    for (let i = 0; i < this.ranges.length; i++)
      if (!this.ranges[i].eq(t.ranges[i], e))
        return !1;
    return !0;
  }
  /**
  Get the primary selection range. Usually, you should make sure
  your code applies to _all_ ranges, by using methods like
  [`changeByRange`](https://codemirror.net/6/docs/ref/#state.EditorState.changeByRange).
  */
  get main() {
    return this.ranges[this.mainIndex];
  }
  /**
  Make sure the selection only has one range. Returns a selection
  holding only the main range from this selection.
  */
  asSingle() {
    return this.ranges.length == 1 ? this : new b([this.main], 0);
  }
  /**
  Extend this selection with an extra range.
  */
  addRange(t, e = !0) {
    return b.create([t].concat(this.ranges), e ? 0 : this.mainIndex + 1);
  }
  /**
  Replace a given range with another range, and then normalize the
  selection to merge and sort ranges if necessary.
  */
  replaceRange(t, e = this.mainIndex) {
    let i = this.ranges.slice();
    return i[e] = t, b.create(i, this.mainIndex);
  }
  /**
  Convert this selection to an object that can be serialized to
  JSON.
  */
  toJSON() {
    return { ranges: this.ranges.map((t) => t.toJSON()), main: this.mainIndex };
  }
  /**
  Create a selection from a JSON representation.
  */
  static fromJSON(t) {
    if (!t || !Array.isArray(t.ranges) || typeof t.main != "number" || t.main >= t.ranges.length)
      throw new RangeError("Invalid JSON representation for EditorSelection");
    return new b(t.ranges.map((e) => gt.fromJSON(e)), t.main);
  }
  /**
  Create a selection holding a single range.
  */
  static single(t, e = t) {
    return new b([b.range(t, e)], 0);
  }
  /**
  Sort and merge the given set of ranges, creating a valid
  selection.
  */
  static create(t, e = 0) {
    if (t.length == 0)
      throw new RangeError("A selection needs at least one range");
    for (let i = 0, n = 0; n < t.length; n++) {
      let s = t[n];
      if (s.empty ? s.from <= i : s.from < i)
        return b.normalized(t.slice(), e);
      i = s.to;
    }
    return new b(t, e);
  }
  /**
  Create a cursor selection range at the given position. You can
  safely ignore the optional arguments in most situations.
  */
  static cursor(t, e = 0, i, n) {
    return gt.create(t, t, (e == 0 ? 0 : e < 0 ? 8 : 16) | (i == null ? 7 : Math.min(6, i)) | (n ?? 16777215) << 6);
  }
  /**
  Create a selection range.
  */
  static range(t, e, i, n) {
    let s = (i ?? 16777215) << 6 | (n == null ? 7 : Math.min(6, n));
    return e < t ? gt.create(e, t, 48 | s) : gt.create(t, e, (e > t ? 8 : 0) | s);
  }
  /**
  @internal
  */
  static normalized(t, e = 0) {
    let i = t[e];
    t.sort((n, s) => n.from - s.from), e = t.indexOf(i);
    for (let n = 1; n < t.length; n++) {
      let s = t[n], o = t[n - 1];
      if (s.empty ? s.from <= o.to : s.from < o.to) {
        let l = o.from, h = Math.max(s.to, o.to);
        n <= e && e--, t.splice(--n, 2, s.anchor > s.head ? b.range(h, l) : b.range(l, h));
      }
    }
    return new b(t, e);
  }
}
function kn(r, t) {
  for (let e of r.ranges)
    if (e.to > t)
      throw new RangeError("Selection points outside of document");
}
let hi = 0;
class v {
  constructor(t, e, i, n, s) {
    this.combine = t, this.compareInput = e, this.compare = i, this.isStatic = n, this.id = hi++, this.default = t([]), this.extensions = typeof s == "function" ? s(this) : s;
  }
  /**
  Returns a facet reader for this facet, which can be used to
  [read](https://codemirror.net/6/docs/ref/#state.EditorState.facet) it but not to define values for it.
  */
  get reader() {
    return this;
  }
  /**
  Define a new facet.
  */
  static define(t = {}) {
    return new v(t.combine || ((e) => e), t.compareInput || ((e, i) => e === i), t.compare || (t.combine ? (e, i) => e === i : ai), !!t.static, t.enables);
  }
  /**
  Returns an extension that adds the given value to this facet.
  */
  of(t) {
    return new fe([], this, 0, t);
  }
  /**
  Create an extension that computes a value for the facet from a
  state. You must take care to declare the parts of the state that
  this value depends on, since your function is only called again
  for a new state when one of those parts changed.
  
  In cases where your value depends only on a single field, you'll
  want to use the [`from`](https://codemirror.net/6/docs/ref/#state.Facet.from) method instead.
  */
  compute(t, e) {
    if (this.isStatic)
      throw new Error("Can't compute a static facet");
    return new fe(t, this, 1, e);
  }
  /**
  Create an extension that computes zero or more values for this
  facet from a state.
  */
  computeN(t, e) {
    if (this.isStatic)
      throw new Error("Can't compute a static facet");
    return new fe(t, this, 2, e);
  }
  from(t, e) {
    return e || (e = (i) => i), this.compute([t], (i) => e(i.field(t)));
  }
}
function ai(r, t) {
  return r == t || r.length == t.length && r.every((e, i) => e === t[i]);
}
class fe {
  constructor(t, e, i, n) {
    this.dependencies = t, this.facet = e, this.type = i, this.value = n, this.id = hi++;
  }
  dynamicSlot(t) {
    var e;
    let i = this.value, n = this.facet.compareInput, s = this.id, o = t[s] >> 1, l = this.type == 2, h = !1, f = !1, a = [];
    for (let c of this.dependencies)
      c == "doc" ? h = !0 : c == "selection" ? f = !0 : ((e = t[c.id]) !== null && e !== void 0 ? e : 1) & 1 || a.push(t[c.id]);
    return {
      create(c) {
        return c.values[o] = i(c), 1;
      },
      update(c, u) {
        if (h && u.docChanged || f && (u.docChanged || u.selection) || Ie(c, a)) {
          let d = i(c);
          if (l ? !ki(d, c.values[o], n) : !n(d, c.values[o]))
            return c.values[o] = d, 1;
        }
        return 0;
      },
      reconfigure: (c, u) => {
        let d, g = u.config.address[s];
        if (g != null) {
          let p = pe(u, g);
          if (this.dependencies.every((m) => m instanceof v ? u.facet(m) === c.facet(m) : m instanceof Pt ? u.field(m, !1) == c.field(m, !1) : !0) || (l ? ki(d = i(c), p, n) : n(d = i(c), p)))
            return c.values[o] = p, 0;
        } else
          d = i(c);
        return c.values[o] = d, 1;
      }
    };
  }
}
function ki(r, t, e) {
  if (r.length != t.length)
    return !1;
  for (let i = 0; i < r.length; i++)
    if (!e(r[i], t[i]))
      return !1;
  return !0;
}
function Ie(r, t) {
  let e = !1;
  for (let i of t)
    zt(r, i) & 1 && (e = !0);
  return e;
}
function Ls(r, t, e) {
  let i = e.map((h) => r[h.id]), n = e.map((h) => h.type), s = i.filter((h) => !(h & 1)), o = r[t.id] >> 1;
  function l(h) {
    let f = [];
    for (let a = 0; a < i.length; a++) {
      let c = pe(h, i[a]);
      if (n[a] == 2)
        for (let u of c)
          f.push(u);
      else
        f.push(c);
    }
    return t.combine(f);
  }
  return {
    create(h) {
      for (let f of i)
        zt(h, f);
      return h.values[o] = l(h), 1;
    },
    update(h, f) {
      if (!Ie(h, s))
        return 0;
      let a = l(h);
      return t.compare(a, h.values[o]) ? 0 : (h.values[o] = a, 1);
    },
    reconfigure(h, f) {
      let a = Ie(h, i), c = f.config.facets[t.id], u = f.facet(t);
      if (c && !a && ai(e, c))
        return h.values[o] = u, 0;
      let d = l(h);
      return t.compare(d, u) ? (h.values[o] = u, 0) : (h.values[o] = d, 1);
    }
  };
}
const Ai = /* @__PURE__ */ v.define({ static: !0 });
class Pt {
  constructor(t, e, i, n, s) {
    this.id = t, this.createF = e, this.updateF = i, this.compareF = n, this.spec = s, this.provides = void 0;
  }
  /**
  Define a state field.
  */
  static define(t) {
    let e = new Pt(hi++, t.create, t.update, t.compare || ((i, n) => i === n), t);
    return t.provide && (e.provides = t.provide(e)), e;
  }
  create(t) {
    let e = t.facet(Ai).find((i) => i.field == this);
    return ((e == null ? void 0 : e.create) || this.createF)(t);
  }
  /**
  @internal
  */
  slot(t) {
    let e = t[this.id] >> 1;
    return {
      create: (i) => (i.values[e] = this.create(i), 1),
      update: (i, n) => {
        let s = i.values[e], o = this.updateF(s, n);
        return this.compareF(s, o) ? 0 : (i.values[e] = o, 1);
      },
      reconfigure: (i, n) => n.config.address[this.id] != null ? (i.values[e] = n.field(this), 0) : (i.values[e] = this.create(i), 1)
    };
  }
  /**
  Returns an extension that enables this field and overrides the
  way it is initialized. Can be useful when you need to provide a
  non-default starting value for the field.
  */
  init(t) {
    return [this, Ai.of({ field: this, create: t })];
  }
  /**
  State field instances can be used as
  [`Extension`](https://codemirror.net/6/docs/ref/#state.Extension) values to enable the field in a
  given state.
  */
  get extension() {
    return this;
  }
}
const dt = { lowest: 4, low: 3, default: 2, high: 1, highest: 0 };
function Lt(r) {
  return (t) => new An(t, r);
}
const Fs = {
  /**
  The highest precedence level, for extensions that should end up
  near the start of the precedence ordering.
  */
  highest: /* @__PURE__ */ Lt(dt.highest),
  /**
  A higher-than-default precedence, for extensions that should
  come before those with default precedence.
  */
  high: /* @__PURE__ */ Lt(dt.high),
  /**
  The default precedence, which is also used for extensions
  without an explicit precedence.
  */
  default: /* @__PURE__ */ Lt(dt.default),
  /**
  A lower-than-default precedence.
  */
  low: /* @__PURE__ */ Lt(dt.low),
  /**
  The lowest precedence level. Meant for things that should end up
  near the end of the extension order.
  */
  lowest: /* @__PURE__ */ Lt(dt.lowest)
};
class An {
  constructor(t, e) {
    this.inner = t, this.prec = e;
  }
}
class ve {
  /**
  Create an instance of this compartment to add to your [state
  configuration](https://codemirror.net/6/docs/ref/#state.EditorStateConfig.extensions).
  */
  of(t) {
    return new Ne(this, t);
  }
  /**
  Create an [effect](https://codemirror.net/6/docs/ref/#state.TransactionSpec.effects) that
  reconfigures this compartment.
  */
  reconfigure(t) {
    return ve.reconfigure.of({ compartment: this, extension: t });
  }
  /**
  Get the current content of the compartment in the state, or
  `undefined` if it isn't present.
  */
  get(t) {
    return t.config.compartments.get(this);
  }
}
class Ne {
  constructor(t, e) {
    this.compartment = t, this.inner = e;
  }
}
class ge {
  constructor(t, e, i, n, s, o) {
    for (this.base = t, this.compartments = e, this.dynamicSlots = i, this.address = n, this.staticValues = s, this.facets = o, this.statusTemplate = []; this.statusTemplate.length < i.length; )
      this.statusTemplate.push(
        0
        /* SlotStatus.Unresolved */
      );
  }
  staticFacet(t) {
    let e = this.address[t.id];
    return e == null ? t.default : this.staticValues[e >> 1];
  }
  static resolve(t, e, i) {
    let n = [], s = /* @__PURE__ */ Object.create(null), o = /* @__PURE__ */ new Map();
    for (let u of Is(t, e, o))
      u instanceof Pt ? n.push(u) : (s[u.facet.id] || (s[u.facet.id] = [])).push(u);
    let l = /* @__PURE__ */ Object.create(null), h = [], f = [];
    for (let u of n)
      l[u.id] = f.length << 1, f.push((d) => u.slot(d));
    let a = i == null ? void 0 : i.config.facets;
    for (let u in s) {
      let d = s[u], g = d[0].facet, p = a && a[u] || [];
      if (d.every(
        (m) => m.type == 0
        /* Provider.Static */
      ))
        if (l[g.id] = h.length << 1 | 1, ai(p, d))
          h.push(i.facet(g));
        else {
          let m = g.combine(d.map((y) => y.value));
          h.push(i && g.compare(m, i.facet(g)) ? i.facet(g) : m);
        }
      else {
        for (let m of d)
          m.type == 0 ? (l[m.id] = h.length << 1 | 1, h.push(m.value)) : (l[m.id] = f.length << 1, f.push((y) => m.dynamicSlot(y)));
        l[g.id] = f.length << 1, f.push((m) => Ls(m, g, d));
      }
    }
    let c = f.map((u) => u(l));
    return new ge(t, o, c, l, h, s);
  }
}
function Is(r, t, e) {
  let i = [[], [], [], [], []], n = /* @__PURE__ */ new Map();
  function s(o, l) {
    let h = n.get(o);
    if (h != null) {
      if (h <= l)
        return;
      let f = i[h].indexOf(o);
      f > -1 && i[h].splice(f, 1), o instanceof Ne && e.delete(o.compartment);
    }
    if (n.set(o, l), Array.isArray(o))
      for (let f of o)
        s(f, l);
    else if (o instanceof Ne) {
      if (e.has(o.compartment))
        throw new RangeError("Duplicate use of compartment in extensions");
      let f = t.get(o.compartment) || o.inner;
      e.set(o.compartment, f), s(f, l);
    } else if (o instanceof An)
      s(o.inner, o.prec);
    else if (o instanceof Pt)
      i[l].push(o), o.provides && s(o.provides, l);
    else if (o instanceof fe)
      i[l].push(o), o.facet.extensions && s(o.facet.extensions, dt.default);
    else {
      let f = o.extension;
      if (!f)
        throw new Error(`Unrecognized extension value in extension set (${o}). This sometimes happens because multiple instances of @codemirror/state are loaded, breaking instanceof checks.`);
      s(f, l);
    }
  }
  return s(r, dt.default), i.reduce((o, l) => o.concat(l));
}
function zt(r, t) {
  if (t & 1)
    return 2;
  let e = t >> 1, i = r.status[e];
  if (i == 4)
    throw new Error("Cyclic dependency between fields and/or facets");
  if (i & 2)
    return i;
  r.status[e] = 4;
  let n = r.computeSlot(r, r.config.dynamicSlots[e]);
  return r.status[e] = 2 | n;
}
function pe(r, t) {
  return t & 1 ? r.config.staticValues[t >> 1] : r.values[t >> 1];
}
const Cn = /* @__PURE__ */ v.define(), He = /* @__PURE__ */ v.define({
  combine: (r) => r.some((t) => t),
  static: !0
}), Mn = /* @__PURE__ */ v.define({
  combine: (r) => r.length ? r[0] : void 0,
  static: !0
}), Dn = /* @__PURE__ */ v.define(), On = /* @__PURE__ */ v.define(), Tn = /* @__PURE__ */ v.define(), Rn = /* @__PURE__ */ v.define({
  combine: (r) => r.length ? r[0] : !1
});
class Bt {
  /**
  @internal
  */
  constructor(t, e) {
    this.type = t, this.value = e;
  }
  /**
  Define a new type of annotation.
  */
  static define() {
    return new Ns();
  }
}
class Ns {
  /**
  Create an instance of this annotation.
  */
  of(t) {
    return new Bt(this, t);
  }
}
class Hs {
  /**
  @internal
  */
  constructor(t) {
    this.map = t;
  }
  /**
  Create a [state effect](https://codemirror.net/6/docs/ref/#state.StateEffect) instance of this
  type.
  */
  of(t) {
    return new F(this, t);
  }
}
class F {
  /**
  @internal
  */
  constructor(t, e) {
    this.type = t, this.value = e;
  }
  /**
  Map this effect through a position mapping. Will return
  `undefined` when that ends up deleting the effect.
  */
  map(t) {
    let e = this.type.map(this.value, t);
    return e === void 0 ? void 0 : e == this.value ? this : new F(this.type, e);
  }
  /**
  Tells you whether this effect object is of a given
  [type](https://codemirror.net/6/docs/ref/#state.StateEffectType).
  */
  is(t) {
    return this.type == t;
  }
  /**
  Define a new effect type. The type parameter indicates the type
  of values that his effect holds. It should be a type that
  doesn't include `undefined`, since that is used in
  [mapping](https://codemirror.net/6/docs/ref/#state.StateEffect.map) to indicate that an effect is
  removed.
  */
  static define(t = {}) {
    return new Hs(t.map || ((e) => e));
  }
  /**
  Map an array of effects through a change set.
  */
  static mapEffects(t, e) {
    if (!t.length)
      return t;
    let i = [];
    for (let n of t) {
      let s = n.map(e);
      s && i.push(s);
    }
    return i;
  }
}
F.reconfigure = /* @__PURE__ */ F.define();
F.appendConfig = /* @__PURE__ */ F.define();
class W {
  constructor(t, e, i, n, s, o) {
    this.startState = t, this.changes = e, this.selection = i, this.effects = n, this.annotations = s, this.scrollIntoView = o, this._doc = null, this._state = null, i && kn(i, e.newLength), s.some((l) => l.type == W.time) || (this.annotations = s.concat(W.time.of(Date.now())));
  }
  /**
  @internal
  */
  static create(t, e, i, n, s, o) {
    return new W(t, e, i, n, s, o);
  }
  /**
  The new document produced by the transaction. Contrary to
  [`.state`](https://codemirror.net/6/docs/ref/#state.Transaction.state)`.doc`, accessing this won't
  force the entire new state to be computed right away, so it is
  recommended that [transaction
  filters](https://codemirror.net/6/docs/ref/#state.EditorState^transactionFilter) use this getter
  when they need to look at the new document.
  */
  get newDoc() {
    return this._doc || (this._doc = this.changes.apply(this.startState.doc));
  }
  /**
  The new selection produced by the transaction. If
  [`this.selection`](https://codemirror.net/6/docs/ref/#state.Transaction.selection) is undefined,
  this will [map](https://codemirror.net/6/docs/ref/#state.EditorSelection.map) the start state's
  current selection through the changes made by the transaction.
  */
  get newSelection() {
    return this.selection || this.startState.selection.map(this.changes);
  }
  /**
  The new state created by the transaction. Computed on demand
  (but retained for subsequent access), so it is recommended not to
  access it in [transaction
  filters](https://codemirror.net/6/docs/ref/#state.EditorState^transactionFilter) when possible.
  */
  get state() {
    return this._state || this.startState.applyTransaction(this), this._state;
  }
  /**
  Get the value of the given annotation type, if any.
  */
  annotation(t) {
    for (let e of this.annotations)
      if (e.type == t)
        return e.value;
  }
  /**
  Indicates whether the transaction changed the document.
  */
  get docChanged() {
    return !this.changes.empty;
  }
  /**
  Indicates whether this transaction reconfigures the state
  (through a [configuration compartment](https://codemirror.net/6/docs/ref/#state.Compartment) or
  with a top-level configuration
  [effect](https://codemirror.net/6/docs/ref/#state.StateEffect^reconfigure).
  */
  get reconfigured() {
    return this.startState.config != this.state.config;
  }
  /**
  Returns true if the transaction has a [user
  event](https://codemirror.net/6/docs/ref/#state.Transaction^userEvent) annotation that is equal to
  or more specific than `event`. For example, if the transaction
  has `"select.pointer"` as user event, `"select"` and
  `"select.pointer"` will match it.
  */
  isUserEvent(t) {
    let e = this.annotation(W.userEvent);
    return !!(e && (e == t || e.length > t.length && e.slice(0, t.length) == t && e[t.length] == "."));
  }
}
W.time = /* @__PURE__ */ Bt.define();
W.userEvent = /* @__PURE__ */ Bt.define();
W.addToHistory = /* @__PURE__ */ Bt.define();
W.remote = /* @__PURE__ */ Bt.define();
function Vs(r, t) {
  let e = [];
  for (let i = 0, n = 0; ; ) {
    let s, o;
    if (i < r.length && (n == t.length || t[n] >= r[i]))
      s = r[i++], o = r[i++];
    else if (n < t.length)
      s = t[n++], o = t[n++];
    else
      return e;
    !e.length || e[e.length - 1] < s ? e.push(s, o) : e[e.length - 1] < o && (e[e.length - 1] = o);
  }
}
function En(r, t, e) {
  var i;
  let n, s, o;
  return e ? (n = t.changes, s = L.empty(t.changes.length), o = r.changes.compose(t.changes)) : (n = t.changes.map(r.changes), s = r.changes.mapDesc(t.changes, !0), o = r.changes.compose(n)), {
    changes: o,
    selection: t.selection ? t.selection.map(s) : (i = r.selection) === null || i === void 0 ? void 0 : i.map(n),
    effects: F.mapEffects(r.effects, n).concat(F.mapEffects(t.effects, s)),
    annotations: r.annotations.length ? r.annotations.concat(t.annotations) : t.annotations,
    scrollIntoView: r.scrollIntoView || t.scrollIntoView
  };
}
function Ve(r, t, e) {
  let i = t.selection, n = xt(t.annotations);
  return t.userEvent && (n = n.concat(W.userEvent.of(t.userEvent))), {
    changes: t.changes instanceof L ? t.changes : L.of(t.changes || [], e, r.facet(Mn)),
    selection: i && (i instanceof b ? i : b.single(i.anchor, i.head)),
    effects: xt(t.effects),
    annotations: n,
    scrollIntoView: !!t.scrollIntoView
  };
}
function Pn(r, t, e) {
  let i = Ve(r, t.length ? t[0] : {}, r.doc.length);
  t.length && t[0].filter === !1 && (e = !1);
  for (let s = 1; s < t.length; s++) {
    t[s].filter === !1 && (e = !1);
    let o = !!t[s].sequential;
    i = En(i, Ve(r, t[s], o ? i.changes.newLength : r.doc.length), o);
  }
  let n = W.create(r, i.changes, i.selection, i.effects, i.annotations, i.scrollIntoView);
  return zs(e ? Ws(n) : n);
}
function Ws(r) {
  let t = r.startState, e = !0;
  for (let n of t.facet(Dn)) {
    let s = n(r);
    if (s === !1) {
      e = !1;
      break;
    }
    Array.isArray(s) && (e = e === !0 ? s : Vs(e, s));
  }
  if (e !== !0) {
    let n, s;
    if (e === !1)
      s = r.changes.invertedDesc, n = L.empty(t.doc.length);
    else {
      let o = r.changes.filter(e);
      n = o.changes, s = o.filtered.mapDesc(o.changes).invertedDesc;
    }
    r = W.create(t, n, r.selection && r.selection.map(s), F.mapEffects(r.effects, s), r.annotations, r.scrollIntoView);
  }
  let i = t.facet(On);
  for (let n = i.length - 1; n >= 0; n--) {
    let s = i[n](r);
    s instanceof W ? r = s : Array.isArray(s) && s.length == 1 && s[0] instanceof W ? r = s[0] : r = Pn(t, xt(s), !1);
  }
  return r;
}
function zs(r) {
  let t = r.startState, e = t.facet(Tn), i = r;
  for (let n = e.length - 1; n >= 0; n--) {
    let s = e[n](r);
    s && Object.keys(s).length && (i = En(i, Ve(t, s, r.changes.newLength), !0));
  }
  return i == r ? r : W.create(t, r.changes, r.selection, i.effects, i.annotations, i.scrollIntoView);
}
const qs = [];
function xt(r) {
  return r == null ? qs : Array.isArray(r) ? r : [r];
}
var nt = /* @__PURE__ */ function(r) {
  return r[r.Word = 0] = "Word", r[r.Space = 1] = "Space", r[r.Other = 2] = "Other", r;
}(nt || (nt = {}));
const Ks = /[\u00df\u0587\u0590-\u05f4\u0600-\u06ff\u3040-\u309f\u30a0-\u30ff\u3400-\u4db5\u4e00-\u9fcc\uac00-\ud7af]/;
let We;
try {
  We = /* @__PURE__ */ new RegExp("[\\p{Alphabetic}\\p{Number}_]", "u");
} catch {
}
function $s(r) {
  if (We)
    return We.test(r);
  for (let t = 0; t < r.length; t++) {
    let e = r[t];
    if (/\w/.test(e) || e > "" && (e.toUpperCase() != e.toLowerCase() || Ks.test(e)))
      return !0;
  }
  return !1;
}
function js(r) {
  return (t) => {
    if (!/\S/.test(t))
      return nt.Space;
    if ($s(t))
      return nt.Word;
    for (let e = 0; e < r.length; e++)
      if (t.indexOf(r[e]) > -1)
        return nt.Word;
    return nt.Other;
  };
}
class T {
  constructor(t, e, i, n, s, o) {
    this.config = t, this.doc = e, this.selection = i, this.values = n, this.status = t.statusTemplate.slice(), this.computeSlot = s, o && (o._state = this);
    for (let l = 0; l < this.config.dynamicSlots.length; l++)
      zt(this, l << 1);
    this.computeSlot = null;
  }
  field(t, e = !0) {
    let i = this.config.address[t.id];
    if (i == null) {
      if (e)
        throw new RangeError("Field is not present in this state");
      return;
    }
    return zt(this, i), pe(this, i);
  }
  /**
  Create a [transaction](https://codemirror.net/6/docs/ref/#state.Transaction) that updates this
  state. Any number of [transaction specs](https://codemirror.net/6/docs/ref/#state.TransactionSpec)
  can be passed. Unless
  [`sequential`](https://codemirror.net/6/docs/ref/#state.TransactionSpec.sequential) is set, the
  [changes](https://codemirror.net/6/docs/ref/#state.TransactionSpec.changes) (if any) of each spec
  are assumed to start in the _current_ document (not the document
  produced by previous specs), and its
  [selection](https://codemirror.net/6/docs/ref/#state.TransactionSpec.selection) and
  [effects](https://codemirror.net/6/docs/ref/#state.TransactionSpec.effects) are assumed to refer
  to the document created by its _own_ changes. The resulting
  transaction contains the combined effect of all the different
  specs. For [selection](https://codemirror.net/6/docs/ref/#state.TransactionSpec.selection), later
  specs take precedence over earlier ones.
  */
  update(...t) {
    return Pn(this, t, !0);
  }
  /**
  @internal
  */
  applyTransaction(t) {
    let e = this.config, { base: i, compartments: n } = e;
    for (let l of t.effects)
      l.is(ve.reconfigure) ? (e && (n = /* @__PURE__ */ new Map(), e.compartments.forEach((h, f) => n.set(f, h)), e = null), n.set(l.value.compartment, l.value.extension)) : l.is(F.reconfigure) ? (e = null, i = l.value) : l.is(F.appendConfig) && (e = null, i = xt(i).concat(l.value));
    let s;
    e ? s = t.startState.values.slice() : (e = ge.resolve(i, n, this), s = new T(e, this.doc, this.selection, e.dynamicSlots.map(() => null), (h, f) => f.reconfigure(h, this), null).values);
    let o = t.startState.facet(He) ? t.newSelection : t.newSelection.asSingle();
    new T(e, t.newDoc, o, s, (l, h) => h.update(l, t), t);
  }
  /**
  Create a [transaction spec](https://codemirror.net/6/docs/ref/#state.TransactionSpec) that
  replaces every selection range with the given content.
  */
  replaceSelection(t) {
    return typeof t == "string" && (t = this.toText(t)), this.changeByRange((e) => ({
      changes: { from: e.from, to: e.to, insert: t },
      range: b.cursor(e.from + t.length)
    }));
  }
  /**
  Create a set of changes and a new selection by running the given
  function for each range in the active selection. The function
  can return an optional set of changes (in the coordinate space
  of the start document), plus an updated range (in the coordinate
  space of the document produced by the call's own changes). This
  method will merge all the changes and ranges into a single
  changeset and selection, and return it as a [transaction
  spec](https://codemirror.net/6/docs/ref/#state.TransactionSpec), which can be passed to
  [`update`](https://codemirror.net/6/docs/ref/#state.EditorState.update).
  */
  changeByRange(t) {
    let e = this.selection, i = t(e.ranges[0]), n = this.changes(i.changes), s = [i.range], o = xt(i.effects);
    for (let l = 1; l < e.ranges.length; l++) {
      let h = t(e.ranges[l]), f = this.changes(h.changes), a = f.map(n);
      for (let u = 0; u < l; u++)
        s[u] = s[u].map(a);
      let c = n.mapDesc(f, !0);
      s.push(h.range.map(c)), n = n.compose(a), o = F.mapEffects(o, a).concat(F.mapEffects(xt(h.effects), c));
    }
    return {
      changes: n,
      selection: b.create(s, e.mainIndex),
      effects: o
    };
  }
  /**
  Create a [change set](https://codemirror.net/6/docs/ref/#state.ChangeSet) from the given change
  description, taking the state's document length and line
  separator into account.
  */
  changes(t = []) {
    return t instanceof L ? t : L.of(t, this.doc.length, this.facet(T.lineSeparator));
  }
  /**
  Using the state's [line
  separator](https://codemirror.net/6/docs/ref/#state.EditorState^lineSeparator), create a
  [`Text`](https://codemirror.net/6/docs/ref/#state.Text) instance from the given string.
  */
  toText(t) {
    return A.of(t.split(this.facet(T.lineSeparator) || Be));
  }
  /**
  Return the given range of the document as a string.
  */
  sliceDoc(t = 0, e = this.doc.length) {
    return this.doc.sliceString(t, e, this.lineBreak);
  }
  /**
  Get the value of a state [facet](https://codemirror.net/6/docs/ref/#state.Facet).
  */
  facet(t) {
    let e = this.config.address[t.id];
    return e == null ? t.default : (zt(this, e), pe(this, e));
  }
  /**
  Convert this state to a JSON-serializable object. When custom
  fields should be serialized, you can pass them in as an object
  mapping property names (in the resulting object, which should
  not use `doc` or `selection`) to fields.
  */
  toJSON(t) {
    let e = {
      doc: this.sliceDoc(),
      selection: this.selection.toJSON()
    };
    if (t)
      for (let i in t) {
        let n = t[i];
        n instanceof Pt && this.config.address[n.id] != null && (e[i] = n.spec.toJSON(this.field(t[i]), this));
      }
    return e;
  }
  /**
  Deserialize a state from its JSON representation. When custom
  fields should be deserialized, pass the same object you passed
  to [`toJSON`](https://codemirror.net/6/docs/ref/#state.EditorState.toJSON) when serializing as
  third argument.
  */
  static fromJSON(t, e = {}, i) {
    if (!t || typeof t.doc != "string")
      throw new RangeError("Invalid JSON representation for EditorState");
    let n = [];
    if (i) {
      for (let s in i)
        if (Object.prototype.hasOwnProperty.call(t, s)) {
          let o = i[s], l = t[s];
          n.push(o.init((h) => o.spec.fromJSON(l, h)));
        }
    }
    return T.create({
      doc: t.doc,
      selection: b.fromJSON(t.selection),
      extensions: e.extensions ? n.concat([e.extensions]) : n
    });
  }
  /**
  Create a new state. You'll usually only need this when
  initializing an editor—updated states are created by applying
  transactions.
  */
  static create(t = {}) {
    let e = ge.resolve(t.extensions || [], /* @__PURE__ */ new Map()), i = t.doc instanceof A ? t.doc : A.of((t.doc || "").split(e.staticFacet(T.lineSeparator) || Be)), n = t.selection ? t.selection instanceof b ? t.selection : b.single(t.selection.anchor, t.selection.head) : b.single(0);
    return kn(n, i.length), e.staticFacet(He) || (n = n.asSingle()), new T(e, i, n, e.dynamicSlots.map(() => null), (s, o) => o.create(s), null);
  }
  /**
  The size (in columns) of a tab in the document, determined by
  the [`tabSize`](https://codemirror.net/6/docs/ref/#state.EditorState^tabSize) facet.
  */
  get tabSize() {
    return this.facet(T.tabSize);
  }
  /**
  Get the proper [line-break](https://codemirror.net/6/docs/ref/#state.EditorState^lineSeparator)
  string for this state.
  */
  get lineBreak() {
    return this.facet(T.lineSeparator) || `
`;
  }
  /**
  Returns true when the editor is
  [configured](https://codemirror.net/6/docs/ref/#state.EditorState^readOnly) to be read-only.
  */
  get readOnly() {
    return this.facet(Rn);
  }
  /**
  Look up a translation for the given phrase (via the
  [`phrases`](https://codemirror.net/6/docs/ref/#state.EditorState^phrases) facet), or return the
  original string if no translation is found.
  
  If additional arguments are passed, they will be inserted in
  place of markers like `$1` (for the first value) and `$2`, etc.
  A single `$` is equivalent to `$1`, and `$$` will produce a
  literal dollar sign.
  */
  phrase(t, ...e) {
    for (let i of this.facet(T.phrases))
      if (Object.prototype.hasOwnProperty.call(i, t)) {
        t = i[t];
        break;
      }
    return e.length && (t = t.replace(/\$(\$|\d*)/g, (i, n) => {
      if (n == "$")
        return "$";
      let s = +(n || 1);
      return !s || s > e.length ? i : e[s - 1];
    })), t;
  }
  /**
  Find the values for a given language data field, provided by the
  the [`languageData`](https://codemirror.net/6/docs/ref/#state.EditorState^languageData) facet.
  
  Examples of language data fields are...
  
  - [`"commentTokens"`](https://codemirror.net/6/docs/ref/#commands.CommentTokens) for specifying
    comment syntax.
  - [`"autocomplete"`](https://codemirror.net/6/docs/ref/#autocomplete.autocompletion^config.override)
    for providing language-specific completion sources.
  - [`"wordChars"`](https://codemirror.net/6/docs/ref/#state.EditorState.charCategorizer) for adding
    characters that should be considered part of words in this
    language.
  - [`"closeBrackets"`](https://codemirror.net/6/docs/ref/#autocomplete.CloseBracketConfig) controls
    bracket closing behavior.
  */
  languageDataAt(t, e, i = -1) {
    let n = [];
    for (let s of this.facet(Cn))
      for (let o of s(this, e, i))
        Object.prototype.hasOwnProperty.call(o, t) && n.push(o[t]);
    return n;
  }
  /**
  Return a function that can categorize strings (expected to
  represent a single [grapheme cluster](https://codemirror.net/6/docs/ref/#state.findClusterBreak))
  into one of:
  
   - Word (contains an alphanumeric character or a character
     explicitly listed in the local language's `"wordChars"`
     language data, which should be a string)
   - Space (contains only whitespace)
   - Other (anything else)
  */
  charCategorizer(t) {
    return js(this.languageDataAt("wordChars", t).join(""));
  }
  /**
  Find the word at the given position, meaning the range
  containing all [word](https://codemirror.net/6/docs/ref/#state.CharCategory.Word) characters
  around it. If no word characters are adjacent to the position,
  this returns null.
  */
  wordAt(t) {
    let { text: e, from: i, length: n } = this.doc.lineAt(t), s = this.charCategorizer(t), o = t - i, l = t - i;
    for (; o > 0; ) {
      let h = it(e, o, !1);
      if (s(e.slice(h, o)) != nt.Word)
        break;
      o = h;
    }
    for (; l < n; ) {
      let h = it(e, l);
      if (s(e.slice(l, h)) != nt.Word)
        break;
      l = h;
    }
    return o == l ? null : b.range(o + i, l + i);
  }
}
T.allowMultipleSelections = He;
T.tabSize = /* @__PURE__ */ v.define({
  combine: (r) => r.length ? r[0] : 4
});
T.lineSeparator = Mn;
T.readOnly = Rn;
T.phrases = /* @__PURE__ */ v.define({
  compare(r, t) {
    let e = Object.keys(r), i = Object.keys(t);
    return e.length == i.length && e.every((n) => r[n] == t[n]);
  }
});
T.languageData = Cn;
T.changeFilter = Dn;
T.transactionFilter = On;
T.transactionExtender = Tn;
ve.reconfigure = /* @__PURE__ */ F.define();
class Ct {
  /**
  Compare this value with another value. Used when comparing
  rangesets. The default implementation compares by identity.
  Unless you are only creating a fixed number of unique instances
  of your value type, it is a good idea to implement this
  properly.
  */
  eq(t) {
    return this == t;
  }
  /**
  Create a [range](https://codemirror.net/6/docs/ref/#state.Range) with this value.
  */
  range(t, e = t) {
    return jt.create(t, e, this);
  }
}
Ct.prototype.startSide = Ct.prototype.endSide = 0;
Ct.prototype.point = !1;
Ct.prototype.mapMode = j.TrackDel;
class jt {
  constructor(t, e, i) {
    this.from = t, this.to = e, this.value = i;
  }
  /**
  @internal
  */
  static create(t, e, i) {
    return new jt(t, e, i);
  }
}
function ze(r, t) {
  return r.from - t.from || r.value.startSide - t.value.startSide;
}
class fi {
  constructor(t, e, i, n) {
    this.from = t, this.to = e, this.value = i, this.maxPoint = n;
  }
  get length() {
    return this.to[this.to.length - 1];
  }
  // Find the index of the given position and side. Use the ranges'
  // `from` pos when `end == false`, `to` when `end == true`.
  findIndex(t, e, i, n = 0) {
    let s = i ? this.to : this.from;
    for (let o = n, l = s.length; ; ) {
      if (o == l)
        return o;
      let h = o + l >> 1, f = s[h] - t || (i ? this.value[h].endSide : this.value[h].startSide) - e;
      if (h == o)
        return f >= 0 ? o : l;
      f >= 0 ? l = h : o = h + 1;
    }
  }
  between(t, e, i, n) {
    for (let s = this.findIndex(e, -1e9, !0), o = this.findIndex(i, 1e9, !1, s); s < o; s++)
      if (n(this.from[s] + t, this.to[s] + t, this.value[s]) === !1)
        return !1;
  }
  map(t, e) {
    let i = [], n = [], s = [], o = -1, l = -1;
    for (let h = 0; h < this.value.length; h++) {
      let f = this.value[h], a = this.from[h] + t, c = this.to[h] + t, u, d;
      if (a == c) {
        let g = e.mapPos(a, f.startSide, f.mapMode);
        if (g == null || (u = d = g, f.startSide != f.endSide && (d = e.mapPos(a, f.endSide), d < u)))
          continue;
      } else if (u = e.mapPos(a, f.startSide), d = e.mapPos(c, f.endSide), u > d || u == d && f.startSide > 0 && f.endSide <= 0)
        continue;
      (d - u || f.endSide - f.startSide) < 0 || (o < 0 && (o = u), f.point && (l = Math.max(l, d - u)), i.push(f), n.push(u - o), s.push(d - o));
    }
    return { mapped: i.length ? new fi(n, s, i, l) : null, pos: o };
  }
}
class R {
  constructor(t, e, i, n) {
    this.chunkPos = t, this.chunk = e, this.nextLayer = i, this.maxPoint = n;
  }
  /**
  @internal
  */
  static create(t, e, i, n) {
    return new R(t, e, i, n);
  }
  /**
  @internal
  */
  get length() {
    let t = this.chunk.length - 1;
    return t < 0 ? 0 : Math.max(this.chunkEnd(t), this.nextLayer.length);
  }
  /**
  The number of ranges in the set.
  */
  get size() {
    if (this.isEmpty)
      return 0;
    let t = this.nextLayer.size;
    for (let e of this.chunk)
      t += e.value.length;
    return t;
  }
  /**
  @internal
  */
  chunkEnd(t) {
    return this.chunkPos[t] + this.chunk[t].length;
  }
  /**
  Update the range set, optionally adding new ranges or filtering
  out existing ones.
  
  (Note: The type parameter is just there as a kludge to work
  around TypeScript variance issues that prevented `RangeSet<X>`
  from being a subtype of `RangeSet<Y>` when `X` is a subtype of
  `Y`.)
  */
  update(t) {
    let { add: e = [], sort: i = !1, filterFrom: n = 0, filterTo: s = this.length } = t, o = t.filter;
    if (e.length == 0 && !o)
      return this;
    if (i && (e = e.slice().sort(ze)), this.isEmpty)
      return e.length ? R.of(e) : this;
    let l = new Bn(this, null, -1).goto(0), h = 0, f = [], a = new me();
    for (; l.value || h < e.length; )
      if (h < e.length && (l.from - e[h].from || l.startSide - e[h].value.startSide) >= 0) {
        let c = e[h++];
        a.addInner(c.from, c.to, c.value) || f.push(c);
      } else l.rangeIndex == 1 && l.chunkIndex < this.chunk.length && (h == e.length || this.chunkEnd(l.chunkIndex) < e[h].from) && (!o || n > this.chunkEnd(l.chunkIndex) || s < this.chunkPos[l.chunkIndex]) && a.addChunk(this.chunkPos[l.chunkIndex], this.chunk[l.chunkIndex]) ? l.nextChunk() : ((!o || n > l.to || s < l.from || o(l.from, l.to, l.value)) && (a.addInner(l.from, l.to, l.value) || f.push(jt.create(l.from, l.to, l.value))), l.next());
    return a.finishInner(this.nextLayer.isEmpty && !f.length ? R.empty : this.nextLayer.update({ add: f, filter: o, filterFrom: n, filterTo: s }));
  }
  /**
  Map this range set through a set of changes, return the new set.
  */
  map(t) {
    if (t.empty || this.isEmpty)
      return this;
    let e = [], i = [], n = -1;
    for (let o = 0; o < this.chunk.length; o++) {
      let l = this.chunkPos[o], h = this.chunk[o], f = t.touchesRange(l, l + h.length);
      if (f === !1)
        n = Math.max(n, h.maxPoint), e.push(h), i.push(t.mapPos(l));
      else if (f === !0) {
        let { mapped: a, pos: c } = h.map(l, t);
        a && (n = Math.max(n, a.maxPoint), e.push(a), i.push(c));
      }
    }
    let s = this.nextLayer.map(t);
    return e.length == 0 ? s : new R(i, e, s || R.empty, n);
  }
  /**
  Iterate over the ranges that touch the region `from` to `to`,
  calling `f` for each. There is no guarantee that the ranges will
  be reported in any specific order. When the callback returns
  `false`, iteration stops.
  */
  between(t, e, i) {
    if (!this.isEmpty) {
      for (let n = 0; n < this.chunk.length; n++) {
        let s = this.chunkPos[n], o = this.chunk[n];
        if (e >= s && t <= s + o.length && o.between(s, t - s, e - s, i) === !1)
          return;
      }
      this.nextLayer.between(t, e, i);
    }
  }
  /**
  Iterate over the ranges in this set, in order, including all
  ranges that end at or after `from`.
  */
  iter(t = 0) {
    return Yt.from([this]).goto(t);
  }
  /**
  @internal
  */
  get isEmpty() {
    return this.nextLayer == this;
  }
  /**
  Iterate over the ranges in a collection of sets, in order,
  starting from `from`.
  */
  static iter(t, e = 0) {
    return Yt.from(t).goto(e);
  }
  /**
  Iterate over two groups of sets, calling methods on `comparator`
  to notify it of possible differences.
  */
  static compare(t, e, i, n, s = -1) {
    let o = t.filter((c) => c.maxPoint > 0 || !c.isEmpty && c.maxPoint >= s), l = e.filter((c) => c.maxPoint > 0 || !c.isEmpty && c.maxPoint >= s), h = Ci(o, l, i), f = new Ft(o, h, s), a = new Ft(l, h, s);
    i.iterGaps((c, u, d) => Mi(f, c, a, u, d, n)), i.empty && i.length == 0 && Mi(f, 0, a, 0, 0, n);
  }
  /**
  Compare the contents of two groups of range sets, returning true
  if they are equivalent in the given range.
  */
  static eq(t, e, i = 0, n) {
    n == null && (n = 999999999);
    let s = t.filter((a) => !a.isEmpty && e.indexOf(a) < 0), o = e.filter((a) => !a.isEmpty && t.indexOf(a) < 0);
    if (s.length != o.length)
      return !1;
    if (!s.length)
      return !0;
    let l = Ci(s, o), h = new Ft(s, l, 0).goto(i), f = new Ft(o, l, 0).goto(i);
    for (; ; ) {
      if (h.to != f.to || !qe(h.active, f.active) || h.point && (!f.point || !h.point.eq(f.point)))
        return !1;
      if (h.to > n)
        return !0;
      h.next(), f.next();
    }
  }
  /**
  Iterate over a group of range sets at the same time, notifying
  the iterator about the ranges covering every given piece of
  content. Returns the open count (see
  [`SpanIterator.span`](https://codemirror.net/6/docs/ref/#state.SpanIterator.span)) at the end
  of the iteration.
  */
  static spans(t, e, i, n, s = -1) {
    let o = new Ft(t, null, s).goto(e), l = e, h = o.openStart;
    for (; ; ) {
      let f = Math.min(o.to, i);
      if (o.point) {
        let a = o.activeForPoint(o.to), c = o.pointFrom < e ? a.length + 1 : o.point.startSide < 0 ? a.length : Math.min(a.length, h);
        n.point(l, f, o.point, a, c, o.pointRank), h = Math.min(o.openEnd(f), a.length);
      } else f > l && (n.span(l, f, o.active, h), h = o.openEnd(f));
      if (o.to > i)
        return h + (o.point && o.to > i ? 1 : 0);
      l = o.to, o.next();
    }
  }
  /**
  Create a range set for the given range or array of ranges. By
  default, this expects the ranges to be _sorted_ (by start
  position and, if two start at the same position,
  `value.startSide`). You can pass `true` as second argument to
  cause the method to sort them.
  */
  static of(t, e = !1) {
    let i = new me();
    for (let n of t instanceof jt ? [t] : e ? Ys(t) : t)
      i.add(n.from, n.to, n.value);
    return i.finish();
  }
  /**
  Join an array of range sets into a single set.
  */
  static join(t) {
    if (!t.length)
      return R.empty;
    let e = t[t.length - 1];
    for (let i = t.length - 2; i >= 0; i--)
      for (let n = t[i]; n != R.empty; n = n.nextLayer)
        e = new R(n.chunkPos, n.chunk, e, Math.max(n.maxPoint, e.maxPoint));
    return e;
  }
}
R.empty = /* @__PURE__ */ new R([], [], null, -1);
function Ys(r) {
  if (r.length > 1)
    for (let t = r[0], e = 1; e < r.length; e++) {
      let i = r[e];
      if (ze(t, i) > 0)
        return r.slice().sort(ze);
      t = i;
    }
  return r;
}
R.empty.nextLayer = R.empty;
class me {
  finishChunk(t) {
    this.chunks.push(new fi(this.from, this.to, this.value, this.maxPoint)), this.chunkPos.push(this.chunkStart), this.chunkStart = -1, this.setMaxPoint = Math.max(this.setMaxPoint, this.maxPoint), this.maxPoint = -1, t && (this.from = [], this.to = [], this.value = []);
  }
  /**
  Create an empty builder.
  */
  constructor() {
    this.chunks = [], this.chunkPos = [], this.chunkStart = -1, this.last = null, this.lastFrom = -1e9, this.lastTo = -1e9, this.from = [], this.to = [], this.value = [], this.maxPoint = -1, this.setMaxPoint = -1, this.nextLayer = null;
  }
  /**
  Add a range. Ranges should be added in sorted (by `from` and
  `value.startSide`) order.
  */
  add(t, e, i) {
    this.addInner(t, e, i) || (this.nextLayer || (this.nextLayer = new me())).add(t, e, i);
  }
  /**
  @internal
  */
  addInner(t, e, i) {
    let n = t - this.lastTo || i.startSide - this.last.endSide;
    if (n <= 0 && (t - this.lastFrom || i.startSide - this.last.startSide) < 0)
      throw new Error("Ranges must be added sorted by `from` position and `startSide`");
    return n < 0 ? !1 : (this.from.length == 250 && this.finishChunk(!0), this.chunkStart < 0 && (this.chunkStart = t), this.from.push(t - this.chunkStart), this.to.push(e - this.chunkStart), this.last = i, this.lastFrom = t, this.lastTo = e, this.value.push(i), i.point && (this.maxPoint = Math.max(this.maxPoint, e - t)), !0);
  }
  /**
  @internal
  */
  addChunk(t, e) {
    if ((t - this.lastTo || e.value[0].startSide - this.last.endSide) < 0)
      return !1;
    this.from.length && this.finishChunk(!0), this.setMaxPoint = Math.max(this.setMaxPoint, e.maxPoint), this.chunks.push(e), this.chunkPos.push(t);
    let i = e.value.length - 1;
    return this.last = e.value[i], this.lastFrom = e.from[i] + t, this.lastTo = e.to[i] + t, !0;
  }
  /**
  Finish the range set. Returns the new set. The builder can't be
  used anymore after this has been called.
  */
  finish() {
    return this.finishInner(R.empty);
  }
  /**
  @internal
  */
  finishInner(t) {
    if (this.from.length && this.finishChunk(!1), this.chunks.length == 0)
      return t;
    let e = R.create(this.chunkPos, this.chunks, this.nextLayer ? this.nextLayer.finishInner(t) : t, this.setMaxPoint);
    return this.from = null, e;
  }
}
function Ci(r, t, e) {
  let i = /* @__PURE__ */ new Map();
  for (let s of r)
    for (let o = 0; o < s.chunk.length; o++)
      s.chunk[o].maxPoint <= 0 && i.set(s.chunk[o], s.chunkPos[o]);
  let n = /* @__PURE__ */ new Set();
  for (let s of t)
    for (let o = 0; o < s.chunk.length; o++) {
      let l = i.get(s.chunk[o]);
      l != null && (e ? e.mapPos(l) : l) == s.chunkPos[o] && !(e != null && e.touchesRange(l, l + s.chunk[o].length)) && n.add(s.chunk[o]);
    }
  return n;
}
class Bn {
  constructor(t, e, i, n = 0) {
    this.layer = t, this.skip = e, this.minPoint = i, this.rank = n;
  }
  get startSide() {
    return this.value ? this.value.startSide : 0;
  }
  get endSide() {
    return this.value ? this.value.endSide : 0;
  }
  goto(t, e = -1e9) {
    return this.chunkIndex = this.rangeIndex = 0, this.gotoInner(t, e, !1), this;
  }
  gotoInner(t, e, i) {
    for (; this.chunkIndex < this.layer.chunk.length; ) {
      let n = this.layer.chunk[this.chunkIndex];
      if (!(this.skip && this.skip.has(n) || this.layer.chunkEnd(this.chunkIndex) < t || n.maxPoint < this.minPoint))
        break;
      this.chunkIndex++, i = !1;
    }
    if (this.chunkIndex < this.layer.chunk.length) {
      let n = this.layer.chunk[this.chunkIndex].findIndex(t - this.layer.chunkPos[this.chunkIndex], e, !0);
      (!i || this.rangeIndex < n) && this.setRangeIndex(n);
    }
    this.next();
  }
  forward(t, e) {
    (this.to - t || this.endSide - e) < 0 && this.gotoInner(t, e, !0);
  }
  next() {
    for (; ; )
      if (this.chunkIndex == this.layer.chunk.length) {
        this.from = this.to = 1e9, this.value = null;
        break;
      } else {
        let t = this.layer.chunkPos[this.chunkIndex], e = this.layer.chunk[this.chunkIndex], i = t + e.from[this.rangeIndex];
        if (this.from = i, this.to = t + e.to[this.rangeIndex], this.value = e.value[this.rangeIndex], this.setRangeIndex(this.rangeIndex + 1), this.minPoint < 0 || this.value.point && this.to - this.from >= this.minPoint)
          break;
      }
  }
  setRangeIndex(t) {
    if (t == this.layer.chunk[this.chunkIndex].value.length) {
      if (this.chunkIndex++, this.skip)
        for (; this.chunkIndex < this.layer.chunk.length && this.skip.has(this.layer.chunk[this.chunkIndex]); )
          this.chunkIndex++;
      this.rangeIndex = 0;
    } else
      this.rangeIndex = t;
  }
  nextChunk() {
    this.chunkIndex++, this.rangeIndex = 0, this.next();
  }
  compare(t) {
    return this.from - t.from || this.startSide - t.startSide || this.rank - t.rank || this.to - t.to || this.endSide - t.endSide;
  }
}
class Yt {
  constructor(t) {
    this.heap = t;
  }
  static from(t, e = null, i = -1) {
    let n = [];
    for (let s = 0; s < t.length; s++)
      for (let o = t[s]; !o.isEmpty; o = o.nextLayer)
        o.maxPoint >= i && n.push(new Bn(o, e, i, s));
    return n.length == 1 ? n[0] : new Yt(n);
  }
  get startSide() {
    return this.value ? this.value.startSide : 0;
  }
  goto(t, e = -1e9) {
    for (let i of this.heap)
      i.goto(t, e);
    for (let i = this.heap.length >> 1; i >= 0; i--)
      Ce(this.heap, i);
    return this.next(), this;
  }
  forward(t, e) {
    for (let i of this.heap)
      i.forward(t, e);
    for (let i = this.heap.length >> 1; i >= 0; i--)
      Ce(this.heap, i);
    (this.to - t || this.value.endSide - e) < 0 && this.next();
  }
  next() {
    if (this.heap.length == 0)
      this.from = this.to = 1e9, this.value = null, this.rank = -1;
    else {
      let t = this.heap[0];
      this.from = t.from, this.to = t.to, this.value = t.value, this.rank = t.rank, t.value && t.next(), Ce(this.heap, 0);
    }
  }
}
function Ce(r, t) {
  for (let e = r[t]; ; ) {
    let i = (t << 1) + 1;
    if (i >= r.length)
      break;
    let n = r[i];
    if (i + 1 < r.length && n.compare(r[i + 1]) >= 0 && (n = r[i + 1], i++), e.compare(n) < 0)
      break;
    r[i] = e, r[t] = n, t = i;
  }
}
class Ft {
  constructor(t, e, i) {
    this.minPoint = i, this.active = [], this.activeTo = [], this.activeRank = [], this.minActive = -1, this.point = null, this.pointFrom = 0, this.pointRank = 0, this.to = -1e9, this.endSide = 0, this.openStart = -1, this.cursor = Yt.from(t, e, i);
  }
  goto(t, e = -1e9) {
    return this.cursor.goto(t, e), this.active.length = this.activeTo.length = this.activeRank.length = 0, this.minActive = -1, this.to = t, this.endSide = e, this.openStart = -1, this.next(), this;
  }
  forward(t, e) {
    for (; this.minActive > -1 && (this.activeTo[this.minActive] - t || this.active[this.minActive].endSide - e) < 0; )
      this.removeActive(this.minActive);
    this.cursor.forward(t, e);
  }
  removeActive(t) {
    _t(this.active, t), _t(this.activeTo, t), _t(this.activeRank, t), this.minActive = Di(this.active, this.activeTo);
  }
  addActive(t) {
    let e = 0, { value: i, to: n, rank: s } = this.cursor;
    for (; e < this.activeRank.length && (s - this.activeRank[e] || n - this.activeTo[e]) > 0; )
      e++;
    te(this.active, e, i), te(this.activeTo, e, n), te(this.activeRank, e, s), t && te(t, e, this.cursor.from), this.minActive = Di(this.active, this.activeTo);
  }
  // After calling this, if `this.point` != null, the next range is a
  // point. Otherwise, it's a regular range, covered by `this.active`.
  next() {
    let t = this.to, e = this.point;
    this.point = null;
    let i = this.openStart < 0 ? [] : null;
    for (; ; ) {
      let n = this.minActive;
      if (n > -1 && (this.activeTo[n] - this.cursor.from || this.active[n].endSide - this.cursor.startSide) < 0) {
        if (this.activeTo[n] > t) {
          this.to = this.activeTo[n], this.endSide = this.active[n].endSide;
          break;
        }
        this.removeActive(n), i && _t(i, n);
      } else if (this.cursor.value)
        if (this.cursor.from > t) {
          this.to = this.cursor.from, this.endSide = this.cursor.startSide;
          break;
        } else {
          let s = this.cursor.value;
          if (!s.point)
            this.addActive(i), this.cursor.next();
          else if (e && this.cursor.to == this.to && this.cursor.from < this.cursor.to)
            this.cursor.next();
          else {
            this.point = s, this.pointFrom = this.cursor.from, this.pointRank = this.cursor.rank, this.to = this.cursor.to, this.endSide = s.endSide, this.cursor.next(), this.forward(this.to, this.endSide);
            break;
          }
        }
      else {
        this.to = this.endSide = 1e9;
        break;
      }
    }
    if (i) {
      this.openStart = 0;
      for (let n = i.length - 1; n >= 0 && i[n] < t; n--)
        this.openStart++;
    }
  }
  activeForPoint(t) {
    if (!this.active.length)
      return this.active;
    let e = [];
    for (let i = this.active.length - 1; i >= 0 && !(this.activeRank[i] < this.pointRank); i--)
      (this.activeTo[i] > t || this.activeTo[i] == t && this.active[i].endSide >= this.point.endSide) && e.push(this.active[i]);
    return e.reverse();
  }
  openEnd(t) {
    let e = 0;
    for (let i = this.activeTo.length - 1; i >= 0 && this.activeTo[i] > t; i--)
      e++;
    return e;
  }
}
function Mi(r, t, e, i, n, s) {
  r.goto(t), e.goto(i);
  let o = i + n, l = i, h = i - t;
  for (; ; ) {
    let f = r.to + h - e.to || r.endSide - e.endSide, a = f < 0 ? r.to + h : e.to, c = Math.min(a, o);
    if (r.point || e.point ? r.point && e.point && (r.point == e.point || r.point.eq(e.point)) && qe(r.activeForPoint(r.to), e.activeForPoint(e.to)) || s.comparePoint(l, c, r.point, e.point) : c > l && !qe(r.active, e.active) && s.compareRange(l, c, r.active, e.active), a > o)
      break;
    l = a, f <= 0 && r.next(), f >= 0 && e.next();
  }
}
function qe(r, t) {
  if (r.length != t.length)
    return !1;
  for (let e = 0; e < r.length; e++)
    if (r[e] != t[e] && !r[e].eq(t[e]))
      return !1;
  return !0;
}
function _t(r, t) {
  for (let e = t, i = r.length - 1; e < i; e++)
    r[e] = r[e + 1];
  r.pop();
}
function te(r, t, e) {
  for (let i = r.length - 1; i >= t; i--)
    r[i + 1] = r[i];
  r[t] = e;
}
function Di(r, t) {
  let e = -1, i = 1e9;
  for (let n = 0; n < t.length; n++)
    (t[n] - i || r[n].endSide - r[e].endSide) < 0 && (e = n, i = t[n]);
  return e;
}
function Gs(r, t, e, i) {
  for (let n = 0, s = 0; ; ) {
    if (s >= t)
      return n;
    if (n == r.length)
      break;
    s += r.charCodeAt(n) == 9 ? e - s % e : 1, n = it(r, n);
  }
  return i === !0 ? -1 : r.length;
}
const Ke = "ͼ", Oi = typeof Symbol > "u" ? "__" + Ke : Symbol.for(Ke), $e = typeof Symbol > "u" ? "__styleSet" + Math.floor(Math.random() * 1e8) : Symbol("styleSet"), Ti = typeof globalThis < "u" ? globalThis : typeof window < "u" ? window : {};
class Mt {
  // :: (Object<Style>, ?{finish: ?(string) → string})
  // Create a style module from the given spec.
  //
  // When `finish` is given, it is called on regular (non-`@`)
  // selectors (after `&` expansion) to compute the final selector.
  constructor(t, e) {
    this.rules = [];
    let { finish: i } = e || {};
    function n(o) {
      return /^@/.test(o) ? [o] : o.split(/,\s*/);
    }
    function s(o, l, h, f) {
      let a = [], c = /^@(\w+)\b/.exec(o[0]), u = c && c[1] == "keyframes";
      if (c && l == null) return h.push(o[0] + ";");
      for (let d in l) {
        let g = l[d];
        if (/&/.test(d))
          s(
            d.split(/,\s*/).map((p) => o.map((m) => p.replace(/&/, m))).reduce((p, m) => p.concat(m)),
            g,
            h
          );
        else if (g && typeof g == "object") {
          if (!c) throw new RangeError("The value of a property (" + d + ") should be a primitive value.");
          s(n(d), g, a, u);
        } else g != null && a.push(d.replace(/_.*/, "").replace(/[A-Z]/g, (p) => "-" + p.toLowerCase()) + ": " + g + ";");
      }
      (a.length || u) && h.push((i && !c && !f ? o.map(i) : o).join(", ") + " {" + a.join(" ") + "}");
    }
    for (let o in t) s(n(o), t[o], this.rules);
  }
  // :: () → string
  // Returns a string containing the module's CSS rules.
  getRules() {
    return this.rules.join(`
`);
  }
  // :: () → string
  // Generate a new unique CSS class name.
  static newName() {
    let t = Ti[Oi] || 1;
    return Ti[Oi] = t + 1, Ke + t.toString(36);
  }
  // :: (union<Document, ShadowRoot>, union<[StyleModule], StyleModule>, ?{nonce: ?string})
  //
  // Mount the given set of modules in the given DOM root, which ensures
  // that the CSS rules defined by the module are available in that
  // context.
  //
  // Rules are only added to the document once per root.
  //
  // Rule order will follow the order of the modules, so that rules from
  // modules later in the array take precedence of those from earlier
  // modules. If you call this function multiple times for the same root
  // in a way that changes the order of already mounted modules, the old
  // order will be changed.
  //
  // If a Content Security Policy nonce is provided, it is added to
  // the `<style>` tag generated by the library.
  static mount(t, e, i) {
    let n = t[$e], s = i && i.nonce;
    n ? s && n.setNonce(s) : n = new Us(t, s), n.mount(Array.isArray(e) ? e : [e], t);
  }
}
let Ri = /* @__PURE__ */ new Map();
class Us {
  constructor(t, e) {
    let i = t.ownerDocument || t, n = i.defaultView;
    if (!t.head && t.adoptedStyleSheets && n.CSSStyleSheet) {
      let s = Ri.get(i);
      if (s) return t[$e] = s;
      this.sheet = new n.CSSStyleSheet(), Ri.set(i, this);
    } else
      this.styleTag = i.createElement("style"), e && this.styleTag.setAttribute("nonce", e);
    this.modules = [], t[$e] = this;
  }
  mount(t, e) {
    let i = this.sheet, n = 0, s = 0;
    for (let o = 0; o < t.length; o++) {
      let l = t[o], h = this.modules.indexOf(l);
      if (h < s && h > -1 && (this.modules.splice(h, 1), s--, h = -1), h == -1) {
        if (this.modules.splice(s++, 0, l), i) for (let f = 0; f < l.rules.length; f++)
          i.insertRule(l.rules[f], n++);
      } else {
        for (; s < h; ) n += this.modules[s++].rules.length;
        n += l.rules.length, s++;
      }
    }
    if (i)
      e.adoptedStyleSheets.indexOf(this.sheet) < 0 && (e.adoptedStyleSheets = [this.sheet, ...e.adoptedStyleSheets]);
    else {
      let o = "";
      for (let h = 0; h < this.modules.length; h++)
        o += this.modules[h].getRules() + `
`;
      this.styleTag.textContent = o;
      let l = e.head || e;
      this.styleTag.parentNode != l && l.insertBefore(this.styleTag, l.firstChild);
    }
  }
  setNonce(t) {
    this.styleTag && this.styleTag.getAttribute("nonce") != t && this.styleTag.setAttribute("nonce", t);
  }
}
var Dt = {
  8: "Backspace",
  9: "Tab",
  10: "Enter",
  12: "NumLock",
  13: "Enter",
  16: "Shift",
  17: "Control",
  18: "Alt",
  20: "CapsLock",
  27: "Escape",
  32: " ",
  33: "PageUp",
  34: "PageDown",
  35: "End",
  36: "Home",
  37: "ArrowLeft",
  38: "ArrowUp",
  39: "ArrowRight",
  40: "ArrowDown",
  44: "PrintScreen",
  45: "Insert",
  46: "Delete",
  59: ";",
  61: "=",
  91: "Meta",
  92: "Meta",
  106: "*",
  107: "+",
  108: ",",
  109: "-",
  110: ".",
  111: "/",
  144: "NumLock",
  145: "ScrollLock",
  160: "Shift",
  161: "Shift",
  162: "Control",
  163: "Control",
  164: "Alt",
  165: "Alt",
  173: "-",
  186: ";",
  187: "=",
  188: ",",
  189: "-",
  190: ".",
  191: "/",
  192: "`",
  219: "[",
  220: "\\",
  221: "]",
  222: "'"
}, je = {
  48: ")",
  49: "!",
  50: "@",
  51: "#",
  52: "$",
  53: "%",
  54: "^",
  55: "&",
  56: "*",
  57: "(",
  59: ":",
  61: "+",
  173: "_",
  186: ":",
  187: "+",
  188: "<",
  189: "_",
  190: ">",
  191: "?",
  192: "~",
  219: "{",
  220: "|",
  221: "}",
  222: '"'
};
for (var N = 0; N < 10; N++) Dt[48 + N] = Dt[96 + N] = String(N);
for (var N = 1; N <= 24; N++) Dt[N + 111] = "F" + N;
for (var N = 65; N <= 90; N++)
  Dt[N] = String.fromCharCode(N + 32), je[N] = String.fromCharCode(N);
for (var Me in Dt) je.hasOwnProperty(Me) || (je[Me] = Dt[Me]);
function Gt(r) {
  let t;
  return r.nodeType == 11 ? t = r.getSelection ? r : r.ownerDocument : t = r, t.getSelection();
}
function Ye(r, t) {
  return t ? r == t || r.contains(t.nodeType != 1 ? t.parentNode : t) : !1;
}
function Xs(r) {
  let t = r.activeElement;
  for (; t && t.shadowRoot; )
    t = t.shadowRoot.activeElement;
  return t;
}
function ce(r, t) {
  if (!t.anchorNode)
    return !1;
  try {
    return Ye(r, t.anchorNode);
  } catch {
    return !1;
  }
}
function Ut(r) {
  return r.nodeType == 3 ? yt(r, 0, r.nodeValue.length).getClientRects() : r.nodeType == 1 ? r.getClientRects() : [];
}
function qt(r, t, e, i) {
  return e ? Ei(r, t, e, i, -1) || Ei(r, t, e, i, 1) : !1;
}
function mt(r) {
  for (var t = 0; ; t++)
    if (r = r.previousSibling, !r)
      return t;
}
function ye(r) {
  return r.nodeType == 1 && /^(DIV|P|LI|UL|OL|BLOCKQUOTE|DD|DT|H\d|SECTION|PRE)$/.test(r.nodeName);
}
function Ei(r, t, e, i, n) {
  for (; ; ) {
    if (r == e && t == i)
      return !0;
    if (t == (n < 0 ? 0 : lt(r))) {
      if (r.nodeName == "DIV")
        return !1;
      let s = r.parentNode;
      if (!s || s.nodeType != 1)
        return !1;
      t = mt(r) + (n < 0 ? 0 : 1), r = s;
    } else if (r.nodeType == 1) {
      if (r = r.childNodes[t + (n < 0 ? -1 : 0)], r.nodeType == 1 && r.contentEditable == "false")
        return !1;
      t = n < 0 ? lt(r) : 0;
    } else
      return !1;
  }
}
function lt(r) {
  return r.nodeType == 3 ? r.nodeValue.length : r.childNodes.length;
}
function ci(r, t) {
  let e = t ? r.left : r.right;
  return { left: e, right: e, top: r.top, bottom: r.bottom };
}
function Js(r) {
  let t = r.visualViewport;
  return t ? {
    left: 0,
    right: t.width,
    top: 0,
    bottom: t.height
  } : {
    left: 0,
    right: r.innerWidth,
    top: 0,
    bottom: r.innerHeight
  };
}
function Ln(r, t) {
  let e = t.width / r.offsetWidth, i = t.height / r.offsetHeight;
  return (e > 0.995 && e < 1.005 || !isFinite(e) || Math.abs(t.width - r.offsetWidth) < 1) && (e = 1), (i > 0.995 && i < 1.005 || !isFinite(i) || Math.abs(t.height - r.offsetHeight) < 1) && (i = 1), { scaleX: e, scaleY: i };
}
function Qs(r, t, e, i, n, s, o, l) {
  let h = r.ownerDocument, f = h.defaultView || window;
  for (let a = r, c = !1; a && !c; )
    if (a.nodeType == 1) {
      let u, d = a == h.body, g = 1, p = 1;
      if (d)
        u = Js(f);
      else {
        if (/^(fixed|sticky)$/.test(getComputedStyle(a).position) && (c = !0), a.scrollHeight <= a.clientHeight && a.scrollWidth <= a.clientWidth) {
          a = a.assignedSlot || a.parentNode;
          continue;
        }
        let S = a.getBoundingClientRect();
        ({ scaleX: g, scaleY: p } = Ln(a, S)), u = {
          left: S.left,
          right: S.left + a.clientWidth * g,
          top: S.top,
          bottom: S.top + a.clientHeight * p
        };
      }
      let m = 0, y = 0;
      if (n == "nearest")
        t.top < u.top ? (y = -(u.top - t.top + o), e > 0 && t.bottom > u.bottom + y && (y = t.bottom - u.bottom + y + o)) : t.bottom > u.bottom && (y = t.bottom - u.bottom + o, e < 0 && t.top - y < u.top && (y = -(u.top + y - t.top + o)));
      else {
        let S = t.bottom - t.top, k = u.bottom - u.top;
        y = (n == "center" && S <= k ? t.top + S / 2 - k / 2 : n == "start" || n == "center" && e < 0 ? t.top - o : t.bottom - k + o) - u.top;
      }
      if (i == "nearest" ? t.left < u.left ? (m = -(u.left - t.left + s), e > 0 && t.right > u.right + m && (m = t.right - u.right + m + s)) : t.right > u.right && (m = t.right - u.right + s, e < 0 && t.left < u.left + m && (m = -(u.left + m - t.left + s))) : m = (i == "center" ? t.left + (t.right - t.left) / 2 - (u.right - u.left) / 2 : i == "start" == l ? t.left - s : t.right - (u.right - u.left) + s) - u.left, m || y)
        if (d)
          f.scrollBy(m, y);
        else {
          let S = 0, k = 0;
          if (y) {
            let x = a.scrollTop;
            a.scrollTop += y / p, k = (a.scrollTop - x) * p;
          }
          if (m) {
            let x = a.scrollLeft;
            a.scrollLeft += m / g, S = (a.scrollLeft - x) * g;
          }
          t = {
            left: t.left - S,
            top: t.top - k,
            right: t.right - S,
            bottom: t.bottom - k
          }, S && Math.abs(S - m) < 1 && (i = "nearest"), k && Math.abs(k - y) < 1 && (n = "nearest");
        }
      if (d)
        break;
      a = a.assignedSlot || a.parentNode;
    } else if (a.nodeType == 11)
      a = a.host;
    else
      break;
}
function Zs(r) {
  let t = r.ownerDocument, e, i;
  for (let n = r.parentNode; n && !(n == t.body || e && i); )
    if (n.nodeType == 1)
      !i && n.scrollHeight > n.clientHeight && (i = n), !e && n.scrollWidth > n.clientWidth && (e = n), n = n.assignedSlot || n.parentNode;
    else if (n.nodeType == 11)
      n = n.host;
    else
      break;
  return { x: e, y: i };
}
class _s {
  constructor() {
    this.anchorNode = null, this.anchorOffset = 0, this.focusNode = null, this.focusOffset = 0;
  }
  eq(t) {
    return this.anchorNode == t.anchorNode && this.anchorOffset == t.anchorOffset && this.focusNode == t.focusNode && this.focusOffset == t.focusOffset;
  }
  setRange(t) {
    let { anchorNode: e, focusNode: i } = t;
    this.set(e, Math.min(t.anchorOffset, e ? lt(e) : 0), i, Math.min(t.focusOffset, i ? lt(i) : 0));
  }
  set(t, e, i, n) {
    this.anchorNode = t, this.anchorOffset = e, this.focusNode = i, this.focusOffset = n;
  }
}
let bt = null;
function Fn(r) {
  if (r.setActive)
    return r.setActive();
  if (bt)
    return r.focus(bt);
  let t = [];
  for (let e = r; e && (t.push(e, e.scrollTop, e.scrollLeft), e != e.ownerDocument); e = e.parentNode)
    ;
  if (r.focus(bt == null ? {
    get preventScroll() {
      return bt = { preventScroll: !0 }, !0;
    }
  } : void 0), !bt) {
    bt = !1;
    for (let e = 0; e < t.length; ) {
      let i = t[e++], n = t[e++], s = t[e++];
      i.scrollTop != n && (i.scrollTop = n), i.scrollLeft != s && (i.scrollLeft = s);
    }
  }
}
let Pi;
function yt(r, t, e = t) {
  let i = Pi || (Pi = document.createRange());
  return i.setEnd(r, e), i.setStart(r, t), i;
}
function vt(r, t, e, i) {
  let n = { key: t, code: t, keyCode: e, which: e, cancelable: !0 };
  i && ({ altKey: n.altKey, ctrlKey: n.ctrlKey, shiftKey: n.shiftKey, metaKey: n.metaKey } = i);
  let s = new KeyboardEvent("keydown", n);
  s.synthetic = !0, r.dispatchEvent(s);
  let o = new KeyboardEvent("keyup", n);
  return o.synthetic = !0, r.dispatchEvent(o), s.defaultPrevented || o.defaultPrevented;
}
function tr(r) {
  for (; r; ) {
    if (r && (r.nodeType == 9 || r.nodeType == 11 && r.host))
      return r;
    r = r.assignedSlot || r.parentNode;
  }
  return null;
}
function In(r) {
  for (; r.attributes.length; )
    r.removeAttributeNode(r.attributes[0]);
}
function er(r, t) {
  let e = t.focusNode, i = t.focusOffset;
  if (!e || t.anchorNode != e || t.anchorOffset != i)
    return !1;
  for (i = Math.min(i, lt(e)); ; )
    if (i) {
      if (e.nodeType != 1)
        return !1;
      let n = e.childNodes[i - 1];
      n.contentEditable == "false" ? i-- : (e = n, i = lt(e));
    } else {
      if (e == r)
        return !0;
      i = mt(e), e = e.parentNode;
    }
}
function Nn(r) {
  return r.scrollTop > Math.max(1, r.scrollHeight - r.clientHeight - 4);
}
function Hn(r, t) {
  for (let e = r, i = t; ; ) {
    if (e.nodeType == 3 && i > 0)
      return { node: e, offset: i };
    if (e.nodeType == 1 && i > 0) {
      if (e.contentEditable == "false")
        return null;
      e = e.childNodes[i - 1], i = lt(e);
    } else if (e.parentNode && !ye(e))
      i = mt(e), e = e.parentNode;
    else
      return null;
  }
}
function Vn(r, t) {
  for (let e = r, i = t; ; ) {
    if (e.nodeType == 3 && i < e.nodeValue.length)
      return { node: e, offset: i };
    if (e.nodeType == 1 && i < e.childNodes.length) {
      if (e.contentEditable == "false")
        return null;
      e = e.childNodes[i], i = 0;
    } else if (e.parentNode && !ye(e))
      i = mt(e) + 1, e = e.parentNode;
    else
      return null;
  }
}
class V {
  constructor(t, e, i = !0) {
    this.node = t, this.offset = e, this.precise = i;
  }
  static before(t, e) {
    return new V(t.parentNode, mt(t), e);
  }
  static after(t, e) {
    return new V(t.parentNode, mt(t) + 1, e);
  }
}
const ui = [];
class M {
  constructor() {
    this.parent = null, this.dom = null, this.flags = 2;
  }
  get overrideDOMText() {
    return null;
  }
  get posAtStart() {
    return this.parent ? this.parent.posBefore(this) : 0;
  }
  get posAtEnd() {
    return this.posAtStart + this.length;
  }
  posBefore(t) {
    let e = this.posAtStart;
    for (let i of this.children) {
      if (i == t)
        return e;
      e += i.length + i.breakAfter;
    }
    throw new RangeError("Invalid child in posBefore");
  }
  posAfter(t) {
    return this.posBefore(t) + t.length;
  }
  sync(t, e) {
    if (this.flags & 2) {
      let i = this.dom, n = null, s;
      for (let o of this.children) {
        if (o.flags & 7) {
          if (!o.dom && (s = n ? n.nextSibling : i.firstChild)) {
            let l = M.get(s);
            (!l || !l.parent && l.canReuseDOM(o)) && o.reuseDOM(s);
          }
          o.sync(t, e), o.flags &= -8;
        }
        if (s = n ? n.nextSibling : i.firstChild, e && !e.written && e.node == i && s != o.dom && (e.written = !0), o.dom.parentNode == i)
          for (; s && s != o.dom; )
            s = Bi(s);
        else
          i.insertBefore(o.dom, s);
        n = o.dom;
      }
      for (s = n ? n.nextSibling : i.firstChild, s && e && e.node == i && (e.written = !0); s; )
        s = Bi(s);
    } else if (this.flags & 1)
      for (let i of this.children)
        i.flags & 7 && (i.sync(t, e), i.flags &= -8);
  }
  reuseDOM(t) {
  }
  localPosFromDOM(t, e) {
    let i;
    if (t == this.dom)
      i = this.dom.childNodes[e];
    else {
      let n = lt(t) == 0 ? 0 : e == 0 ? -1 : 1;
      for (; ; ) {
        let s = t.parentNode;
        if (s == this.dom)
          break;
        n == 0 && s.firstChild != s.lastChild && (t == s.firstChild ? n = -1 : n = 1), t = s;
      }
      n < 0 ? i = t : i = t.nextSibling;
    }
    if (i == this.dom.firstChild)
      return 0;
    for (; i && !M.get(i); )
      i = i.nextSibling;
    if (!i)
      return this.length;
    for (let n = 0, s = 0; ; n++) {
      let o = this.children[n];
      if (o.dom == i)
        return s;
      s += o.length + o.breakAfter;
    }
  }
  domBoundsAround(t, e, i = 0) {
    let n = -1, s = -1, o = -1, l = -1;
    for (let h = 0, f = i, a = i; h < this.children.length; h++) {
      let c = this.children[h], u = f + c.length;
      if (f < t && u > e)
        return c.domBoundsAround(t, e, f);
      if (u >= t && n == -1 && (n = h, s = f), f > e && c.dom.parentNode == this.dom) {
        o = h, l = a;
        break;
      }
      a = u, f = u + c.breakAfter;
    }
    return {
      from: s,
      to: l < 0 ? i + this.length : l,
      startDOM: (n ? this.children[n - 1].dom.nextSibling : null) || this.dom.firstChild,
      endDOM: o < this.children.length && o >= 0 ? this.children[o].dom : null
    };
  }
  markDirty(t = !1) {
    this.flags |= 2, this.markParentsDirty(t);
  }
  markParentsDirty(t) {
    for (let e = this.parent; e; e = e.parent) {
      if (t && (e.flags |= 2), e.flags & 1)
        return;
      e.flags |= 1, t = !1;
    }
  }
  setParent(t) {
    this.parent != t && (this.parent = t, this.flags & 7 && this.markParentsDirty(!0));
  }
  setDOM(t) {
    this.dom != t && (this.dom && (this.dom.cmView = null), this.dom = t, t.cmView = this);
  }
  get rootView() {
    for (let t = this; ; ) {
      let e = t.parent;
      if (!e)
        return t;
      t = e;
    }
  }
  replaceChildren(t, e, i = ui) {
    this.markDirty();
    for (let n = t; n < e; n++) {
      let s = this.children[n];
      s.parent == this && i.indexOf(s) < 0 && s.destroy();
    }
    this.children.splice(t, e - t, ...i);
    for (let n = 0; n < i.length; n++)
      i[n].setParent(this);
  }
  ignoreMutation(t) {
    return !1;
  }
  ignoreEvent(t) {
    return !1;
  }
  childCursor(t = this.length) {
    return new Wn(this.children, t, this.children.length);
  }
  childPos(t, e = 1) {
    return this.childCursor().findPos(t, e);
  }
  toString() {
    let t = this.constructor.name.replace("View", "");
    return t + (this.children.length ? "(" + this.children.join() + ")" : this.length ? "[" + (t == "Text" ? this.text : this.length) + "]" : "") + (this.breakAfter ? "#" : "");
  }
  static get(t) {
    return t.cmView;
  }
  get isEditable() {
    return !0;
  }
  get isWidget() {
    return !1;
  }
  get isHidden() {
    return !1;
  }
  merge(t, e, i, n, s, o) {
    return !1;
  }
  become(t) {
    return !1;
  }
  canReuseDOM(t) {
    return t.constructor == this.constructor && !((this.flags | t.flags) & 8);
  }
  // When this is a zero-length view with a side, this should return a
  // number <= 0 to indicate it is before its position, or a
  // number > 0 when after its position.
  getSide() {
    return 0;
  }
  destroy() {
    for (let t of this.children)
      t.parent == this && t.destroy();
    this.parent = null;
  }
}
M.prototype.breakAfter = 0;
function Bi(r) {
  let t = r.nextSibling;
  return r.parentNode.removeChild(r), t;
}
class Wn {
  constructor(t, e, i) {
    this.children = t, this.pos = e, this.i = i, this.off = 0;
  }
  findPos(t, e = 1) {
    for (; ; ) {
      if (t > this.pos || t == this.pos && (e > 0 || this.i == 0 || this.children[this.i - 1].breakAfter))
        return this.off = t - this.pos, this;
      let i = this.children[--this.i];
      this.pos -= i.length + i.breakAfter;
    }
  }
}
function zn(r, t, e, i, n, s, o, l, h) {
  let { children: f } = r, a = f.length ? f[t] : null, c = s.length ? s[s.length - 1] : null, u = c ? c.breakAfter : o;
  if (!(t == i && a && !o && !u && s.length < 2 && a.merge(e, n, s.length ? c : null, e == 0, l, h))) {
    if (i < f.length) {
      let d = f[i];
      d && (n < d.length || d.breakAfter && (c != null && c.breakAfter)) ? (t == i && (d = d.split(n), n = 0), !u && c && d.merge(0, n, c, !0, 0, h) ? s[s.length - 1] = d : ((n || d.children.length && !d.children[0].length) && d.merge(0, n, null, !1, 0, h), s.push(d))) : d != null && d.breakAfter && (c ? c.breakAfter = 1 : o = 1), i++;
    }
    for (a && (a.breakAfter = o, e > 0 && (!o && s.length && a.merge(e, a.length, s[0], !1, l, 0) ? a.breakAfter = s.shift().breakAfter : (e < a.length || a.children.length && a.children[a.children.length - 1].length == 0) && a.merge(e, a.length, null, !1, l, 0), t++)); t < i && s.length; )
      if (f[i - 1].become(s[s.length - 1]))
        i--, s.pop(), h = s.length ? 0 : l;
      else if (f[t].become(s[0]))
        t++, s.shift(), l = s.length ? 0 : h;
      else
        break;
    !s.length && t && i < f.length && !f[t - 1].breakAfter && f[i].merge(0, 0, f[t - 1], !1, l, h) && t--, (t < i || s.length) && r.replaceChildren(t, i, s);
  }
}
function qn(r, t, e, i, n, s) {
  let o = r.childCursor(), { i: l, off: h } = o.findPos(e, 1), { i: f, off: a } = o.findPos(t, -1), c = t - e;
  for (let u of i)
    c += u.length;
  r.length += c, zn(r, f, a, l, h, i, 0, n, s);
}
let K = typeof navigator < "u" ? navigator : { userAgent: "", vendor: "", platform: "" }, Ge = typeof document < "u" ? document : { documentElement: { style: {} } };
const Ue = /* @__PURE__ */ /Edge\/(\d+)/.exec(K.userAgent), Kn = /* @__PURE__ */ /MSIE \d/.test(K.userAgent), Xe = /* @__PURE__ */ /Trident\/(?:[7-9]|\d{2,})\..*rv:(\d+)/.exec(K.userAgent), ke = !!(Kn || Xe || Ue), Li = !ke && /* @__PURE__ */ /gecko\/(\d+)/i.test(K.userAgent), De = !ke && /* @__PURE__ */ /Chrome\/(\d+)/.exec(K.userAgent), Fi = "webkitFontSmoothing" in Ge.documentElement.style, $n = !ke && /* @__PURE__ */ /Apple Computer/.test(K.vendor), Ii = $n && (/* @__PURE__ */ /Mobile\/\w+/.test(K.userAgent) || K.maxTouchPoints > 2);
var w = {
  mac: Ii || /* @__PURE__ */ /Mac/.test(K.platform),
  windows: /* @__PURE__ */ /Win/.test(K.platform),
  linux: /* @__PURE__ */ /Linux|X11/.test(K.platform),
  ie: ke,
  ie_version: Kn ? Ge.documentMode || 6 : Xe ? +Xe[1] : Ue ? +Ue[1] : 0,
  gecko: Li,
  gecko_version: Li ? +(/* @__PURE__ */ /Firefox\/(\d+)/.exec(K.userAgent) || [0, 0])[1] : 0,
  chrome: !!De,
  chrome_version: De ? +De[1] : 0,
  ios: Ii,
  android: /* @__PURE__ */ /Android\b/.test(K.userAgent),
  webkit: Fi,
  safari: $n,
  webkit_version: Fi ? +(/* @__PURE__ */ /\bAppleWebKit\/(\d+)/.exec(K.userAgent) || [0, 0])[1] : 0,
  tabSize: Ge.documentElement.style.tabSize != null ? "tab-size" : "-moz-tab-size"
};
const ir = 256;
class J extends M {
  constructor(t) {
    super(), this.text = t;
  }
  get length() {
    return this.text.length;
  }
  createDOM(t) {
    this.setDOM(t || document.createTextNode(this.text));
  }
  sync(t, e) {
    this.dom || this.createDOM(), this.dom.nodeValue != this.text && (e && e.node == this.dom && (e.written = !0), this.dom.nodeValue = this.text);
  }
  reuseDOM(t) {
    t.nodeType == 3 && this.createDOM(t);
  }
  merge(t, e, i) {
    return this.flags & 8 || i && (!(i instanceof J) || this.length - (e - t) + i.length > ir || i.flags & 8) ? !1 : (this.text = this.text.slice(0, t) + (i ? i.text : "") + this.text.slice(e), this.markDirty(), !0);
  }
  split(t) {
    let e = new J(this.text.slice(t));
    return this.text = this.text.slice(0, t), this.markDirty(), e.flags |= this.flags & 8, e;
  }
  localPosFromDOM(t, e) {
    return t == this.dom ? e : e ? this.text.length : 0;
  }
  domAtPos(t) {
    return new V(this.dom, t);
  }
  domBoundsAround(t, e, i) {
    return { from: i, to: i + this.length, startDOM: this.dom, endDOM: this.dom.nextSibling };
  }
  coordsAt(t, e) {
    return nr(this.dom, t, e);
  }
}
class ht extends M {
  constructor(t, e = [], i = 0) {
    super(), this.mark = t, this.children = e, this.length = i;
    for (let n of e)
      n.setParent(this);
  }
  setAttrs(t) {
    if (In(t), this.mark.class && (t.className = this.mark.class), this.mark.attrs)
      for (let e in this.mark.attrs)
        t.setAttribute(e, this.mark.attrs[e]);
    return t;
  }
  canReuseDOM(t) {
    return super.canReuseDOM(t) && !((this.flags | t.flags) & 8);
  }
  reuseDOM(t) {
    t.nodeName == this.mark.tagName.toUpperCase() && (this.setDOM(t), this.flags |= 6);
  }
  sync(t, e) {
    this.dom ? this.flags & 4 && this.setAttrs(this.dom) : this.setDOM(this.setAttrs(document.createElement(this.mark.tagName))), super.sync(t, e);
  }
  merge(t, e, i, n, s, o) {
    return i && (!(i instanceof ht && i.mark.eq(this.mark)) || t && s <= 0 || e < this.length && o <= 0) ? !1 : (qn(this, t, e, i ? i.children.slice() : [], s - 1, o - 1), this.markDirty(), !0);
  }
  split(t) {
    let e = [], i = 0, n = -1, s = 0;
    for (let l of this.children) {
      let h = i + l.length;
      h > t && e.push(i < t ? l.split(t - i) : l), n < 0 && i >= t && (n = s), i = h, s++;
    }
    let o = this.length - t;
    return this.length = t, n > -1 && (this.children.length = n, this.markDirty()), new ht(this.mark, e, o);
  }
  domAtPos(t) {
    return jn(this, t);
  }
  coordsAt(t, e) {
    return Gn(this, t, e);
  }
}
function nr(r, t, e) {
  let i = r.nodeValue.length;
  t > i && (t = i);
  let n = t, s = t, o = 0;
  t == 0 && e < 0 || t == i && e >= 0 ? w.chrome || w.gecko || (t ? (n--, o = 1) : s < i && (s++, o = -1)) : e < 0 ? n-- : s < i && s++;
  let l = yt(r, n, s).getClientRects();
  if (!l.length)
    return null;
  let h = l[(o ? o < 0 : e >= 0) ? 0 : l.length - 1];
  return w.safari && !o && h.width == 0 && (h = Array.prototype.find.call(l, (f) => f.width) || h), o ? ci(h, o < 0) : h || null;
}
class pt extends M {
  static create(t, e, i) {
    return new pt(t, e, i);
  }
  constructor(t, e, i) {
    super(), this.widget = t, this.length = e, this.side = i, this.prevWidget = null;
  }
  split(t) {
    let e = pt.create(this.widget, this.length - t, this.side);
    return this.length -= t, e;
  }
  sync(t) {
    (!this.dom || !this.widget.updateDOM(this.dom, t)) && (this.dom && this.prevWidget && this.prevWidget.destroy(this.dom), this.prevWidget = null, this.setDOM(this.widget.toDOM(t)), this.widget.editable || (this.dom.contentEditable = "false"));
  }
  getSide() {
    return this.side;
  }
  merge(t, e, i, n, s, o) {
    return i && (!(i instanceof pt) || !this.widget.compare(i.widget) || t > 0 && s <= 0 || e < this.length && o <= 0) ? !1 : (this.length = t + (i ? i.length : 0) + (this.length - e), !0);
  }
  become(t) {
    return t instanceof pt && t.side == this.side && this.widget.constructor == t.widget.constructor ? (this.widget.compare(t.widget) || this.markDirty(!0), this.dom && !this.prevWidget && (this.prevWidget = this.widget), this.widget = t.widget, this.length = t.length, !0) : !1;
  }
  ignoreMutation() {
    return !0;
  }
  ignoreEvent(t) {
    return this.widget.ignoreEvent(t);
  }
  get overrideDOMText() {
    if (this.length == 0)
      return A.empty;
    let t = this;
    for (; t.parent; )
      t = t.parent;
    let { view: e } = t, i = e && e.state.doc, n = this.posAtStart;
    return i ? i.slice(n, n + this.length) : A.empty;
  }
  domAtPos(t) {
    return (this.length ? t == 0 : this.side > 0) ? V.before(this.dom) : V.after(this.dom, t == this.length);
  }
  domBoundsAround() {
    return null;
  }
  coordsAt(t, e) {
    let i = this.widget.coordsAt(this.dom, t, e);
    if (i)
      return i;
    let n = this.dom.getClientRects(), s = null;
    if (!n.length)
      return null;
    let o = this.side ? this.side < 0 : t > 0;
    for (let l = o ? n.length - 1 : 0; s = n[l], !(t > 0 ? l == 0 : l == n.length - 1 || s.top < s.bottom); l += o ? -1 : 1)
      ;
    return ci(s, !o);
  }
  get isEditable() {
    return !1;
  }
  get isWidget() {
    return !0;
  }
  get isHidden() {
    return this.widget.isHidden;
  }
  destroy() {
    super.destroy(), this.dom && this.widget.destroy(this.dom);
  }
}
class Ot extends M {
  constructor(t) {
    super(), this.side = t;
  }
  get length() {
    return 0;
  }
  merge() {
    return !1;
  }
  become(t) {
    return t instanceof Ot && t.side == this.side;
  }
  split() {
    return new Ot(this.side);
  }
  sync() {
    if (!this.dom) {
      let t = document.createElement("img");
      t.className = "cm-widgetBuffer", t.setAttribute("aria-hidden", "true"), this.setDOM(t);
    }
  }
  getSide() {
    return this.side;
  }
  domAtPos(t) {
    return this.side > 0 ? V.before(this.dom) : V.after(this.dom);
  }
  localPosFromDOM() {
    return 0;
  }
  domBoundsAround() {
    return null;
  }
  coordsAt(t) {
    return this.dom.getBoundingClientRect();
  }
  get overrideDOMText() {
    return A.empty;
  }
  get isHidden() {
    return !0;
  }
}
J.prototype.children = pt.prototype.children = Ot.prototype.children = ui;
function jn(r, t) {
  let e = r.dom, { children: i } = r, n = 0;
  for (let s = 0; n < i.length; n++) {
    let o = i[n], l = s + o.length;
    if (!(l == s && o.getSide() <= 0)) {
      if (t > s && t < l && o.dom.parentNode == e)
        return o.domAtPos(t - s);
      if (t <= s)
        break;
      s = l;
    }
  }
  for (let s = n; s > 0; s--) {
    let o = i[s - 1];
    if (o.dom.parentNode == e)
      return o.domAtPos(o.length);
  }
  for (let s = n; s < i.length; s++) {
    let o = i[s];
    if (o.dom.parentNode == e)
      return o.domAtPos(0);
  }
  return new V(e, 0);
}
function Yn(r, t, e) {
  let i, { children: n } = r;
  e > 0 && t instanceof ht && n.length && (i = n[n.length - 1]) instanceof ht && i.mark.eq(t.mark) ? Yn(i, t.children[0], e - 1) : (n.push(t), t.setParent(r)), r.length += t.length;
}
function Gn(r, t, e) {
  let i = null, n = -1, s = null, o = -1;
  function l(f, a) {
    for (let c = 0, u = 0; c < f.children.length && u <= a; c++) {
      let d = f.children[c], g = u + d.length;
      g >= a && (d.children.length ? l(d, a - u) : (!s || s.isHidden && e > 0) && (g > a || u == g && d.getSide() > 0) ? (s = d, o = a - u) : (u < a || u == g && d.getSide() < 0 && !d.isHidden) && (i = d, n = a - u)), u = g;
    }
  }
  l(r, t);
  let h = (e < 0 ? i : s) || i || s;
  return h ? h.coordsAt(Math.max(0, h == i ? n : o), e) : sr(r);
}
function sr(r) {
  let t = r.dom.lastChild;
  if (!t)
    return r.dom.getBoundingClientRect();
  let e = Ut(t);
  return e[e.length - 1] || null;
}
function Je(r, t) {
  for (let e in r)
    e == "class" && t.class ? t.class += " " + r.class : e == "style" && t.style ? t.style += ";" + r.style : t[e] = r[e];
  return t;
}
const Ni = /* @__PURE__ */ Object.create(null);
function we(r, t, e) {
  if (r == t)
    return !0;
  r || (r = Ni), t || (t = Ni);
  let i = Object.keys(r), n = Object.keys(t);
  if (i.length - (e && i.indexOf(e) > -1 ? 1 : 0) != n.length - (e && n.indexOf(e) > -1 ? 1 : 0))
    return !1;
  for (let s of i)
    if (s != e && (n.indexOf(s) == -1 || r[s] !== t[s]))
      return !1;
  return !0;
}
function Qe(r, t, e) {
  let i = !1;
  if (t)
    for (let n in t)
      e && n in e || (i = !0, n == "style" ? r.style.cssText = "" : r.removeAttribute(n));
  if (e)
    for (let n in e)
      t && t[n] == e[n] || (i = !0, n == "style" ? r.style.cssText = e[n] : r.setAttribute(n, e[n]));
  return i;
}
function rr(r) {
  let t = /* @__PURE__ */ Object.create(null);
  for (let e = 0; e < r.attributes.length; e++) {
    let i = r.attributes[e];
    t[i.name] = i.value;
  }
  return t;
}
class B extends M {
  constructor() {
    super(...arguments), this.children = [], this.length = 0, this.prevAttrs = void 0, this.attrs = null, this.breakAfter = 0;
  }
  // Consumes source
  merge(t, e, i, n, s, o) {
    if (i) {
      if (!(i instanceof B))
        return !1;
      this.dom || i.transferDOM(this);
    }
    return n && this.setDeco(i ? i.attrs : null), qn(this, t, e, i ? i.children.slice() : [], s, o), !0;
  }
  split(t) {
    let e = new B();
    if (e.breakAfter = this.breakAfter, this.length == 0)
      return e;
    let { i, off: n } = this.childPos(t);
    n && (e.append(this.children[i].split(n), 0), this.children[i].merge(n, this.children[i].length, null, !1, 0, 0), i++);
    for (let s = i; s < this.children.length; s++)
      e.append(this.children[s], 0);
    for (; i > 0 && this.children[i - 1].length == 0; )
      this.children[--i].destroy();
    return this.children.length = i, this.markDirty(), this.length = t, e;
  }
  transferDOM(t) {
    this.dom && (this.markDirty(), t.setDOM(this.dom), t.prevAttrs = this.prevAttrs === void 0 ? this.attrs : this.prevAttrs, this.prevAttrs = void 0, this.dom = null);
  }
  setDeco(t) {
    we(this.attrs, t) || (this.dom && (this.prevAttrs = this.attrs, this.markDirty()), this.attrs = t);
  }
  append(t, e) {
    Yn(this, t, e);
  }
  // Only called when building a line view in ContentBuilder
  addLineDeco(t) {
    let e = t.spec.attributes, i = t.spec.class;
    e && (this.attrs = Je(e, this.attrs || {})), i && (this.attrs = Je({ class: i }, this.attrs || {}));
  }
  domAtPos(t) {
    return jn(this, t);
  }
  reuseDOM(t) {
    t.nodeName == "DIV" && (this.setDOM(t), this.flags |= 6);
  }
  sync(t, e) {
    var i;
    this.dom ? this.flags & 4 && (In(this.dom), this.dom.className = "cm-line", this.prevAttrs = this.attrs ? null : void 0) : (this.setDOM(document.createElement("div")), this.dom.className = "cm-line", this.prevAttrs = this.attrs ? null : void 0), this.prevAttrs !== void 0 && (Qe(this.dom, this.prevAttrs, this.attrs), this.dom.classList.add("cm-line"), this.prevAttrs = void 0), super.sync(t, e);
    let n = this.dom.lastChild;
    for (; n && M.get(n) instanceof ht; )
      n = n.lastChild;
    if (!n || !this.length || n.nodeName != "BR" && ((i = M.get(n)) === null || i === void 0 ? void 0 : i.isEditable) == !1 && (!w.ios || !this.children.some((s) => s instanceof J))) {
      let s = document.createElement("BR");
      s.cmIgnore = !0, this.dom.appendChild(s);
    }
  }
  measureTextSize() {
    if (this.children.length == 0 || this.length > 20)
      return null;
    let t = 0, e;
    for (let i of this.children) {
      if (!(i instanceof J) || /[^ -~]/.test(i.text))
        return null;
      let n = Ut(i.dom);
      if (n.length != 1)
        return null;
      t += n[0].width, e = n[0].height;
    }
    return t ? {
      lineHeight: this.dom.getBoundingClientRect().height,
      charWidth: t / this.length,
      textHeight: e
    } : null;
  }
  coordsAt(t, e) {
    let i = Gn(this, t, e);
    if (!this.children.length && i && this.parent) {
      let { heightOracle: n } = this.parent.view.viewState, s = i.bottom - i.top;
      if (Math.abs(s - n.lineHeight) < 2 && n.textHeight < s) {
        let o = (s - n.textHeight) / 2;
        return { top: i.top + o, bottom: i.bottom - o, left: i.left, right: i.left };
      }
    }
    return i;
  }
  become(t) {
    return t instanceof B && this.children.length == 0 && t.children.length == 0 && we(this.attrs, t.attrs) && this.breakAfter == t.breakAfter;
  }
  covers() {
    return !0;
  }
  static find(t, e) {
    for (let i = 0, n = 0; i < t.children.length; i++) {
      let s = t.children[i], o = n + s.length;
      if (o >= e) {
        if (s instanceof B)
          return s;
        if (o > e)
          break;
      }
      n = o + s.breakAfter;
    }
    return null;
  }
}
class ot extends M {
  constructor(t, e, i) {
    super(), this.widget = t, this.length = e, this.deco = i, this.breakAfter = 0, this.prevWidget = null;
  }
  merge(t, e, i, n, s, o) {
    return i && (!(i instanceof ot) || !this.widget.compare(i.widget) || t > 0 && s <= 0 || e < this.length && o <= 0) ? !1 : (this.length = t + (i ? i.length : 0) + (this.length - e), !0);
  }
  domAtPos(t) {
    return t == 0 ? V.before(this.dom) : V.after(this.dom, t == this.length);
  }
  split(t) {
    let e = this.length - t;
    this.length = t;
    let i = new ot(this.widget, e, this.deco);
    return i.breakAfter = this.breakAfter, i;
  }
  get children() {
    return ui;
  }
  sync(t) {
    (!this.dom || !this.widget.updateDOM(this.dom, t)) && (this.dom && this.prevWidget && this.prevWidget.destroy(this.dom), this.prevWidget = null, this.setDOM(this.widget.toDOM(t)), this.widget.editable || (this.dom.contentEditable = "false"));
  }
  get overrideDOMText() {
    return this.parent ? this.parent.view.state.doc.slice(this.posAtStart, this.posAtEnd) : A.empty;
  }
  domBoundsAround() {
    return null;
  }
  become(t) {
    return t instanceof ot && t.widget.constructor == this.widget.constructor ? (t.widget.compare(this.widget) || this.markDirty(!0), this.dom && !this.prevWidget && (this.prevWidget = this.widget), this.widget = t.widget, this.length = t.length, this.deco = t.deco, this.breakAfter = t.breakAfter, !0) : !1;
  }
  ignoreMutation() {
    return !0;
  }
  ignoreEvent(t) {
    return this.widget.ignoreEvent(t);
  }
  get isEditable() {
    return !1;
  }
  get isWidget() {
    return !0;
  }
  coordsAt(t, e) {
    return this.widget.coordsAt(this.dom, t, e);
  }
  destroy() {
    super.destroy(), this.dom && this.widget.destroy(this.dom);
  }
  covers(t) {
    let { startSide: e, endSide: i } = this.deco;
    return e == i ? !1 : t < 0 ? e < 0 : i > 0;
  }
}
class di {
  /**
  Compare this instance to another instance of the same type.
  (TypeScript can't express this, but only instances of the same
  specific class will be passed to this method.) This is used to
  avoid redrawing widgets when they are replaced by a new
  decoration of the same type. The default implementation just
  returns `false`, which will cause new instances of the widget to
  always be redrawn.
  */
  eq(t) {
    return !1;
  }
  /**
  Update a DOM element created by a widget of the same type (but
  different, non-`eq` content) to reflect this widget. May return
  true to indicate that it could update, false to indicate it
  couldn't (in which case the widget will be redrawn). The default
  implementation just returns false.
  */
  updateDOM(t, e) {
    return !1;
  }
  /**
  @internal
  */
  compare(t) {
    return this == t || this.constructor == t.constructor && this.eq(t);
  }
  /**
  The estimated height this widget will have, to be used when
  estimating the height of content that hasn't been drawn. May
  return -1 to indicate you don't know. The default implementation
  returns -1.
  */
  get estimatedHeight() {
    return -1;
  }
  /**
  For inline widgets that are displayed inline (as opposed to
  `inline-block`) and introduce line breaks (through `<br>` tags
  or textual newlines), this must indicate the amount of line
  breaks they introduce. Defaults to 0.
  */
  get lineBreaks() {
    return 0;
  }
  /**
  Can be used to configure which kinds of events inside the widget
  should be ignored by the editor. The default is to ignore all
  events.
  */
  ignoreEvent(t) {
    return !0;
  }
  /**
  Override the way screen coordinates for positions at/in the
  widget are found. `pos` will be the offset into the widget, and
  `side` the side of the position that is being queried—less than
  zero for before, greater than zero for after, and zero for
  directly at that position.
  */
  coordsAt(t, e, i) {
    return null;
  }
  /**
  @internal
  */
  get isHidden() {
    return !1;
  }
  /**
  @internal
  */
  get editable() {
    return !1;
  }
  /**
  This is called when the an instance of the widget is removed
  from the editor view.
  */
  destroy(t) {
  }
}
var et = /* @__PURE__ */ function(r) {
  return r[r.Text = 0] = "Text", r[r.WidgetBefore = 1] = "WidgetBefore", r[r.WidgetAfter = 2] = "WidgetAfter", r[r.WidgetRange = 3] = "WidgetRange", r;
}(et || (et = {}));
class Y extends Ct {
  constructor(t, e, i, n) {
    super(), this.startSide = t, this.endSide = e, this.widget = i, this.spec = n;
  }
  /**
  @internal
  */
  get heightRelevant() {
    return !1;
  }
  /**
  Create a mark decoration, which influences the styling of the
  content in its range. Nested mark decorations will cause nested
  DOM elements to be created. Nesting order is determined by
  precedence of the [facet](https://codemirror.net/6/docs/ref/#view.EditorView^decorations), with
  the higher-precedence decorations creating the inner DOM nodes.
  Such elements are split on line boundaries and on the boundaries
  of lower-precedence decorations.
  */
  static mark(t) {
    return new Qt(t);
  }
  /**
  Create a widget decoration, which displays a DOM element at the
  given position.
  */
  static widget(t) {
    let e = Math.max(-1e4, Math.min(1e4, t.side || 0)), i = !!t.block;
    return e += i && !t.inlineOrder ? e > 0 ? 3e8 : -4e8 : e > 0 ? 1e8 : -1e8, new ut(t, e, e, i, t.widget || null, !1);
  }
  /**
  Create a replace decoration which replaces the given range with
  a widget, or simply hides it.
  */
  static replace(t) {
    let e = !!t.block, i, n;
    if (t.isBlockGap)
      i = -5e8, n = 4e8;
    else {
      let { start: s, end: o } = Un(t, e);
      i = (s ? e ? -3e8 : -1 : 5e8) - 1, n = (o ? e ? 2e8 : 1 : -6e8) + 1;
    }
    return new ut(t, i, n, e, t.widget || null, !0);
  }
  /**
  Create a line decoration, which can add DOM attributes to the
  line starting at the given position.
  */
  static line(t) {
    return new Zt(t);
  }
  /**
  Build a [`DecorationSet`](https://codemirror.net/6/docs/ref/#view.DecorationSet) from the given
  decorated range or ranges. If the ranges aren't already sorted,
  pass `true` for `sort` to make the library sort them for you.
  */
  static set(t, e = !1) {
    return R.of(t, e);
  }
  /**
  @internal
  */
  hasHeight() {
    return this.widget ? this.widget.estimatedHeight > -1 : !1;
  }
}
Y.none = R.empty;
class Qt extends Y {
  constructor(t) {
    let { start: e, end: i } = Un(t);
    super(e ? -1 : 5e8, i ? 1 : -6e8, null, t), this.tagName = t.tagName || "span", this.class = t.class || "", this.attrs = t.attributes || null;
  }
  eq(t) {
    var e, i;
    return this == t || t instanceof Qt && this.tagName == t.tagName && (this.class || ((e = this.attrs) === null || e === void 0 ? void 0 : e.class)) == (t.class || ((i = t.attrs) === null || i === void 0 ? void 0 : i.class)) && we(this.attrs, t.attrs, "class");
  }
  range(t, e = t) {
    if (t >= e)
      throw new RangeError("Mark decorations may not be empty");
    return super.range(t, e);
  }
}
Qt.prototype.point = !1;
class Zt extends Y {
  constructor(t) {
    super(-2e8, -2e8, null, t);
  }
  eq(t) {
    return t instanceof Zt && this.spec.class == t.spec.class && we(this.spec.attributes, t.spec.attributes);
  }
  range(t, e = t) {
    if (e != t)
      throw new RangeError("Line decoration ranges must be zero-length");
    return super.range(t, e);
  }
}
Zt.prototype.mapMode = j.TrackBefore;
Zt.prototype.point = !0;
class ut extends Y {
  constructor(t, e, i, n, s, o) {
    super(e, i, s, t), this.block = n, this.isReplace = o, this.mapMode = n ? e <= 0 ? j.TrackBefore : j.TrackAfter : j.TrackDel;
  }
  // Only relevant when this.block == true
  get type() {
    return this.startSide != this.endSide ? et.WidgetRange : this.startSide <= 0 ? et.WidgetBefore : et.WidgetAfter;
  }
  get heightRelevant() {
    return this.block || !!this.widget && (this.widget.estimatedHeight >= 5 || this.widget.lineBreaks > 0);
  }
  eq(t) {
    return t instanceof ut && or(this.widget, t.widget) && this.block == t.block && this.startSide == t.startSide && this.endSide == t.endSide;
  }
  range(t, e = t) {
    if (this.isReplace && (t > e || t == e && this.startSide > 0 && this.endSide <= 0))
      throw new RangeError("Invalid range for replacement decoration");
    if (!this.isReplace && e != t)
      throw new RangeError("Widget decorations can only have zero-length ranges");
    return super.range(t, e);
  }
}
ut.prototype.point = !0;
function Un(r, t = !1) {
  let { inclusiveStart: e, inclusiveEnd: i } = r;
  return e == null && (e = r.inclusive), i == null && (i = r.inclusive), { start: e ?? t, end: i ?? t };
}
function or(r, t) {
  return r == t || !!(r && t && r.compare(t));
}
function Ze(r, t, e, i = 0) {
  let n = e.length - 1;
  n >= 0 && e[n] + i >= r ? e[n] = Math.max(e[n], t) : e.push(r, t);
}
class Kt {
  constructor(t, e, i, n) {
    this.doc = t, this.pos = e, this.end = i, this.disallowBlockEffectsFor = n, this.content = [], this.curLine = null, this.breakAtStart = 0, this.pendingBuffer = 0, this.bufferMarks = [], this.atCursorPos = !0, this.openStart = -1, this.openEnd = -1, this.text = "", this.textOff = 0, this.cursor = t.iter(), this.skip = e;
  }
  posCovered() {
    if (this.content.length == 0)
      return !this.breakAtStart && this.doc.lineAt(this.pos).from != this.pos;
    let t = this.content[this.content.length - 1];
    return !(t.breakAfter || t instanceof ot && t.deco.endSide < 0);
  }
  getLine() {
    return this.curLine || (this.content.push(this.curLine = new B()), this.atCursorPos = !0), this.curLine;
  }
  flushBuffer(t = this.bufferMarks) {
    this.pendingBuffer && (this.curLine.append(ee(new Ot(-1), t), t.length), this.pendingBuffer = 0);
  }
  addBlockWidget(t) {
    this.flushBuffer(), this.curLine = null, this.content.push(t);
  }
  finish(t) {
    this.pendingBuffer && t <= this.bufferMarks.length ? this.flushBuffer() : this.pendingBuffer = 0, !this.posCovered() && !(t && this.content.length && this.content[this.content.length - 1] instanceof ot) && this.getLine();
  }
  buildText(t, e, i) {
    for (; t > 0; ) {
      if (this.textOff == this.text.length) {
        let { value: s, lineBreak: o, done: l } = this.cursor.next(this.skip);
        if (this.skip = 0, l)
          throw new Error("Ran out of text content when drawing inline views");
        if (o) {
          this.posCovered() || this.getLine(), this.content.length ? this.content[this.content.length - 1].breakAfter = 1 : this.breakAtStart = 1, this.flushBuffer(), this.curLine = null, this.atCursorPos = !0, t--;
          continue;
        } else
          this.text = s, this.textOff = 0;
      }
      let n = Math.min(
        this.text.length - this.textOff,
        t,
        512
        /* T.Chunk */
      );
      this.flushBuffer(e.slice(e.length - i)), this.getLine().append(ee(new J(this.text.slice(this.textOff, this.textOff + n)), e), i), this.atCursorPos = !0, this.textOff += n, t -= n, i = 0;
    }
  }
  span(t, e, i, n) {
    this.buildText(e - t, i, n), this.pos = e, this.openStart < 0 && (this.openStart = n);
  }
  point(t, e, i, n, s, o) {
    if (this.disallowBlockEffectsFor[o] && i instanceof ut) {
      if (i.block)
        throw new RangeError("Block decorations may not be specified via plugins");
      if (e > this.doc.lineAt(this.pos).to)
        throw new RangeError("Decorations that replace line breaks may not be specified via plugins");
    }
    let l = e - t;
    if (i instanceof ut)
      if (i.block)
        i.startSide > 0 && !this.posCovered() && this.getLine(), this.addBlockWidget(new ot(i.widget || Tt.block, l, i));
      else {
        let h = pt.create(i.widget || Tt.inline, l, l ? 0 : i.startSide), f = this.atCursorPos && !h.isEditable && s <= n.length && (t < e || i.startSide > 0), a = !h.isEditable && (t < e || s > n.length || i.startSide <= 0), c = this.getLine();
        this.pendingBuffer == 2 && !f && !h.isEditable && (this.pendingBuffer = 0), this.flushBuffer(n), f && (c.append(ee(new Ot(1), n), s), s = n.length + Math.max(0, s - n.length)), c.append(ee(h, n), s), this.atCursorPos = a, this.pendingBuffer = a ? t < e || s > n.length ? 1 : 2 : 0, this.pendingBuffer && (this.bufferMarks = n.slice());
      }
    else this.doc.lineAt(this.pos).from == this.pos && this.getLine().addLineDeco(i);
    l && (this.textOff + l <= this.text.length ? this.textOff += l : (this.skip += l - (this.text.length - this.textOff), this.text = "", this.textOff = 0), this.pos = e), this.openStart < 0 && (this.openStart = s);
  }
  static build(t, e, i, n, s) {
    let o = new Kt(t, e, i, s);
    return o.openEnd = R.spans(n, e, i, o), o.openStart < 0 && (o.openStart = o.openEnd), o.finish(o.openEnd), o;
  }
}
function ee(r, t) {
  for (let e of t)
    r = new ht(e, [r], r.length);
  return r;
}
class Tt extends di {
  constructor(t) {
    super(), this.tag = t;
  }
  eq(t) {
    return t.tag == this.tag;
  }
  toDOM() {
    return document.createElement(this.tag);
  }
  updateDOM(t) {
    return t.nodeName.toLowerCase() == this.tag;
  }
  get isHidden() {
    return !0;
  }
}
Tt.inline = /* @__PURE__ */ new Tt("span");
Tt.block = /* @__PURE__ */ new Tt("div");
var z = /* @__PURE__ */ function(r) {
  return r[r.LTR = 0] = "LTR", r[r.RTL = 1] = "RTL", r;
}(z || (z = {}));
const wt = z.LTR, gi = z.RTL;
function Xn(r) {
  let t = [];
  for (let e = 0; e < r.length; e++)
    t.push(1 << +r[e]);
  return t;
}
const lr = /* @__PURE__ */ Xn("88888888888888888888888888888888888666888888787833333333337888888000000000000000000000000008888880000000000000000000000000088888888888888888888888888888888888887866668888088888663380888308888800000000000000000000000800000000000000000000000000000008"), hr = /* @__PURE__ */ Xn("4444448826627288999999999992222222222222222222222222222222222222222222222229999999999999999999994444444444644222822222222222222222222222222222222222222222222222222222222222222222222222222222222222222222222222222222999999949999999229989999223333333333"), _e = /* @__PURE__ */ Object.create(null), Z = [];
for (let r of ["()", "[]", "{}"]) {
  let t = /* @__PURE__ */ r.charCodeAt(0), e = /* @__PURE__ */ r.charCodeAt(1);
  _e[t] = e, _e[e] = -t;
}
function Jn(r) {
  return r <= 247 ? lr[r] : 1424 <= r && r <= 1524 ? 2 : 1536 <= r && r <= 1785 ? hr[r - 1536] : 1774 <= r && r <= 2220 ? 4 : 8192 <= r && r <= 8204 ? 256 : 64336 <= r && r <= 65023 ? 4 : 1;
}
const ar = /[\u0590-\u05f4\u0600-\u06ff\u0700-\u08ac\ufb50-\ufdff]/;
class ct {
  /**
  The direction of this span.
  */
  get dir() {
    return this.level % 2 ? gi : wt;
  }
  /**
  @internal
  */
  constructor(t, e, i) {
    this.from = t, this.to = e, this.level = i;
  }
  /**
  @internal
  */
  side(t, e) {
    return this.dir == e == t ? this.to : this.from;
  }
  /**
  @internal
  */
  forward(t, e) {
    return t == (this.dir == e);
  }
  /**
  @internal
  */
  static find(t, e, i, n) {
    let s = -1;
    for (let o = 0; o < t.length; o++) {
      let l = t[o];
      if (l.from <= e && l.to >= e) {
        if (l.level == i)
          return o;
        (s < 0 || (n != 0 ? n < 0 ? l.from < e : l.to > e : t[s].level > l.level)) && (s = o);
      }
    }
    if (s < 0)
      throw new RangeError("Index out of range");
    return s;
  }
}
function Qn(r, t) {
  if (r.length != t.length)
    return !1;
  for (let e = 0; e < r.length; e++) {
    let i = r[e], n = t[e];
    if (i.from != n.from || i.to != n.to || i.direction != n.direction || !Qn(i.inner, n.inner))
      return !1;
  }
  return !0;
}
const C = [];
function fr(r, t, e, i, n) {
  for (let s = 0; s <= i.length; s++) {
    let o = s ? i[s - 1].to : t, l = s < i.length ? i[s].from : e, h = s ? 256 : n;
    for (let f = o, a = h, c = h; f < l; f++) {
      let u = Jn(r.charCodeAt(f));
      u == 512 ? u = a : u == 8 && c == 4 && (u = 16), C[f] = u == 4 ? 2 : u, u & 7 && (c = u), a = u;
    }
    for (let f = o, a = h, c = h; f < l; f++) {
      let u = C[f];
      if (u == 128)
        f < l - 1 && a == C[f + 1] && a & 24 ? u = C[f] = a : C[f] = 256;
      else if (u == 64) {
        let d = f + 1;
        for (; d < l && C[d] == 64; )
          d++;
        let g = f && a == 8 || d < e && C[d] == 8 ? c == 1 ? 1 : 8 : 256;
        for (let p = f; p < d; p++)
          C[p] = g;
        f = d - 1;
      } else u == 8 && c == 1 && (C[f] = 1);
      a = u, u & 7 && (c = u);
    }
  }
}
function cr(r, t, e, i, n) {
  let s = n == 1 ? 2 : 1;
  for (let o = 0, l = 0, h = 0; o <= i.length; o++) {
    let f = o ? i[o - 1].to : t, a = o < i.length ? i[o].from : e;
    for (let c = f, u, d, g; c < a; c++)
      if (d = _e[u = r.charCodeAt(c)])
        if (d < 0) {
          for (let p = l - 3; p >= 0; p -= 3)
            if (Z[p + 1] == -d) {
              let m = Z[p + 2], y = m & 2 ? n : m & 4 ? m & 1 ? s : n : 0;
              y && (C[c] = C[Z[p]] = y), l = p;
              break;
            }
        } else {
          if (Z.length == 189)
            break;
          Z[l++] = c, Z[l++] = u, Z[l++] = h;
        }
      else if ((g = C[c]) == 2 || g == 1) {
        let p = g == n;
        h = p ? 0 : 1;
        for (let m = l - 3; m >= 0; m -= 3) {
          let y = Z[m + 2];
          if (y & 2)
            break;
          if (p)
            Z[m + 2] |= 2;
          else {
            if (y & 4)
              break;
            Z[m + 2] |= 4;
          }
        }
      }
  }
}
function ur(r, t, e, i) {
  for (let n = 0, s = i; n <= e.length; n++) {
    let o = n ? e[n - 1].to : r, l = n < e.length ? e[n].from : t;
    for (let h = o; h < l; ) {
      let f = C[h];
      if (f == 256) {
        let a = h + 1;
        for (; ; )
          if (a == l) {
            if (n == e.length)
              break;
            a = e[n++].to, l = n < e.length ? e[n].from : t;
          } else if (C[a] == 256)
            a++;
          else
            break;
        let c = s == 1, u = (a < t ? C[a] : i) == 1, d = c == u ? c ? 1 : 2 : i;
        for (let g = a, p = n, m = p ? e[p - 1].to : r; g > h; )
          g == m && (g = e[--p].from, m = p ? e[p - 1].to : r), C[--g] = d;
        h = a;
      } else
        s = f, h++;
    }
  }
}
function ti(r, t, e, i, n, s, o) {
  let l = i % 2 ? 2 : 1;
  if (i % 2 == n % 2)
    for (let h = t, f = 0; h < e; ) {
      let a = !0, c = !1;
      if (f == s.length || h < s[f].from) {
        let p = C[h];
        p != l && (a = !1, c = p == 16);
      }
      let u = !a && l == 1 ? [] : null, d = a ? i : i + 1, g = h;
      t: for (; ; )
        if (f < s.length && g == s[f].from) {
          if (c)
            break t;
          let p = s[f];
          if (!a)
            for (let m = p.to, y = f + 1; ; ) {
              if (m == e)
                break t;
              if (y < s.length && s[y].from == m)
                m = s[y++].to;
              else {
                if (C[m] == l)
                  break t;
                break;
              }
            }
          if (f++, u)
            u.push(p);
          else {
            p.from > h && o.push(new ct(h, p.from, d));
            let m = p.direction == wt != !(d % 2);
            ei(r, m ? i + 1 : i, n, p.inner, p.from, p.to, o), h = p.to;
          }
          g = p.to;
        } else {
          if (g == e || (a ? C[g] != l : C[g] == l))
            break;
          g++;
        }
      u ? ti(r, h, g, i + 1, n, u, o) : h < g && o.push(new ct(h, g, d)), h = g;
    }
  else
    for (let h = e, f = s.length; h > t; ) {
      let a = !0, c = !1;
      if (!f || h > s[f - 1].to) {
        let p = C[h - 1];
        p != l && (a = !1, c = p == 16);
      }
      let u = !a && l == 1 ? [] : null, d = a ? i : i + 1, g = h;
      t: for (; ; )
        if (f && g == s[f - 1].to) {
          if (c)
            break t;
          let p = s[--f];
          if (!a)
            for (let m = p.from, y = f; ; ) {
              if (m == t)
                break t;
              if (y && s[y - 1].to == m)
                m = s[--y].from;
              else {
                if (C[m - 1] == l)
                  break t;
                break;
              }
            }
          if (u)
            u.push(p);
          else {
            p.to < h && o.push(new ct(p.to, h, d));
            let m = p.direction == wt != !(d % 2);
            ei(r, m ? i + 1 : i, n, p.inner, p.from, p.to, o), h = p.from;
          }
          g = p.from;
        } else {
          if (g == t || (a ? C[g - 1] != l : C[g - 1] == l))
            break;
          g--;
        }
      u ? ti(r, g, h, i + 1, n, u, o) : g < h && o.push(new ct(g, h, d)), h = g;
    }
}
function ei(r, t, e, i, n, s, o) {
  let l = t % 2 ? 2 : 1;
  fr(r, n, s, i, l), cr(r, n, s, i, l), ur(n, s, i, l), ti(r, n, s, t, e, i, o);
}
function dr(r, t, e) {
  if (!r)
    return [new ct(0, 0, t == gi ? 1 : 0)];
  if (t == wt && !e.length && !ar.test(r))
    return Zn(r.length);
  if (e.length)
    for (; r.length > C.length; )
      C[C.length] = 256;
  let i = [], n = t == wt ? 0 : 1;
  return ei(r, n, n, e, 0, r.length, i), i;
}
function Zn(r) {
  return [new ct(0, r, 0)];
}
let _n = "";
function gr(r, t, e, i, n) {
  var s;
  let o = i.head - r.from, l = ct.find(t, o, (s = i.bidiLevel) !== null && s !== void 0 ? s : -1, i.assoc), h = t[l], f = h.side(n, e);
  if (o == f) {
    let u = l += n ? 1 : -1;
    if (u < 0 || u >= t.length)
      return null;
    h = t[l = u], o = h.side(!n, e), f = h.side(n, e);
  }
  let a = it(r.text, o, h.forward(n, e));
  (a < h.from || a > h.to) && (a = f), _n = r.text.slice(Math.min(o, a), Math.max(o, a));
  let c = l == (n ? t.length - 1 : 0) ? null : t[l + (n ? 1 : -1)];
  return c && a == f && c.level + (n ? 0 : 1) < h.level ? b.cursor(c.side(!n, e) + r.from, c.forward(n, e) ? 1 : -1, c.level) : b.cursor(a + r.from, h.forward(n, e) ? -1 : 1, h.level);
}
function pr(r, t, e) {
  for (let i = t; i < e; i++) {
    let n = Jn(r.charCodeAt(i));
    if (n == 1)
      return wt;
    if (n == 2 || n == 4)
      return gi;
  }
  return wt;
}
const ts = /* @__PURE__ */ v.define(), es = /* @__PURE__ */ v.define(), is = /* @__PURE__ */ v.define(), ns = /* @__PURE__ */ v.define(), ii = /* @__PURE__ */ v.define(), ss = /* @__PURE__ */ v.define(), rs = /* @__PURE__ */ v.define(), os = /* @__PURE__ */ v.define({
  combine: (r) => r.some((t) => t)
}), mr = /* @__PURE__ */ v.define({
  combine: (r) => r.some((t) => t)
}), ls = /* @__PURE__ */ v.define();
class kt {
  constructor(t, e = "nearest", i = "nearest", n = 5, s = 5, o = !1) {
    this.range = t, this.y = e, this.x = i, this.yMargin = n, this.xMargin = s, this.isSnapshot = o;
  }
  map(t) {
    return t.empty ? this : new kt(this.range.map(t), this.y, this.x, this.yMargin, this.xMargin, this.isSnapshot);
  }
  clip(t) {
    return this.range.to <= t.doc.length ? this : new kt(b.cursor(t.doc.length), this.y, this.x, this.yMargin, this.xMargin, this.isSnapshot);
  }
}
const ie = /* @__PURE__ */ F.define({ map: (r, t) => r.map(t) }), hs = /* @__PURE__ */ F.define();
function st(r, t, e) {
  let i = r.facet(ns);
  i.length ? i[0](t) : window.onerror ? window.onerror(String(t), e, void 0, void 0, t) : e ? console.error(e + ":", t) : console.error(t);
}
const at = /* @__PURE__ */ v.define({ combine: (r) => r.length ? r[0] : !0 });
let yr = 0;
const It = /* @__PURE__ */ v.define();
class Xt {
  constructor(t, e, i, n, s) {
    this.id = t, this.create = e, this.domEventHandlers = i, this.domEventObservers = n, this.extension = s(this);
  }
  /**
  Define a plugin from a constructor function that creates the
  plugin's value, given an editor view.
  */
  static define(t, e) {
    const { eventHandlers: i, eventObservers: n, provide: s, decorations: o } = e || {};
    return new Xt(yr++, t, i, n, (l) => {
      let h = [It.of(l)];
      return o && h.push(Jt.of((f) => {
        let a = f.plugin(l);
        return a ? o(a) : Y.none;
      })), s && h.push(s(l)), h;
    });
  }
  /**
  Create a plugin for a class whose constructor takes a single
  editor view as argument.
  */
  static fromClass(t, e) {
    return Xt.define((i) => new t(i), e);
  }
}
class Oe {
  constructor(t) {
    this.spec = t, this.mustUpdate = null, this.value = null;
  }
  update(t) {
    if (this.value) {
      if (this.mustUpdate) {
        let e = this.mustUpdate;
        if (this.mustUpdate = null, this.value.update)
          try {
            this.value.update(e);
          } catch (i) {
            if (st(e.state, i, "CodeMirror plugin crashed"), this.value.destroy)
              try {
                this.value.destroy();
              } catch {
              }
            this.deactivate();
          }
      }
    } else if (this.spec)
      try {
        this.value = this.spec.create(t);
      } catch (e) {
        st(t.state, e, "CodeMirror plugin crashed"), this.deactivate();
      }
    return this;
  }
  destroy(t) {
    var e;
    if (!((e = this.value) === null || e === void 0) && e.destroy)
      try {
        this.value.destroy();
      } catch (i) {
        st(t.state, i, "CodeMirror plugin crashed");
      }
  }
  deactivate() {
    this.spec = this.value = null;
  }
}
const as = /* @__PURE__ */ v.define(), pi = /* @__PURE__ */ v.define(), Jt = /* @__PURE__ */ v.define(), fs = /* @__PURE__ */ v.define(), mi = /* @__PURE__ */ v.define(), cs = /* @__PURE__ */ v.define();
function Hi(r, t) {
  let e = r.state.facet(cs);
  if (!e.length)
    return e;
  let i = e.map((s) => s instanceof Function ? s(r) : s), n = [];
  return R.spans(i, t.from, t.to, {
    point() {
    },
    span(s, o, l, h) {
      let f = s - t.from, a = o - t.from, c = n;
      for (let u = l.length - 1; u >= 0; u--, h--) {
        let d = l[u].spec.bidiIsolate, g;
        if (d == null && (d = pr(t.text, f, a)), h > 0 && c.length && (g = c[c.length - 1]).to == f && g.direction == d)
          g.to = a, c = g.inner;
        else {
          let p = { from: f, to: a, direction: d, inner: [] };
          c.push(p), c = p.inner;
        }
      }
    }
  }), n;
}
const us = /* @__PURE__ */ v.define();
function ds(r) {
  let t = 0, e = 0, i = 0, n = 0;
  for (let s of r.state.facet(us)) {
    let o = s(r);
    o && (o.left != null && (t = Math.max(t, o.left)), o.right != null && (e = Math.max(e, o.right)), o.top != null && (i = Math.max(i, o.top)), o.bottom != null && (n = Math.max(n, o.bottom)));
  }
  return { left: t, right: e, top: i, bottom: n };
}
const Nt = /* @__PURE__ */ v.define();
class U {
  constructor(t, e, i, n) {
    this.fromA = t, this.toA = e, this.fromB = i, this.toB = n;
  }
  join(t) {
    return new U(Math.min(this.fromA, t.fromA), Math.max(this.toA, t.toA), Math.min(this.fromB, t.fromB), Math.max(this.toB, t.toB));
  }
  addToSet(t) {
    let e = t.length, i = this;
    for (; e > 0; e--) {
      let n = t[e - 1];
      if (!(n.fromA > i.toA)) {
        if (n.toA < i.fromA)
          break;
        i = i.join(n), t.splice(e - 1, 1);
      }
    }
    return t.splice(e, 0, i), t;
  }
  static extendWithRanges(t, e) {
    if (e.length == 0)
      return t;
    let i = [];
    for (let n = 0, s = 0, o = 0, l = 0; ; n++) {
      let h = n == t.length ? null : t[n], f = o - l, a = h ? h.fromB : 1e9;
      for (; s < e.length && e[s] < a; ) {
        let c = e[s], u = e[s + 1], d = Math.max(l, c), g = Math.min(a, u);
        if (d <= g && new U(d + f, g + f, d, g).addToSet(i), u > a)
          break;
        s += 2;
      }
      if (!h)
        return i;
      new U(h.fromA, h.toA, h.fromB, h.toB).addToSet(i), o = h.toA, l = h.toB;
    }
  }
}
class be {
  constructor(t, e, i) {
    this.view = t, this.state = e, this.transactions = i, this.flags = 0, this.startState = t.state, this.changes = L.empty(this.startState.doc.length);
    for (let s of i)
      this.changes = this.changes.compose(s.changes);
    let n = [];
    this.changes.iterChangedRanges((s, o, l, h) => n.push(new U(s, o, l, h))), this.changedRanges = n;
  }
  /**
  @internal
  */
  static create(t, e, i) {
    return new be(t, e, i);
  }
  /**
  Tells you whether the [viewport](https://codemirror.net/6/docs/ref/#view.EditorView.viewport) or
  [visible ranges](https://codemirror.net/6/docs/ref/#view.EditorView.visibleRanges) changed in this
  update.
  */
  get viewportChanged() {
    return (this.flags & 4) > 0;
  }
  /**
  Indicates whether the height of a block element in the editor
  changed in this update.
  */
  get heightChanged() {
    return (this.flags & 2) > 0;
  }
  /**
  Returns true when the document was modified or the size of the
  editor, or elements within the editor, changed.
  */
  get geometryChanged() {
    return this.docChanged || (this.flags & 10) > 0;
  }
  /**
  True when this update indicates a focus change.
  */
  get focusChanged() {
    return (this.flags & 1) > 0;
  }
  /**
  Whether the document changed in this update.
  */
  get docChanged() {
    return !this.changes.empty;
  }
  /**
  Whether the selection was explicitly set in this update.
  */
  get selectionSet() {
    return this.transactions.some((t) => t.selection);
  }
  /**
  @internal
  */
  get empty() {
    return this.flags == 0 && this.transactions.length == 0;
  }
}
class Vi extends M {
  get length() {
    return this.view.state.doc.length;
  }
  constructor(t) {
    super(), this.view = t, this.decorations = [], this.dynamicDecorationMap = [!1], this.domChanged = null, this.hasComposition = null, this.markedForComposition = /* @__PURE__ */ new Set(), this.editContextFormatting = Y.none, this.lastCompositionAfterCursor = !1, this.minWidth = 0, this.minWidthFrom = 0, this.minWidthTo = 0, this.impreciseAnchor = null, this.impreciseHead = null, this.forceSelection = !1, this.lastUpdate = Date.now(), this.setDOM(t.contentDOM), this.children = [new B()], this.children[0].setParent(this), this.updateDeco(), this.updateInner([new U(0, 0, 0, t.state.doc.length)], 0, null);
  }
  // Update the document view to a given state.
  update(t) {
    var e;
    let i = t.changedRanges;
    this.minWidth > 0 && i.length && (i.every(({ fromA: f, toA: a }) => a < this.minWidthFrom || f > this.minWidthTo) ? (this.minWidthFrom = t.changes.mapPos(this.minWidthFrom, 1), this.minWidthTo = t.changes.mapPos(this.minWidthTo, 1)) : this.minWidth = this.minWidthFrom = this.minWidthTo = 0), this.updateEditContextFormatting(t);
    let n = -1;
    this.view.inputState.composing >= 0 && !this.view.observer.editContext && (!((e = this.domChanged) === null || e === void 0) && e.newSel ? n = this.domChanged.newSel.head : !Ar(t.changes, this.hasComposition) && !t.selectionSet && (n = t.state.selection.main.head));
    let s = n > -1 ? br(this.view, t.changes, n) : null;
    if (this.domChanged = null, this.hasComposition) {
      this.markedForComposition.clear();
      let { from: f, to: a } = this.hasComposition;
      i = new U(f, a, t.changes.mapPos(f, -1), t.changes.mapPos(a, 1)).addToSet(i.slice());
    }
    this.hasComposition = s ? { from: s.range.fromB, to: s.range.toB } : null, (w.ie || w.chrome) && !s && t && t.state.doc.lines != t.startState.doc.lines && (this.forceSelection = !0);
    let o = this.decorations, l = this.updateDeco(), h = vr(o, l, t.changes);
    return i = U.extendWithRanges(i, h), !(this.flags & 7) && i.length == 0 ? !1 : (this.updateInner(i, t.startState.doc.length, s), t.transactions.length && (this.lastUpdate = Date.now()), !0);
  }
  // Used by update and the constructor do perform the actual DOM
  // update
  updateInner(t, e, i) {
    this.view.viewState.mustMeasureContent = !0, this.updateChildren(t, e, i);
    let { observer: n } = this.view;
    n.ignore(() => {
      this.dom.style.height = this.view.viewState.contentHeight / this.view.scaleY + "px", this.dom.style.flexBasis = this.minWidth ? this.minWidth + "px" : "";
      let o = w.chrome || w.ios ? { node: n.selectionRange.focusNode, written: !1 } : void 0;
      this.sync(this.view, o), this.flags &= -8, o && (o.written || n.selectionRange.focusNode != o.node) && (this.forceSelection = !0), this.dom.style.height = "";
    }), this.markedForComposition.forEach(
      (o) => o.flags &= -9
      /* ViewFlag.Composition */
    );
    let s = [];
    if (this.view.viewport.from || this.view.viewport.to < this.view.state.doc.length)
      for (let o of this.children)
        o instanceof ot && o.widget instanceof Wi && s.push(o.dom);
    n.updateGaps(s);
  }
  updateChildren(t, e, i) {
    let n = i ? i.range.addToSet(t.slice()) : t, s = this.childCursor(e);
    for (let o = n.length - 1; ; o--) {
      let l = o >= 0 ? n[o] : null;
      if (!l)
        break;
      let { fromA: h, toA: f, fromB: a, toB: c } = l, u, d, g, p;
      if (i && i.range.fromB < c && i.range.toB > a) {
        let x = Kt.build(this.view.state.doc, a, i.range.fromB, this.decorations, this.dynamicDecorationMap), E = Kt.build(this.view.state.doc, i.range.toB, c, this.decorations, this.dynamicDecorationMap);
        d = x.breakAtStart, g = x.openStart, p = E.openEnd;
        let G = this.compositionView(i);
        E.breakAtStart ? G.breakAfter = 1 : E.content.length && G.merge(G.length, G.length, E.content[0], !1, E.openStart, 0) && (G.breakAfter = E.content[0].breakAfter, E.content.shift()), x.content.length && G.merge(0, 0, x.content[x.content.length - 1], !0, 0, x.openEnd) && x.content.pop(), u = x.content.concat(G).concat(E.content);
      } else
        ({ content: u, breakAtStart: d, openStart: g, openEnd: p } = Kt.build(this.view.state.doc, a, c, this.decorations, this.dynamicDecorationMap));
      let { i: m, off: y } = s.findPos(f, 1), { i: S, off: k } = s.findPos(h, -1);
      zn(this, S, k, m, y, u, d, g, p);
    }
    i && this.fixCompositionDOM(i);
  }
  updateEditContextFormatting(t) {
    this.editContextFormatting = this.editContextFormatting.map(t.changes);
    for (let e of t.transactions)
      for (let i of e.effects)
        i.is(hs) && (this.editContextFormatting = i.value);
  }
  compositionView(t) {
    let e = new J(t.text.nodeValue);
    e.flags |= 8;
    for (let { deco: n } of t.marks)
      e = new ht(n, [e], e.length);
    let i = new B();
    return i.append(e, 0), i;
  }
  fixCompositionDOM(t) {
    let e = (s, o) => {
      o.flags |= 8 | (o.children.some(
        (h) => h.flags & 7
        /* ViewFlag.Dirty */
      ) ? 1 : 0), this.markedForComposition.add(o);
      let l = M.get(s);
      l && l != o && (l.dom = null), o.setDOM(s);
    }, i = this.childPos(t.range.fromB, 1), n = this.children[i.i];
    e(t.line, n);
    for (let s = t.marks.length - 1; s >= -1; s--)
      i = n.childPos(i.off, 1), n = n.children[i.i], e(s >= 0 ? t.marks[s].node : t.text, n);
  }
  // Sync the DOM selection to this.state.selection
  updateSelection(t = !1, e = !1) {
    (t || !this.view.observer.selectionRange.focusNode) && this.view.observer.readSelectionRange();
    let i = this.view.root.activeElement, n = i == this.dom, s = !n && ce(this.dom, this.view.observer.selectionRange) && !(i && this.dom.contains(i));
    if (!(n || e || s))
      return;
    let o = this.forceSelection;
    this.forceSelection = !1;
    let l = this.view.state.selection.main, h = this.moveToLine(this.domAtPos(l.anchor)), f = l.empty ? h : this.moveToLine(this.domAtPos(l.head));
    if (w.gecko && l.empty && !this.hasComposition && wr(h)) {
      let c = document.createTextNode("");
      this.view.observer.ignore(() => h.node.insertBefore(c, h.node.childNodes[h.offset] || null)), h = f = new V(c, 0), o = !0;
    }
    let a = this.view.observer.selectionRange;
    (o || !a.focusNode || (!qt(h.node, h.offset, a.anchorNode, a.anchorOffset) || !qt(f.node, f.offset, a.focusNode, a.focusOffset)) && !this.suppressWidgetCursorChange(a, l)) && (this.view.observer.ignore(() => {
      w.android && w.chrome && this.dom.contains(a.focusNode) && kr(a.focusNode, this.dom) && (this.dom.blur(), this.dom.focus({ preventScroll: !0 }));
      let c = Gt(this.view.root);
      if (c) if (l.empty) {
        if (w.gecko) {
          let u = Sr(h.node, h.offset);
          if (u && u != 3) {
            let d = (u == 1 ? Hn : Vn)(h.node, h.offset);
            d && (h = new V(d.node, d.offset));
          }
        }
        c.collapse(h.node, h.offset), l.bidiLevel != null && c.caretBidiLevel !== void 0 && (c.caretBidiLevel = l.bidiLevel);
      } else if (c.extend) {
        c.collapse(h.node, h.offset);
        try {
          c.extend(f.node, f.offset);
        } catch {
        }
      } else {
        let u = document.createRange();
        l.anchor > l.head && ([h, f] = [f, h]), u.setEnd(f.node, f.offset), u.setStart(h.node, h.offset), c.removeAllRanges(), c.addRange(u);
      }
      s && this.view.root.activeElement == this.dom && (this.dom.blur(), i && i.focus());
    }), this.view.observer.setSelectionRange(h, f)), this.impreciseAnchor = h.precise ? null : new V(a.anchorNode, a.anchorOffset), this.impreciseHead = f.precise ? null : new V(a.focusNode, a.focusOffset);
  }
  // If a zero-length widget is inserted next to the cursor during
  // composition, avoid moving it across it and disrupting the
  // composition.
  suppressWidgetCursorChange(t, e) {
    return this.hasComposition && e.empty && qt(t.focusNode, t.focusOffset, t.anchorNode, t.anchorOffset) && this.posFromDOM(t.focusNode, t.focusOffset) == e.head;
  }
  enforceCursorAssoc() {
    if (this.hasComposition)
      return;
    let { view: t } = this, e = t.state.selection.main, i = Gt(t.root), { anchorNode: n, anchorOffset: s } = t.observer.selectionRange;
    if (!i || !e.empty || !e.assoc || !i.modify)
      return;
    let o = B.find(this, e.head);
    if (!o)
      return;
    let l = o.posAtStart;
    if (e.head == l || e.head == l + o.length)
      return;
    let h = this.coordsAt(e.head, -1), f = this.coordsAt(e.head, 1);
    if (!h || !f || h.bottom > f.top)
      return;
    let a = this.domAtPos(e.head + e.assoc);
    i.collapse(a.node, a.offset), i.modify("move", e.assoc < 0 ? "forward" : "backward", "lineboundary"), t.observer.readSelectionRange();
    let c = t.observer.selectionRange;
    t.docView.posFromDOM(c.anchorNode, c.anchorOffset) != e.from && i.collapse(n, s);
  }
  // If a position is in/near a block widget, move it to a nearby text
  // line, since we don't want the cursor inside a block widget.
  moveToLine(t) {
    let e = this.dom, i;
    if (t.node != e)
      return t;
    for (let n = t.offset; !i && n < e.childNodes.length; n++) {
      let s = M.get(e.childNodes[n]);
      s instanceof B && (i = s.domAtPos(0));
    }
    for (let n = t.offset - 1; !i && n >= 0; n--) {
      let s = M.get(e.childNodes[n]);
      s instanceof B && (i = s.domAtPos(s.length));
    }
    return i ? new V(i.node, i.offset, !0) : t;
  }
  nearest(t) {
    for (let e = t; e; ) {
      let i = M.get(e);
      if (i && i.rootView == this)
        return i;
      e = e.parentNode;
    }
    return null;
  }
  posFromDOM(t, e) {
    let i = this.nearest(t);
    if (!i)
      throw new RangeError("Trying to find position for a DOM position outside of the document");
    return i.localPosFromDOM(t, e) + i.posAtStart;
  }
  domAtPos(t) {
    let { i: e, off: i } = this.childCursor().findPos(t, -1);
    for (; e < this.children.length - 1; ) {
      let n = this.children[e];
      if (i < n.length || n instanceof B)
        break;
      e++, i = 0;
    }
    return this.children[e].domAtPos(i);
  }
  coordsAt(t, e) {
    let i = null, n = 0;
    for (let s = this.length, o = this.children.length - 1; o >= 0; o--) {
      let l = this.children[o], h = s - l.breakAfter, f = h - l.length;
      if (h < t)
        break;
      if (f <= t && (f < t || l.covers(-1)) && (h > t || l.covers(1)) && (!i || l instanceof B && !(i instanceof B && e >= 0)))
        i = l, n = f;
      else if (i && f == t && h == t && l instanceof ot && Math.abs(e) < 2) {
        if (l.deco.startSide < 0)
          break;
        o && (i = null);
      }
      s = f;
    }
    return i ? i.coordsAt(t - n, e) : null;
  }
  coordsForChar(t) {
    let { i: e, off: i } = this.childPos(t, 1), n = this.children[e];
    if (!(n instanceof B))
      return null;
    for (; n.children.length; ) {
      let { i: l, off: h } = n.childPos(i, 1);
      for (; ; l++) {
        if (l == n.children.length)
          return null;
        if ((n = n.children[l]).length)
          break;
      }
      i = h;
    }
    if (!(n instanceof J))
      return null;
    let s = it(n.text, i);
    if (s == i)
      return null;
    let o = yt(n.dom, i, s).getClientRects();
    for (let l = 0; l < o.length; l++) {
      let h = o[l];
      if (l == o.length - 1 || h.top < h.bottom && h.left < h.right)
        return h;
    }
    return null;
  }
  measureVisibleLineHeights(t) {
    let e = [], { from: i, to: n } = t, s = this.view.contentDOM.clientWidth, o = s > Math.max(this.view.scrollDOM.clientWidth, this.minWidth) + 1, l = -1, h = this.view.textDirection == z.LTR;
    for (let f = 0, a = 0; a < this.children.length; a++) {
      let c = this.children[a], u = f + c.length;
      if (u > n)
        break;
      if (f >= i) {
        let d = c.dom.getBoundingClientRect();
        if (e.push(d.height), o) {
          let g = c.dom.lastChild, p = g ? Ut(g) : [];
          if (p.length) {
            let m = p[p.length - 1], y = h ? m.right - d.left : d.right - m.left;
            y > l && (l = y, this.minWidth = s, this.minWidthFrom = f, this.minWidthTo = u);
          }
        }
      }
      f = u + c.breakAfter;
    }
    return e;
  }
  textDirectionAt(t) {
    let { i: e } = this.childPos(t, 1);
    return getComputedStyle(this.children[e].dom).direction == "rtl" ? z.RTL : z.LTR;
  }
  measureTextSize() {
    for (let s of this.children)
      if (s instanceof B) {
        let o = s.measureTextSize();
        if (o)
          return o;
      }
    let t = document.createElement("div"), e, i, n;
    return t.className = "cm-line", t.style.width = "99999px", t.style.position = "absolute", t.textContent = "abc def ghi jkl mno pqr stu", this.view.observer.ignore(() => {
      this.dom.appendChild(t);
      let s = Ut(t.firstChild)[0];
      e = t.getBoundingClientRect().height, i = s ? s.width / 27 : 7, n = s ? s.height : e, t.remove();
    }), { lineHeight: e, charWidth: i, textHeight: n };
  }
  childCursor(t = this.length) {
    let e = this.children.length;
    return e && (t -= this.children[--e].length), new Wn(this.children, t, e);
  }
  computeBlockGapDeco() {
    let t = [], e = this.view.viewState;
    for (let i = 0, n = 0; ; n++) {
      let s = n == e.viewports.length ? null : e.viewports[n], o = s ? s.from - 1 : this.length;
      if (o > i) {
        let l = (e.lineBlockAt(o).bottom - e.lineBlockAt(i).top) / this.view.scaleY;
        t.push(Y.replace({
          widget: new Wi(l),
          block: !0,
          inclusive: !0,
          isBlockGap: !0
        }).range(i, o));
      }
      if (!s)
        break;
      i = s.to + 1;
    }
    return Y.set(t);
  }
  updateDeco() {
    let t = 1, e = this.view.state.facet(Jt).map((s) => (this.dynamicDecorationMap[t++] = typeof s == "function") ? s(this.view) : s), i = !1, n = this.view.state.facet(fs).map((s, o) => {
      let l = typeof s == "function";
      return l && (i = !0), l ? s(this.view) : s;
    });
    for (n.length && (this.dynamicDecorationMap[t++] = i, e.push(R.join(n))), this.decorations = [
      this.editContextFormatting,
      ...e,
      this.computeBlockGapDeco(),
      this.view.viewState.lineGapDeco
    ]; t < this.decorations.length; )
      this.dynamicDecorationMap[t++] = !1;
    return this.decorations;
  }
  scrollIntoView(t) {
    if (t.isSnapshot) {
      let f = this.view.viewState.lineBlockAt(t.range.head);
      this.view.scrollDOM.scrollTop = f.top - t.yMargin, this.view.scrollDOM.scrollLeft = t.xMargin;
      return;
    }
    for (let f of this.view.state.facet(ls))
      try {
        if (f(this.view, t.range, t))
          return !0;
      } catch (a) {
        st(this.view.state, a, "scroll handler");
      }
    let { range: e } = t, i = this.coordsAt(e.head, e.empty ? e.assoc : e.head > e.anchor ? -1 : 1), n;
    if (!i)
      return;
    !e.empty && (n = this.coordsAt(e.anchor, e.anchor > e.head ? -1 : 1)) && (i = {
      left: Math.min(i.left, n.left),
      top: Math.min(i.top, n.top),
      right: Math.max(i.right, n.right),
      bottom: Math.max(i.bottom, n.bottom)
    });
    let s = ds(this.view), o = {
      left: i.left - s.left,
      top: i.top - s.top,
      right: i.right + s.right,
      bottom: i.bottom + s.bottom
    }, { offsetWidth: l, offsetHeight: h } = this.view.scrollDOM;
    Qs(this.view.scrollDOM, o, e.head < e.anchor ? -1 : 1, t.x, t.y, Math.max(Math.min(t.xMargin, l), -l), Math.max(Math.min(t.yMargin, h), -h), this.view.textDirection == z.LTR);
  }
}
function wr(r) {
  return r.node.nodeType == 1 && r.node.firstChild && (r.offset == 0 || r.node.childNodes[r.offset - 1].contentEditable == "false") && (r.offset == r.node.childNodes.length || r.node.childNodes[r.offset].contentEditable == "false");
}
class Wi extends di {
  constructor(t) {
    super(), this.height = t;
  }
  toDOM() {
    let t = document.createElement("div");
    return t.className = "cm-gap", this.updateDOM(t), t;
  }
  eq(t) {
    return t.height == this.height;
  }
  updateDOM(t) {
    return t.style.height = this.height + "px", !0;
  }
  get editable() {
    return !0;
  }
  get estimatedHeight() {
    return this.height;
  }
  ignoreEvent() {
    return !1;
  }
}
function gs(r, t) {
  let e = r.observer.selectionRange;
  if (!e.focusNode)
    return null;
  let i = Hn(e.focusNode, e.focusOffset), n = Vn(e.focusNode, e.focusOffset), s = i || n;
  if (n && i && n.node != i.node) {
    let l = M.get(n.node);
    if (!l || l instanceof J && l.text != n.node.nodeValue)
      s = n;
    else if (r.docView.lastCompositionAfterCursor) {
      let h = M.get(i.node);
      !h || h instanceof J && h.text != i.node.nodeValue || (s = n);
    }
  }
  if (r.docView.lastCompositionAfterCursor = s != i, !s)
    return null;
  let o = t - s.offset;
  return { from: o, to: o + s.node.nodeValue.length, node: s.node };
}
function br(r, t, e) {
  let i = gs(r, e);
  if (!i)
    return null;
  let { node: n, from: s, to: o } = i, l = n.nodeValue;
  if (/[\n\r]/.test(l) || r.state.doc.sliceString(i.from, i.to) != l)
    return null;
  let h = t.invertedDesc, f = new U(h.mapPos(s), h.mapPos(o), s, o), a = [];
  for (let c = n.parentNode; ; c = c.parentNode) {
    let u = M.get(c);
    if (u instanceof ht)
      a.push({ node: c, deco: u.mark });
    else {
      if (u instanceof B || c.nodeName == "DIV" && c.parentNode == r.contentDOM)
        return { range: f, text: n, marks: a, line: c };
      if (c != r.contentDOM)
        a.push({ node: c, deco: new Qt({
          inclusive: !0,
          attributes: rr(c),
          tagName: c.tagName.toLowerCase()
        }) });
      else
        return null;
    }
  }
}
function Sr(r, t) {
  return r.nodeType != 1 ? 0 : (t && r.childNodes[t - 1].contentEditable == "false" ? 1 : 0) | (t < r.childNodes.length && r.childNodes[t].contentEditable == "false" ? 2 : 0);
}
let xr = class {
  constructor() {
    this.changes = [];
  }
  compareRange(t, e) {
    Ze(t, e, this.changes);
  }
  comparePoint(t, e) {
    Ze(t, e, this.changes);
  }
};
function vr(r, t, e) {
  let i = new xr();
  return R.compare(r, t, e, i), i.changes;
}
function kr(r, t) {
  for (let e = r; e && e != t; e = e.assignedSlot || e.parentNode)
    if (e.nodeType == 1 && e.contentEditable == "false")
      return !0;
  return !1;
}
function Ar(r, t) {
  let e = !1;
  return t && r.iterChangedRanges((i, n) => {
    i < t.to && n > t.from && (e = !0);
  }), e;
}
function Cr(r, t, e = 1) {
  let i = r.charCategorizer(t), n = r.doc.lineAt(t), s = t - n.from;
  if (n.length == 0)
    return b.cursor(t);
  s == 0 ? e = 1 : s == n.length && (e = -1);
  let o = s, l = s;
  e < 0 ? o = it(n.text, s, !1) : l = it(n.text, s);
  let h = i(n.text.slice(o, l));
  for (; o > 0; ) {
    let f = it(n.text, o, !1);
    if (i(n.text.slice(f, o)) != h)
      break;
    o = f;
  }
  for (; l < n.length; ) {
    let f = it(n.text, l);
    if (i(n.text.slice(l, f)) != h)
      break;
    l = f;
  }
  return b.range(o + n.from, l + n.from);
}
function Mr(r, t) {
  return t.left > r ? t.left - r : Math.max(0, r - t.right);
}
function Dr(r, t) {
  return t.top > r ? t.top - r : Math.max(0, r - t.bottom);
}
function Te(r, t) {
  return r.top < t.bottom - 1 && r.bottom > t.top + 1;
}
function zi(r, t) {
  return t < r.top ? { top: t, left: r.left, right: r.right, bottom: r.bottom } : r;
}
function qi(r, t) {
  return t > r.bottom ? { top: r.top, left: r.left, right: r.right, bottom: t } : r;
}
function ni(r, t, e) {
  let i, n, s, o, l = !1, h, f, a, c;
  for (let g = r.firstChild; g; g = g.nextSibling) {
    let p = Ut(g);
    for (let m = 0; m < p.length; m++) {
      let y = p[m];
      n && Te(n, y) && (y = zi(qi(y, n.bottom), n.top));
      let S = Mr(t, y), k = Dr(e, y);
      if (S == 0 && k == 0)
        return g.nodeType == 3 ? Ki(g, t, e) : ni(g, t, e);
      if (!i || o > k || o == k && s > S) {
        i = g, n = y, s = S, o = k;
        let x = k ? e < y.top ? -1 : 1 : S ? t < y.left ? -1 : 1 : 0;
        l = !x || (x > 0 ? m < p.length - 1 : m > 0);
      }
      S == 0 ? e > y.bottom && (!a || a.bottom < y.bottom) ? (h = g, a = y) : e < y.top && (!c || c.top > y.top) && (f = g, c = y) : a && Te(a, y) ? a = qi(a, y.bottom) : c && Te(c, y) && (c = zi(c, y.top));
    }
  }
  if (a && a.bottom >= e ? (i = h, n = a) : c && c.top <= e && (i = f, n = c), !i)
    return { node: r, offset: 0 };
  let u = Math.max(n.left, Math.min(n.right, t));
  if (i.nodeType == 3)
    return Ki(i, u, e);
  if (l && i.contentEditable != "false")
    return ni(i, u, e);
  let d = Array.prototype.indexOf.call(r.childNodes, i) + (t >= (n.left + n.right) / 2 ? 1 : 0);
  return { node: r, offset: d };
}
function Ki(r, t, e) {
  let i = r.nodeValue.length, n = -1, s = 1e9, o = 0;
  for (let l = 0; l < i; l++) {
    let h = yt(r, l, l + 1).getClientRects();
    for (let f = 0; f < h.length; f++) {
      let a = h[f];
      if (a.top == a.bottom)
        continue;
      o || (o = t - a.left);
      let c = (a.top > e ? a.top - e : e - a.bottom) - 1;
      if (a.left - 1 <= t && a.right + 1 >= t && c < s) {
        let u = t >= (a.left + a.right) / 2, d = u;
        if ((w.chrome || w.gecko) && yt(r, l).getBoundingClientRect().left == a.right && (d = !u), c <= 0)
          return { node: r, offset: l + (d ? 1 : 0) };
        n = l + (d ? 1 : 0), s = c;
      }
    }
  }
  return { node: r, offset: n > -1 ? n : o > 0 ? r.nodeValue.length : 0 };
}
function ps(r, t, e, i = -1) {
  var n, s;
  let o = r.contentDOM.getBoundingClientRect(), l = o.top + r.viewState.paddingTop, h, { docHeight: f } = r.viewState, { x: a, y: c } = t, u = c - l;
  if (u < 0)
    return 0;
  if (u > f)
    return r.state.doc.length;
  for (let x = r.viewState.heightOracle.textHeight / 2, E = !1; h = r.elementAtHeight(u), h.type != et.Text; )
    for (; u = i > 0 ? h.bottom + x : h.top - x, !(u >= 0 && u <= f); ) {
      if (E)
        return e ? null : 0;
      E = !0, i = -i;
    }
  c = l + u;
  let d = h.from;
  if (d < r.viewport.from)
    return r.viewport.from == 0 ? 0 : e ? null : $i(r, o, h, a, c);
  if (d > r.viewport.to)
    return r.viewport.to == r.state.doc.length ? r.state.doc.length : e ? null : $i(r, o, h, a, c);
  let g = r.dom.ownerDocument, p = r.root.elementFromPoint ? r.root : g, m = p.elementFromPoint(a, c);
  m && !r.contentDOM.contains(m) && (m = null), m || (a = Math.max(o.left + 1, Math.min(o.right - 1, a)), m = p.elementFromPoint(a, c), m && !r.contentDOM.contains(m) && (m = null));
  let y, S = -1;
  if (m && ((n = r.docView.nearest(m)) === null || n === void 0 ? void 0 : n.isEditable) != !1) {
    if (g.caretPositionFromPoint) {
      let x = g.caretPositionFromPoint(a, c);
      x && ({ offsetNode: y, offset: S } = x);
    } else if (g.caretRangeFromPoint) {
      let x = g.caretRangeFromPoint(a, c);
      x && ({ startContainer: y, startOffset: S } = x, (!r.contentDOM.contains(y) || w.safari && Or(y, S, a) || w.chrome && Tr(y, S, a)) && (y = void 0));
    }
  }
  if (!y || !r.docView.dom.contains(y)) {
    let x = B.find(r.docView, d);
    if (!x)
      return u > h.top + h.height / 2 ? h.to : h.from;
    ({ node: y, offset: S } = ni(x.dom, a, c));
  }
  let k = r.docView.nearest(y);
  if (!k)
    return null;
  if (k.isWidget && ((s = k.dom) === null || s === void 0 ? void 0 : s.nodeType) == 1) {
    let x = k.dom.getBoundingClientRect();
    return t.y < x.top || t.y <= x.bottom && t.x <= (x.left + x.right) / 2 ? k.posAtStart : k.posAtEnd;
  } else
    return k.localPosFromDOM(y, S) + k.posAtStart;
}
function $i(r, t, e, i, n) {
  let s = Math.round((i - t.left) * r.defaultCharacterWidth);
  if (r.lineWrapping && e.height > r.defaultLineHeight * 1.5) {
    let l = r.viewState.heightOracle.textHeight, h = Math.floor((n - e.top - (r.defaultLineHeight - l) * 0.5) / l);
    s += h * r.viewState.heightOracle.lineLength;
  }
  let o = r.state.sliceDoc(e.from, e.to);
  return e.from + Gs(o, s, r.state.tabSize);
}
function Or(r, t, e) {
  let i;
  if (r.nodeType != 3 || t != (i = r.nodeValue.length))
    return !1;
  for (let n = r.nextSibling; n; n = n.nextSibling)
    if (n.nodeType != 1 || n.nodeName != "BR")
      return !1;
  return yt(r, i - 1, i).getBoundingClientRect().left > e;
}
function Tr(r, t, e) {
  if (t != 0)
    return !1;
  for (let n = r; ; ) {
    let s = n.parentNode;
    if (!s || s.nodeType != 1 || s.firstChild != n)
      return !1;
    if (s.classList.contains("cm-line"))
      break;
    n = s;
  }
  let i = r.nodeType == 1 ? r.getBoundingClientRect() : yt(r, 0, Math.max(r.nodeValue.length, 1)).getBoundingClientRect();
  return e - i.left > 5;
}
function Rr(r, t) {
  let e = r.lineBlockAt(t);
  if (Array.isArray(e.type)) {
    for (let i of e.type)
      if (i.to > t || i.to == t && (i.to == e.to || i.type == et.Text))
        return i;
  }
  return e;
}
function Er(r, t, e, i) {
  let n = Rr(r, t.head), s = !i || n.type != et.Text || !(r.lineWrapping || n.widgetLineBreaks) ? null : r.coordsAtPos(t.assoc < 0 && t.head > n.from ? t.head - 1 : t.head);
  if (s) {
    let o = r.dom.getBoundingClientRect(), l = r.textDirectionAt(n.from), h = r.posAtCoords({
      x: e == (l == z.LTR) ? o.right - 1 : o.left + 1,
      y: (s.top + s.bottom) / 2
    });
    if (h != null)
      return b.cursor(h, e ? -1 : 1);
  }
  return b.cursor(e ? n.to : n.from, e ? -1 : 1);
}
function ji(r, t, e, i) {
  let n = r.state.doc.lineAt(t.head), s = r.bidiSpans(n), o = r.textDirectionAt(n.from);
  for (let l = t, h = null; ; ) {
    let f = gr(n, s, o, l, e), a = _n;
    if (!f) {
      if (n.number == (e ? r.state.doc.lines : 1))
        return l;
      a = `
`, n = r.state.doc.line(n.number + (e ? 1 : -1)), s = r.bidiSpans(n), f = r.visualLineSide(n, !e);
    }
    if (h) {
      if (!h(a))
        return l;
    } else {
      if (!i)
        return f;
      h = i(a);
    }
    l = f;
  }
}
function Pr(r, t, e) {
  let i = r.state.charCategorizer(t), n = i(e);
  return (s) => {
    let o = i(s);
    return n == nt.Space && (n = o), n == o;
  };
}
function Br(r, t, e, i) {
  let n = t.head, s = e ? 1 : -1;
  if (n == (e ? r.state.doc.length : 0))
    return b.cursor(n, t.assoc);
  let o = t.goalColumn, l, h = r.contentDOM.getBoundingClientRect(), f = r.coordsAtPos(n, t.assoc || -1), a = r.documentTop;
  if (f)
    o == null && (o = f.left - h.left), l = s < 0 ? f.top : f.bottom;
  else {
    let d = r.viewState.lineBlockAt(n);
    o == null && (o = Math.min(h.right - h.left, r.defaultCharacterWidth * (n - d.from))), l = (s < 0 ? d.top : d.bottom) + a;
  }
  let c = h.left + o, u = i ?? r.viewState.heightOracle.textHeight >> 1;
  for (let d = 0; ; d += 10) {
    let g = l + (u + d) * s, p = ps(r, { x: c, y: g }, !1, s);
    if (g < h.top || g > h.bottom || (s < 0 ? p < n : p > n)) {
      let m = r.docView.coordsForChar(p), y = !m || g < m.top ? -1 : 1;
      return b.cursor(p, y, void 0, o);
    }
  }
}
function ue(r, t, e) {
  for (; ; ) {
    let i = 0;
    for (let n of r)
      n.between(t - 1, t + 1, (s, o, l) => {
        if (t > s && t < o) {
          let h = i || e || (t - s < o - t ? -1 : 1);
          t = h < 0 ? s : o, i = h;
        }
      });
    if (!i)
      return t;
  }
}
function Re(r, t, e) {
  let i = ue(r.state.facet(mi).map((n) => n(r)), e.from, t.head > e.from ? -1 : 1);
  return i == e.from ? e : b.cursor(i, i < e.from ? 1 : -1);
}
class Lr {
  setSelectionOrigin(t) {
    this.lastSelectionOrigin = t, this.lastSelectionTime = Date.now();
  }
  constructor(t) {
    this.view = t, this.lastKeyCode = 0, this.lastKeyTime = 0, this.lastTouchTime = 0, this.lastFocusTime = 0, this.lastScrollTop = 0, this.lastScrollLeft = 0, this.pendingIOSKey = void 0, this.tabFocusMode = -1, this.lastSelectionOrigin = null, this.lastSelectionTime = 0, this.lastContextMenu = 0, this.scrollHandlers = [], this.handlers = /* @__PURE__ */ Object.create(null), this.composing = -1, this.compositionFirstChange = null, this.compositionEndedAt = 0, this.compositionPendingKey = !1, this.compositionPendingChange = !1, this.mouseSelection = null, this.draggedContent = null, this.handleEvent = this.handleEvent.bind(this), this.notifiedFocused = t.hasFocus, w.safari && t.contentDOM.addEventListener("input", () => null), w.gecko && Qr(t.contentDOM.ownerDocument);
  }
  handleEvent(t) {
    !Kr(this.view, t) || this.ignoreDuringComposition(t) || t.type == "keydown" && this.keydown(t) || this.runHandlers(t.type, t);
  }
  runHandlers(t, e) {
    let i = this.handlers[t];
    if (i) {
      for (let n of i.observers)
        n(this.view, e);
      for (let n of i.handlers) {
        if (e.defaultPrevented)
          break;
        if (n(this.view, e)) {
          e.preventDefault();
          break;
        }
      }
    }
  }
  ensureHandlers(t) {
    let e = Fr(t), i = this.handlers, n = this.view.contentDOM;
    for (let s in e)
      if (s != "scroll") {
        let o = !e[s].handlers.length, l = i[s];
        l && o != !l.handlers.length && (n.removeEventListener(s, this.handleEvent), l = null), l || n.addEventListener(s, this.handleEvent, { passive: o });
      }
    for (let s in i)
      s != "scroll" && !e[s] && n.removeEventListener(s, this.handleEvent);
    this.handlers = e;
  }
  keydown(t) {
    if (this.lastKeyCode = t.keyCode, this.lastKeyTime = Date.now(), t.keyCode == 9 && this.tabFocusMode > -1 && (!this.tabFocusMode || Date.now() <= this.tabFocusMode))
      return !0;
    if (this.tabFocusMode > 0 && t.keyCode != 27 && Nr.indexOf(t.keyCode) < 0 && (this.tabFocusMode = -1), w.android && w.chrome && !t.synthetic && (t.keyCode == 13 || t.keyCode == 8))
      return this.view.observer.delayAndroidKey(t.key, t.keyCode), !0;
    let e;
    return w.ios && !t.synthetic && !t.altKey && !t.metaKey && ((e = ms.find((i) => i.keyCode == t.keyCode)) && !t.ctrlKey || Ir.indexOf(t.key) > -1 && t.ctrlKey && !t.shiftKey) ? (this.pendingIOSKey = e || t, setTimeout(() => this.flushIOSKey(), 250), !0) : (t.keyCode != 229 && this.view.observer.forceFlush(), !1);
  }
  flushIOSKey(t) {
    let e = this.pendingIOSKey;
    return !e || e.key == "Enter" && t && t.from < t.to && /^\S+$/.test(t.insert.toString()) ? !1 : (this.pendingIOSKey = void 0, vt(this.view.contentDOM, e.key, e.keyCode, e instanceof KeyboardEvent ? e : void 0));
  }
  ignoreDuringComposition(t) {
    return /^key/.test(t.type) ? this.composing > 0 ? !0 : w.safari && !w.ios && this.compositionPendingKey && Date.now() - this.compositionEndedAt < 100 ? (this.compositionPendingKey = !1, !0) : !1 : !1;
  }
  startMouseSelection(t) {
    this.mouseSelection && this.mouseSelection.destroy(), this.mouseSelection = t;
  }
  update(t) {
    this.view.observer.update(t), this.mouseSelection && this.mouseSelection.update(t), this.draggedContent && t.docChanged && (this.draggedContent = this.draggedContent.map(t.changes)), t.transactions.length && (this.lastKeyCode = this.lastSelectionTime = 0);
  }
  destroy() {
    this.mouseSelection && this.mouseSelection.destroy();
  }
}
function Yi(r, t) {
  return (e, i) => {
    try {
      return t.call(r, i, e);
    } catch (n) {
      st(e.state, n);
    }
  };
}
function Fr(r) {
  let t = /* @__PURE__ */ Object.create(null);
  function e(i) {
    return t[i] || (t[i] = { observers: [], handlers: [] });
  }
  for (let i of r) {
    let n = i.spec;
    if (n && n.domEventHandlers)
      for (let s in n.domEventHandlers) {
        let o = n.domEventHandlers[s];
        o && e(s).handlers.push(Yi(i.value, o));
      }
    if (n && n.domEventObservers)
      for (let s in n.domEventObservers) {
        let o = n.domEventObservers[s];
        o && e(s).observers.push(Yi(i.value, o));
      }
  }
  for (let i in Q)
    e(i).handlers.push(Q[i]);
  for (let i in X)
    e(i).observers.push(X[i]);
  return t;
}
const ms = [
  { key: "Backspace", keyCode: 8, inputType: "deleteContentBackward" },
  { key: "Enter", keyCode: 13, inputType: "insertParagraph" },
  { key: "Enter", keyCode: 13, inputType: "insertLineBreak" },
  { key: "Delete", keyCode: 46, inputType: "deleteContentForward" }
], Ir = "dthko", Nr = [16, 17, 18, 20, 91, 92, 224, 225], ne = 6;
function se(r) {
  return Math.max(0, r) * 0.7 + 8;
}
function Hr(r, t) {
  return Math.max(Math.abs(r.clientX - t.clientX), Math.abs(r.clientY - t.clientY));
}
class Vr {
  constructor(t, e, i, n) {
    this.view = t, this.startEvent = e, this.style = i, this.mustSelect = n, this.scrollSpeed = { x: 0, y: 0 }, this.scrolling = -1, this.lastEvent = e, this.scrollParents = Zs(t.contentDOM), this.atoms = t.state.facet(mi).map((o) => o(t));
    let s = t.contentDOM.ownerDocument;
    s.addEventListener("mousemove", this.move = this.move.bind(this)), s.addEventListener("mouseup", this.up = this.up.bind(this)), this.extend = e.shiftKey, this.multiple = t.state.facet(T.allowMultipleSelections) && Wr(t, e), this.dragging = qr(t, e) && bs(e) == 1 ? null : !1;
  }
  start(t) {
    this.dragging === !1 && this.select(t);
  }
  move(t) {
    if (t.buttons == 0)
      return this.destroy();
    if (this.dragging || this.dragging == null && Hr(this.startEvent, t) < 10)
      return;
    this.select(this.lastEvent = t);
    let e = 0, i = 0, n = 0, s = 0, o = this.view.win.innerWidth, l = this.view.win.innerHeight;
    this.scrollParents.x && ({ left: n, right: o } = this.scrollParents.x.getBoundingClientRect()), this.scrollParents.y && ({ top: s, bottom: l } = this.scrollParents.y.getBoundingClientRect());
    let h = ds(this.view);
    t.clientX - h.left <= n + ne ? e = -se(n - t.clientX) : t.clientX + h.right >= o - ne && (e = se(t.clientX - o)), t.clientY - h.top <= s + ne ? i = -se(s - t.clientY) : t.clientY + h.bottom >= l - ne && (i = se(t.clientY - l)), this.setScrollSpeed(e, i);
  }
  up(t) {
    this.dragging == null && this.select(this.lastEvent), this.dragging || t.preventDefault(), this.destroy();
  }
  destroy() {
    this.setScrollSpeed(0, 0);
    let t = this.view.contentDOM.ownerDocument;
    t.removeEventListener("mousemove", this.move), t.removeEventListener("mouseup", this.up), this.view.inputState.mouseSelection = this.view.inputState.draggedContent = null;
  }
  setScrollSpeed(t, e) {
    this.scrollSpeed = { x: t, y: e }, t || e ? this.scrolling < 0 && (this.scrolling = setInterval(() => this.scroll(), 50)) : this.scrolling > -1 && (clearInterval(this.scrolling), this.scrolling = -1);
  }
  scroll() {
    let { x: t, y: e } = this.scrollSpeed;
    t && this.scrollParents.x && (this.scrollParents.x.scrollLeft += t, t = 0), e && this.scrollParents.y && (this.scrollParents.y.scrollTop += e, e = 0), (t || e) && this.view.win.scrollBy(t, e), this.dragging === !1 && this.select(this.lastEvent);
  }
  skipAtoms(t) {
    let e = null;
    for (let i = 0; i < t.ranges.length; i++) {
      let n = t.ranges[i], s = null;
      if (n.empty) {
        let o = ue(this.atoms, n.from, 0);
        o != n.from && (s = b.cursor(o, -1));
      } else {
        let o = ue(this.atoms, n.from, -1), l = ue(this.atoms, n.to, 1);
        (o != n.from || l != n.to) && (s = b.range(n.from == n.anchor ? o : l, n.from == n.head ? o : l));
      }
      s && (e || (e = t.ranges.slice()), e[i] = s);
    }
    return e ? b.create(e, t.mainIndex) : t;
  }
  select(t) {
    let { view: e } = this, i = this.skipAtoms(this.style.get(t, this.extend, this.multiple));
    (this.mustSelect || !i.eq(e.state.selection, this.dragging === !1)) && this.view.dispatch({
      selection: i,
      userEvent: "select.pointer"
    }), this.mustSelect = !1;
  }
  update(t) {
    t.transactions.some((e) => e.isUserEvent("input.type")) ? this.destroy() : this.style.update(t) && setTimeout(() => this.select(this.lastEvent), 20);
  }
}
function Wr(r, t) {
  let e = r.state.facet(ts);
  return e.length ? e[0](t) : w.mac ? t.metaKey : t.ctrlKey;
}
function zr(r, t) {
  let e = r.state.facet(es);
  return e.length ? e[0](t) : w.mac ? !t.altKey : !t.ctrlKey;
}
function qr(r, t) {
  let { main: e } = r.state.selection;
  if (e.empty)
    return !1;
  let i = Gt(r.root);
  if (!i || i.rangeCount == 0)
    return !0;
  let n = i.getRangeAt(0).getClientRects();
  for (let s = 0; s < n.length; s++) {
    let o = n[s];
    if (o.left <= t.clientX && o.right >= t.clientX && o.top <= t.clientY && o.bottom >= t.clientY)
      return !0;
  }
  return !1;
}
function Kr(r, t) {
  if (!t.bubbles)
    return !0;
  if (t.defaultPrevented)
    return !1;
  for (let e = t.target, i; e != r.contentDOM; e = e.parentNode)
    if (!e || e.nodeType == 11 || (i = M.get(e)) && i.ignoreEvent(t))
      return !1;
  return !0;
}
const Q = /* @__PURE__ */ Object.create(null), X = /* @__PURE__ */ Object.create(null), ys = w.ie && w.ie_version < 15 || w.ios && w.webkit_version < 604;
function $r(r) {
  let t = r.dom.parentNode;
  if (!t)
    return;
  let e = t.appendChild(document.createElement("textarea"));
  e.style.cssText = "position: fixed; left: -10000px; top: 10px", e.focus(), setTimeout(() => {
    r.focus(), e.remove(), ws(r, e.value);
  }, 50);
}
function ws(r, t) {
  let { state: e } = r, i, n = 1, s = e.toText(t), o = s.lines == e.selection.ranges.length;
  if (si != null && e.selection.ranges.every((h) => h.empty) && si == s.toString()) {
    let h = -1;
    i = e.changeByRange((f) => {
      let a = e.doc.lineAt(f.from);
      if (a.from == h)
        return { range: f };
      h = a.from;
      let c = e.toText((o ? s.line(n++).text : t) + e.lineBreak);
      return {
        changes: { from: a.from, insert: c },
        range: b.cursor(f.from + c.length)
      };
    });
  } else o ? i = e.changeByRange((h) => {
    let f = s.line(n++);
    return {
      changes: { from: h.from, to: h.to, insert: f.text },
      range: b.cursor(h.from + f.length)
    };
  }) : i = e.replaceSelection(s);
  r.dispatch(i, {
    userEvent: "input.paste",
    scrollIntoView: !0
  });
}
X.scroll = (r) => {
  r.inputState.lastScrollTop = r.scrollDOM.scrollTop, r.inputState.lastScrollLeft = r.scrollDOM.scrollLeft;
};
Q.keydown = (r, t) => (r.inputState.setSelectionOrigin("select"), t.keyCode == 27 && r.inputState.tabFocusMode != 0 && (r.inputState.tabFocusMode = Date.now() + 2e3), !1);
X.touchstart = (r, t) => {
  r.inputState.lastTouchTime = Date.now(), r.inputState.setSelectionOrigin("select.pointer");
};
X.touchmove = (r) => {
  r.inputState.setSelectionOrigin("select.pointer");
};
Q.mousedown = (r, t) => {
  if (r.observer.flush(), r.inputState.lastTouchTime > Date.now() - 2e3)
    return !1;
  let e = null;
  for (let i of r.state.facet(is))
    if (e = i(r, t), e)
      break;
  if (!e && t.button == 0 && (e = Gr(r, t)), e) {
    let i = !r.hasFocus;
    r.inputState.startMouseSelection(new Vr(r, t, e, i)), i && r.observer.ignore(() => {
      Fn(r.contentDOM);
      let s = r.root.activeElement;
      s && !s.contains(r.contentDOM) && s.blur();
    });
    let n = r.inputState.mouseSelection;
    if (n)
      return n.start(t), n.dragging === !1;
  }
  return !1;
};
function Gi(r, t, e, i) {
  if (i == 1)
    return b.cursor(t, e);
  if (i == 2)
    return Cr(r.state, t, e);
  {
    let n = B.find(r.docView, t), s = r.state.doc.lineAt(n ? n.posAtEnd : t), o = n ? n.posAtStart : s.from, l = n ? n.posAtEnd : s.to;
    return l < r.state.doc.length && l == s.to && l++, b.range(o, l);
  }
}
let Ui = (r, t, e) => t >= e.top && t <= e.bottom && r >= e.left && r <= e.right;
function jr(r, t, e, i) {
  let n = B.find(r.docView, t);
  if (!n)
    return 1;
  let s = t - n.posAtStart;
  if (s == 0)
    return 1;
  if (s == n.length)
    return -1;
  let o = n.coordsAt(s, -1);
  if (o && Ui(e, i, o))
    return -1;
  let l = n.coordsAt(s, 1);
  return l && Ui(e, i, l) ? 1 : o && o.bottom >= i ? -1 : 1;
}
function Xi(r, t) {
  let e = r.posAtCoords({ x: t.clientX, y: t.clientY }, !1);
  return { pos: e, bias: jr(r, e, t.clientX, t.clientY) };
}
const Yr = w.ie && w.ie_version <= 11;
let Ji = null, Qi = 0, Zi = 0;
function bs(r) {
  if (!Yr)
    return r.detail;
  let t = Ji, e = Zi;
  return Ji = r, Zi = Date.now(), Qi = !t || e > Date.now() - 400 && Math.abs(t.clientX - r.clientX) < 2 && Math.abs(t.clientY - r.clientY) < 2 ? (Qi + 1) % 3 : 1;
}
function Gr(r, t) {
  let e = Xi(r, t), i = bs(t), n = r.state.selection;
  return {
    update(s) {
      s.docChanged && (e.pos = s.changes.mapPos(e.pos), n = n.map(s.changes));
    },
    get(s, o, l) {
      let h = Xi(r, s), f, a = Gi(r, h.pos, h.bias, i);
      if (e.pos != h.pos && !o) {
        let c = Gi(r, e.pos, e.bias, i), u = Math.min(c.from, a.from), d = Math.max(c.to, a.to);
        a = u < a.from ? b.range(u, d) : b.range(d, u);
      }
      return o ? n.replaceRange(n.main.extend(a.from, a.to)) : l && i == 1 && n.ranges.length > 1 && (f = Ur(n, h.pos)) ? f : l ? n.addRange(a) : b.create([a]);
    }
  };
}
function Ur(r, t) {
  for (let e = 0; e < r.ranges.length; e++) {
    let { from: i, to: n } = r.ranges[e];
    if (i <= t && n >= t)
      return b.create(r.ranges.slice(0, e).concat(r.ranges.slice(e + 1)), r.mainIndex == e ? 0 : r.mainIndex - (r.mainIndex > e ? 1 : 0));
  }
  return null;
}
Q.dragstart = (r, t) => {
  let { selection: { main: e } } = r.state;
  if (t.target.draggable) {
    let n = r.docView.nearest(t.target);
    if (n && n.isWidget) {
      let s = n.posAtStart, o = s + n.length;
      (s >= e.to || o <= e.from) && (e = b.range(s, o));
    }
  }
  let { inputState: i } = r;
  return i.mouseSelection && (i.mouseSelection.dragging = !0), i.draggedContent = e, t.dataTransfer && (t.dataTransfer.setData("Text", r.state.sliceDoc(e.from, e.to)), t.dataTransfer.effectAllowed = "copyMove"), !1;
};
Q.dragend = (r) => (r.inputState.draggedContent = null, !1);
function _i(r, t, e, i) {
  if (!e)
    return;
  let n = r.posAtCoords({ x: t.clientX, y: t.clientY }, !1), { draggedContent: s } = r.inputState, o = i && s && zr(r, t) ? { from: s.from, to: s.to } : null, l = { from: n, insert: e }, h = r.state.changes(o ? [o, l] : l);
  r.focus(), r.dispatch({
    changes: h,
    selection: { anchor: h.mapPos(n, -1), head: h.mapPos(n, 1) },
    userEvent: o ? "move.drop" : "input.drop"
  }), r.inputState.draggedContent = null;
}
Q.drop = (r, t) => {
  if (!t.dataTransfer)
    return !1;
  if (r.state.readOnly)
    return !0;
  let e = t.dataTransfer.files;
  if (e && e.length) {
    let i = Array(e.length), n = 0, s = () => {
      ++n == e.length && _i(r, t, i.filter((o) => o != null).join(r.state.lineBreak), !1);
    };
    for (let o = 0; o < e.length; o++) {
      let l = new FileReader();
      l.onerror = s, l.onload = () => {
        /[\x00-\x08\x0e-\x1f]{2}/.test(l.result) || (i[o] = l.result), s();
      }, l.readAsText(e[o]);
    }
    return !0;
  } else {
    let i = t.dataTransfer.getData("Text");
    if (i)
      return _i(r, t, i, !0), !0;
  }
  return !1;
};
Q.paste = (r, t) => {
  if (r.state.readOnly)
    return !0;
  r.observer.flush();
  let e = ys ? null : t.clipboardData;
  return e ? (ws(r, e.getData("text/plain") || e.getData("text/uri-list")), !0) : ($r(r), !1);
};
function Xr(r, t) {
  let e = r.dom.parentNode;
  if (!e)
    return;
  let i = e.appendChild(document.createElement("textarea"));
  i.style.cssText = "position: fixed; left: -10000px; top: 10px", i.value = t, i.focus(), i.selectionEnd = t.length, i.selectionStart = 0, setTimeout(() => {
    i.remove(), r.focus();
  }, 50);
}
function Jr(r) {
  let t = [], e = [], i = !1;
  for (let n of r.selection.ranges)
    n.empty || (t.push(r.sliceDoc(n.from, n.to)), e.push(n));
  if (!t.length) {
    let n = -1;
    for (let { from: s } of r.selection.ranges) {
      let o = r.doc.lineAt(s);
      o.number > n && (t.push(o.text), e.push({ from: o.from, to: Math.min(r.doc.length, o.to + 1) })), n = o.number;
    }
    i = !0;
  }
  return { text: t.join(r.lineBreak), ranges: e, linewise: i };
}
let si = null;
Q.copy = Q.cut = (r, t) => {
  let { text: e, ranges: i, linewise: n } = Jr(r.state);
  if (!e && !n)
    return !1;
  si = n ? e : null, t.type == "cut" && !r.state.readOnly && r.dispatch({
    changes: i,
    scrollIntoView: !0,
    userEvent: "delete.cut"
  });
  let s = ys ? null : t.clipboardData;
  return s ? (s.clearData(), s.setData("text/plain", e), !0) : (Xr(r, e), !1);
};
const Ss = /* @__PURE__ */ Bt.define();
function xs(r, t) {
  let e = [];
  for (let i of r.facet(rs)) {
    let n = i(r, t);
    n && e.push(n);
  }
  return e ? r.update({ effects: e, annotations: Ss.of(!0) }) : null;
}
function vs(r) {
  setTimeout(() => {
    let t = r.hasFocus;
    if (t != r.inputState.notifiedFocused) {
      let e = xs(r.state, t);
      e ? r.dispatch(e) : r.update([]);
    }
  }, 10);
}
X.focus = (r) => {
  r.inputState.lastFocusTime = Date.now(), !r.scrollDOM.scrollTop && (r.inputState.lastScrollTop || r.inputState.lastScrollLeft) && (r.scrollDOM.scrollTop = r.inputState.lastScrollTop, r.scrollDOM.scrollLeft = r.inputState.lastScrollLeft), vs(r);
};
X.blur = (r) => {
  r.observer.clearSelectionRange(), vs(r);
};
X.compositionstart = X.compositionupdate = (r) => {
  r.observer.editContext || (r.inputState.compositionFirstChange == null && (r.inputState.compositionFirstChange = !0), r.inputState.composing < 0 && (r.inputState.composing = 0));
};
X.compositionend = (r) => {
  r.observer.editContext || (r.inputState.composing = -1, r.inputState.compositionEndedAt = Date.now(), r.inputState.compositionPendingKey = !0, r.inputState.compositionPendingChange = r.observer.pendingRecords().length > 0, r.inputState.compositionFirstChange = null, w.chrome && w.android ? r.observer.flushSoon() : r.inputState.compositionPendingChange ? Promise.resolve().then(() => r.observer.flush()) : setTimeout(() => {
    r.inputState.composing < 0 && r.docView.hasComposition && r.update([]);
  }, 50));
};
X.contextmenu = (r) => {
  r.inputState.lastContextMenu = Date.now();
};
Q.beforeinput = (r, t) => {
  var e;
  let i;
  if (w.chrome && w.android && (i = ms.find((n) => n.inputType == t.inputType)) && (r.observer.delayAndroidKey(i.key, i.keyCode), i.key == "Backspace" || i.key == "Delete")) {
    let n = ((e = window.visualViewport) === null || e === void 0 ? void 0 : e.height) || 0;
    setTimeout(() => {
      var s;
      (((s = window.visualViewport) === null || s === void 0 ? void 0 : s.height) || 0) > n + 10 && r.hasFocus && (r.contentDOM.blur(), r.focus());
    }, 100);
  }
  return w.ios && t.inputType == "deleteContentForward" && r.observer.flushSoon(), w.safari && t.inputType == "insertText" && r.inputState.composing >= 0 && setTimeout(() => X.compositionend(r, t), 20), !1;
};
const tn = /* @__PURE__ */ new Set();
function Qr(r) {
  tn.has(r) || (tn.add(r), r.addEventListener("copy", () => {
  }), r.addEventListener("cut", () => {
  }));
}
const en = ["pre-wrap", "normal", "pre-line", "break-spaces"];
let Rt = !1;
function nn() {
  Rt = !1;
}
class Zr {
  constructor(t) {
    this.lineWrapping = t, this.doc = A.empty, this.heightSamples = {}, this.lineHeight = 14, this.charWidth = 7, this.textHeight = 14, this.lineLength = 30;
  }
  heightForGap(t, e) {
    let i = this.doc.lineAt(e).number - this.doc.lineAt(t).number + 1;
    return this.lineWrapping && (i += Math.max(0, Math.ceil((e - t - i * this.lineLength * 0.5) / this.lineLength))), this.lineHeight * i;
  }
  heightForLine(t) {
    return this.lineWrapping ? (1 + Math.max(0, Math.ceil((t - this.lineLength) / (this.lineLength - 5)))) * this.lineHeight : this.lineHeight;
  }
  setDoc(t) {
    return this.doc = t, this;
  }
  mustRefreshForWrapping(t) {
    return en.indexOf(t) > -1 != this.lineWrapping;
  }
  mustRefreshForHeights(t) {
    let e = !1;
    for (let i = 0; i < t.length; i++) {
      let n = t[i];
      n < 0 ? i++ : this.heightSamples[Math.floor(n * 10)] || (e = !0, this.heightSamples[Math.floor(n * 10)] = !0);
    }
    return e;
  }
  refresh(t, e, i, n, s, o) {
    let l = en.indexOf(t) > -1, h = Math.round(e) != Math.round(this.lineHeight) || this.lineWrapping != l;
    if (this.lineWrapping = l, this.lineHeight = e, this.charWidth = i, this.textHeight = n, this.lineLength = s, h) {
      this.heightSamples = {};
      for (let f = 0; f < o.length; f++) {
        let a = o[f];
        a < 0 ? f++ : this.heightSamples[Math.floor(a * 10)] = !0;
      }
    }
    return h;
  }
}
class _r {
  constructor(t, e) {
    this.from = t, this.heights = e, this.index = 0;
  }
  get more() {
    return this.index < this.heights.length;
  }
}
class tt {
  /**
  @internal
  */
  constructor(t, e, i, n, s) {
    this.from = t, this.length = e, this.top = i, this.height = n, this._content = s;
  }
  /**
  The type of element this is. When querying lines, this may be
  an array of all the blocks that make up the line.
  */
  get type() {
    return typeof this._content == "number" ? et.Text : Array.isArray(this._content) ? this._content : this._content.type;
  }
  /**
  The end of the element as a document position.
  */
  get to() {
    return this.from + this.length;
  }
  /**
  The bottom position of the element.
  */
  get bottom() {
    return this.top + this.height;
  }
  /**
  If this is a widget block, this will return the widget
  associated with it.
  */
  get widget() {
    return this._content instanceof ut ? this._content.widget : null;
  }
  /**
  If this is a textblock, this holds the number of line breaks
  that appear in widgets inside the block.
  */
  get widgetLineBreaks() {
    return typeof this._content == "number" ? this._content : 0;
  }
  /**
  @internal
  */
  join(t) {
    let e = (Array.isArray(this._content) ? this._content : [this]).concat(Array.isArray(t._content) ? t._content : [t]);
    return new tt(this.from, this.length + t.length, this.top, this.height + t.height, e);
  }
}
var O = /* @__PURE__ */ function(r) {
  return r[r.ByPos = 0] = "ByPos", r[r.ByHeight = 1] = "ByHeight", r[r.ByPosNoHeight = 2] = "ByPosNoHeight", r;
}(O || (O = {}));
const de = 1e-3;
class q {
  constructor(t, e, i = 2) {
    this.length = t, this.height = e, this.flags = i;
  }
  get outdated() {
    return (this.flags & 2) > 0;
  }
  set outdated(t) {
    this.flags = (t ? 2 : 0) | this.flags & -3;
  }
  setHeight(t) {
    this.height != t && (Math.abs(this.height - t) > de && (Rt = !0), this.height = t);
  }
  // Base case is to replace a leaf node, which simply builds a tree
  // from the new nodes and returns that (HeightMapBranch and
  // HeightMapGap override this to actually use from/to)
  replace(t, e, i) {
    return q.of(i);
  }
  // Again, these are base cases, and are overridden for branch and gap nodes.
  decomposeLeft(t, e) {
    e.push(this);
  }
  decomposeRight(t, e) {
    e.push(this);
  }
  applyChanges(t, e, i, n) {
    let s = this, o = i.doc;
    for (let l = n.length - 1; l >= 0; l--) {
      let { fromA: h, toA: f, fromB: a, toB: c } = n[l], u = s.lineAt(h, O.ByPosNoHeight, i.setDoc(e), 0, 0), d = u.to >= f ? u : s.lineAt(f, O.ByPosNoHeight, i, 0, 0);
      for (c += d.to - f, f = d.to; l > 0 && u.from <= n[l - 1].toA; )
        h = n[l - 1].fromA, a = n[l - 1].fromB, l--, h < u.from && (u = s.lineAt(h, O.ByPosNoHeight, i, 0, 0));
      a += u.from - h, h = u.from;
      let g = yi.build(i.setDoc(o), t, a, c);
      s = Se(s, s.replace(h, f, g));
    }
    return s.updateHeight(i, 0);
  }
  static empty() {
    return new $(0, 0);
  }
  // nodes uses null values to indicate the position of line breaks.
  // There are never line breaks at the start or end of the array, or
  // two line breaks next to each other, and the array isn't allowed
  // to be empty (same restrictions as return value from the builder).
  static of(t) {
    if (t.length == 1)
      return t[0];
    let e = 0, i = t.length, n = 0, s = 0;
    for (; ; )
      if (e == i)
        if (n > s * 2) {
          let l = t[e - 1];
          l.break ? t.splice(--e, 1, l.left, null, l.right) : t.splice(--e, 1, l.left, l.right), i += 1 + l.break, n -= l.size;
        } else if (s > n * 2) {
          let l = t[i];
          l.break ? t.splice(i, 1, l.left, null, l.right) : t.splice(i, 1, l.left, l.right), i += 2 + l.break, s -= l.size;
        } else
          break;
      else if (n < s) {
        let l = t[e++];
        l && (n += l.size);
      } else {
        let l = t[--i];
        l && (s += l.size);
      }
    let o = 0;
    return t[e - 1] == null ? (o = 1, e--) : t[e] == null && (o = 1, i++), new to(q.of(t.slice(0, e)), o, q.of(t.slice(i)));
  }
}
function Se(r, t) {
  return r == t ? r : (r.constructor != t.constructor && (Rt = !0), t);
}
q.prototype.size = 1;
class ks extends q {
  constructor(t, e, i) {
    super(t, e), this.deco = i;
  }
  blockAt(t, e, i, n) {
    return new tt(n, this.length, i, this.height, this.deco || 0);
  }
  lineAt(t, e, i, n, s) {
    return this.blockAt(0, i, n, s);
  }
  forEachLine(t, e, i, n, s, o) {
    t <= s + this.length && e >= s && o(this.blockAt(0, i, n, s));
  }
  updateHeight(t, e = 0, i = !1, n) {
    return n && n.from <= e && n.more && this.setHeight(n.heights[n.index++]), this.outdated = !1, this;
  }
  toString() {
    return `block(${this.length})`;
  }
}
class $ extends ks {
  constructor(t, e) {
    super(t, e, null), this.collapsed = 0, this.widgetHeight = 0, this.breaks = 0;
  }
  blockAt(t, e, i, n) {
    return new tt(n, this.length, i, this.height, this.breaks);
  }
  replace(t, e, i) {
    let n = i[0];
    return i.length == 1 && (n instanceof $ || n instanceof I && n.flags & 4) && Math.abs(this.length - n.length) < 10 ? (n instanceof I ? n = new $(n.length, this.height) : n.height = this.height, this.outdated || (n.outdated = !1), n) : q.of(i);
  }
  updateHeight(t, e = 0, i = !1, n) {
    return n && n.from <= e && n.more ? this.setHeight(n.heights[n.index++]) : (i || this.outdated) && this.setHeight(Math.max(this.widgetHeight, t.heightForLine(this.length - this.collapsed)) + this.breaks * t.lineHeight), this.outdated = !1, this;
  }
  toString() {
    return `line(${this.length}${this.collapsed ? -this.collapsed : ""}${this.widgetHeight ? ":" + this.widgetHeight : ""})`;
  }
}
class I extends q {
  constructor(t) {
    super(t, 0);
  }
  heightMetrics(t, e) {
    let i = t.doc.lineAt(e).number, n = t.doc.lineAt(e + this.length).number, s = n - i + 1, o, l = 0;
    if (t.lineWrapping) {
      let h = Math.min(this.height, t.lineHeight * s);
      o = h / s, this.length > s + 1 && (l = (this.height - h) / (this.length - s - 1));
    } else
      o = this.height / s;
    return { firstLine: i, lastLine: n, perLine: o, perChar: l };
  }
  blockAt(t, e, i, n) {
    let { firstLine: s, lastLine: o, perLine: l, perChar: h } = this.heightMetrics(e, n);
    if (e.lineWrapping) {
      let f = n + (t < e.lineHeight ? 0 : Math.round(Math.max(0, Math.min(1, (t - i) / this.height)) * this.length)), a = e.doc.lineAt(f), c = l + a.length * h, u = Math.max(i, t - c / 2);
      return new tt(a.from, a.length, u, c, 0);
    } else {
      let f = Math.max(0, Math.min(o - s, Math.floor((t - i) / l))), { from: a, length: c } = e.doc.line(s + f);
      return new tt(a, c, i + l * f, l, 0);
    }
  }
  lineAt(t, e, i, n, s) {
    if (e == O.ByHeight)
      return this.blockAt(t, i, n, s);
    if (e == O.ByPosNoHeight) {
      let { from: d, to: g } = i.doc.lineAt(t);
      return new tt(d, g - d, 0, 0, 0);
    }
    let { firstLine: o, perLine: l, perChar: h } = this.heightMetrics(i, s), f = i.doc.lineAt(t), a = l + f.length * h, c = f.number - o, u = n + l * c + h * (f.from - s - c);
    return new tt(f.from, f.length, Math.max(n, Math.min(u, n + this.height - a)), a, 0);
  }
  forEachLine(t, e, i, n, s, o) {
    t = Math.max(t, s), e = Math.min(e, s + this.length);
    let { firstLine: l, perLine: h, perChar: f } = this.heightMetrics(i, s);
    for (let a = t, c = n; a <= e; ) {
      let u = i.doc.lineAt(a);
      if (a == t) {
        let g = u.number - l;
        c += h * g + f * (t - s - g);
      }
      let d = h + f * u.length;
      o(new tt(u.from, u.length, c, d, 0)), c += d, a = u.to + 1;
    }
  }
  replace(t, e, i) {
    let n = this.length - e;
    if (n > 0) {
      let s = i[i.length - 1];
      s instanceof I ? i[i.length - 1] = new I(s.length + n) : i.push(null, new I(n - 1));
    }
    if (t > 0) {
      let s = i[0];
      s instanceof I ? i[0] = new I(t + s.length) : i.unshift(new I(t - 1), null);
    }
    return q.of(i);
  }
  decomposeLeft(t, e) {
    e.push(new I(t - 1), null);
  }
  decomposeRight(t, e) {
    e.push(null, new I(this.length - t - 1));
  }
  updateHeight(t, e = 0, i = !1, n) {
    let s = e + this.length;
    if (n && n.from <= e + this.length && n.more) {
      let o = [], l = Math.max(e, n.from), h = -1;
      for (n.from > e && o.push(new I(n.from - e - 1).updateHeight(t, e)); l <= s && n.more; ) {
        let a = t.doc.lineAt(l).length;
        o.length && o.push(null);
        let c = n.heights[n.index++];
        h == -1 ? h = c : Math.abs(c - h) >= de && (h = -2);
        let u = new $(a, c);
        u.outdated = !1, o.push(u), l += a + 1;
      }
      l <= s && o.push(null, new I(s - l).updateHeight(t, l));
      let f = q.of(o);
      return (h < 0 || Math.abs(f.height - this.height) >= de || Math.abs(h - this.heightMetrics(t, e).perLine) >= de) && (Rt = !0), Se(this, f);
    } else (i || this.outdated) && (this.setHeight(t.heightForGap(e, e + this.length)), this.outdated = !1);
    return this;
  }
  toString() {
    return `gap(${this.length})`;
  }
}
class to extends q {
  constructor(t, e, i) {
    super(t.length + e + i.length, t.height + i.height, e | (t.outdated || i.outdated ? 2 : 0)), this.left = t, this.right = i, this.size = t.size + i.size;
  }
  get break() {
    return this.flags & 1;
  }
  blockAt(t, e, i, n) {
    let s = i + this.left.height;
    return t < s ? this.left.blockAt(t, e, i, n) : this.right.blockAt(t, e, s, n + this.left.length + this.break);
  }
  lineAt(t, e, i, n, s) {
    let o = n + this.left.height, l = s + this.left.length + this.break, h = e == O.ByHeight ? t < o : t < l, f = h ? this.left.lineAt(t, e, i, n, s) : this.right.lineAt(t, e, i, o, l);
    if (this.break || (h ? f.to < l : f.from > l))
      return f;
    let a = e == O.ByPosNoHeight ? O.ByPosNoHeight : O.ByPos;
    return h ? f.join(this.right.lineAt(l, a, i, o, l)) : this.left.lineAt(l, a, i, n, s).join(f);
  }
  forEachLine(t, e, i, n, s, o) {
    let l = n + this.left.height, h = s + this.left.length + this.break;
    if (this.break)
      t < h && this.left.forEachLine(t, e, i, n, s, o), e >= h && this.right.forEachLine(t, e, i, l, h, o);
    else {
      let f = this.lineAt(h, O.ByPos, i, n, s);
      t < f.from && this.left.forEachLine(t, f.from - 1, i, n, s, o), f.to >= t && f.from <= e && o(f), e > f.to && this.right.forEachLine(f.to + 1, e, i, l, h, o);
    }
  }
  replace(t, e, i) {
    let n = this.left.length + this.break;
    if (e < n)
      return this.balanced(this.left.replace(t, e, i), this.right);
    if (t > this.left.length)
      return this.balanced(this.left, this.right.replace(t - n, e - n, i));
    let s = [];
    t > 0 && this.decomposeLeft(t, s);
    let o = s.length;
    for (let l of i)
      s.push(l);
    if (t > 0 && sn(s, o - 1), e < this.length) {
      let l = s.length;
      this.decomposeRight(e, s), sn(s, l);
    }
    return q.of(s);
  }
  decomposeLeft(t, e) {
    let i = this.left.length;
    if (t <= i)
      return this.left.decomposeLeft(t, e);
    e.push(this.left), this.break && (i++, t >= i && e.push(null)), t > i && this.right.decomposeLeft(t - i, e);
  }
  decomposeRight(t, e) {
    let i = this.left.length, n = i + this.break;
    if (t >= n)
      return this.right.decomposeRight(t - n, e);
    t < i && this.left.decomposeRight(t, e), this.break && t < n && e.push(null), e.push(this.right);
  }
  balanced(t, e) {
    return t.size > 2 * e.size || e.size > 2 * t.size ? q.of(this.break ? [t, null, e] : [t, e]) : (this.left = Se(this.left, t), this.right = Se(this.right, e), this.setHeight(t.height + e.height), this.outdated = t.outdated || e.outdated, this.size = t.size + e.size, this.length = t.length + this.break + e.length, this);
  }
  updateHeight(t, e = 0, i = !1, n) {
    let { left: s, right: o } = this, l = e + s.length + this.break, h = null;
    return n && n.from <= e + s.length && n.more ? h = s = s.updateHeight(t, e, i, n) : s.updateHeight(t, e, i), n && n.from <= l + o.length && n.more ? h = o = o.updateHeight(t, l, i, n) : o.updateHeight(t, l, i), h ? this.balanced(s, o) : (this.height = this.left.height + this.right.height, this.outdated = !1, this);
  }
  toString() {
    return this.left + (this.break ? " " : "-") + this.right;
  }
}
function sn(r, t) {
  let e, i;
  r[t] == null && (e = r[t - 1]) instanceof I && (i = r[t + 1]) instanceof I && r.splice(t - 1, 3, new I(e.length + 1 + i.length));
}
const eo = 5;
class yi {
  constructor(t, e) {
    this.pos = t, this.oracle = e, this.nodes = [], this.lineStart = -1, this.lineEnd = -1, this.covering = null, this.writtenTo = t;
  }
  get isCovered() {
    return this.covering && this.nodes[this.nodes.length - 1] == this.covering;
  }
  span(t, e) {
    if (this.lineStart > -1) {
      let i = Math.min(e, this.lineEnd), n = this.nodes[this.nodes.length - 1];
      n instanceof $ ? n.length += i - this.pos : (i > this.pos || !this.isCovered) && this.nodes.push(new $(i - this.pos, -1)), this.writtenTo = i, e > i && (this.nodes.push(null), this.writtenTo++, this.lineStart = -1);
    }
    this.pos = e;
  }
  point(t, e, i) {
    if (t < e || i.heightRelevant) {
      let n = i.widget ? i.widget.estimatedHeight : 0, s = i.widget ? i.widget.lineBreaks : 0;
      n < 0 && (n = this.oracle.lineHeight);
      let o = e - t;
      i.block ? this.addBlock(new ks(o, n, i)) : (o || s || n >= eo) && this.addLineDeco(n, s, o);
    } else e > t && this.span(t, e);
    this.lineEnd > -1 && this.lineEnd < this.pos && (this.lineEnd = this.oracle.doc.lineAt(this.pos).to);
  }
  enterLine() {
    if (this.lineStart > -1)
      return;
    let { from: t, to: e } = this.oracle.doc.lineAt(this.pos);
    this.lineStart = t, this.lineEnd = e, this.writtenTo < t && ((this.writtenTo < t - 1 || this.nodes[this.nodes.length - 1] == null) && this.nodes.push(this.blankContent(this.writtenTo, t - 1)), this.nodes.push(null)), this.pos > t && this.nodes.push(new $(this.pos - t, -1)), this.writtenTo = this.pos;
  }
  blankContent(t, e) {
    let i = new I(e - t);
    return this.oracle.doc.lineAt(t).to == e && (i.flags |= 4), i;
  }
  ensureLine() {
    this.enterLine();
    let t = this.nodes.length ? this.nodes[this.nodes.length - 1] : null;
    if (t instanceof $)
      return t;
    let e = new $(0, -1);
    return this.nodes.push(e), e;
  }
  addBlock(t) {
    this.enterLine();
    let e = t.deco;
    e && e.startSide > 0 && !this.isCovered && this.ensureLine(), this.nodes.push(t), this.writtenTo = this.pos = this.pos + t.length, e && e.endSide > 0 && (this.covering = t);
  }
  addLineDeco(t, e, i) {
    let n = this.ensureLine();
    n.length += i, n.collapsed += i, n.widgetHeight = Math.max(n.widgetHeight, t), n.breaks += e, this.writtenTo = this.pos = this.pos + i;
  }
  finish(t) {
    let e = this.nodes.length == 0 ? null : this.nodes[this.nodes.length - 1];
    this.lineStart > -1 && !(e instanceof $) && !this.isCovered ? this.nodes.push(new $(0, -1)) : (this.writtenTo < this.pos || e == null) && this.nodes.push(this.blankContent(this.writtenTo, this.pos));
    let i = t;
    for (let n of this.nodes)
      n instanceof $ && n.updateHeight(this.oracle, i), i += n ? n.length : 1;
    return this.nodes;
  }
  // Always called with a region that on both sides either stretches
  // to a line break or the end of the document.
  // The returned array uses null to indicate line breaks, but never
  // starts or ends in a line break, or has multiple line breaks next
  // to each other.
  static build(t, e, i, n) {
    let s = new yi(i, t);
    return R.spans(e, i, n, s, 0), s.finish(i);
  }
}
function io(r, t, e) {
  let i = new no();
  return R.compare(r, t, e, i, 0), i.changes;
}
class no {
  constructor() {
    this.changes = [];
  }
  compareRange() {
  }
  comparePoint(t, e, i, n) {
    (t < e || i && i.heightRelevant || n && n.heightRelevant) && Ze(t, e, this.changes, 5);
  }
}
function so(r, t) {
  let e = r.getBoundingClientRect(), i = r.ownerDocument, n = i.defaultView || window, s = Math.max(0, e.left), o = Math.min(n.innerWidth, e.right), l = Math.max(0, e.top), h = Math.min(n.innerHeight, e.bottom);
  for (let f = r.parentNode; f && f != i.body; )
    if (f.nodeType == 1) {
      let a = f, c = window.getComputedStyle(a);
      if ((a.scrollHeight > a.clientHeight || a.scrollWidth > a.clientWidth) && c.overflow != "visible") {
        let u = a.getBoundingClientRect();
        s = Math.max(s, u.left), o = Math.min(o, u.right), l = Math.max(l, u.top), h = f == r.parentNode ? u.bottom : Math.min(h, u.bottom);
      }
      f = c.position == "absolute" || c.position == "fixed" ? a.offsetParent : a.parentNode;
    } else if (f.nodeType == 11)
      f = f.host;
    else
      break;
  return {
    left: s - e.left,
    right: Math.max(s, o) - e.left,
    top: l - (e.top + t),
    bottom: Math.max(l, h) - (e.top + t)
  };
}
function ro(r, t) {
  let e = r.getBoundingClientRect();
  return {
    left: 0,
    right: e.right - e.left,
    top: t,
    bottom: e.bottom - (e.top + t)
  };
}
class Ee {
  constructor(t, e, i) {
    this.from = t, this.to = e, this.size = i;
  }
  static same(t, e) {
    if (t.length != e.length)
      return !1;
    for (let i = 0; i < t.length; i++) {
      let n = t[i], s = e[i];
      if (n.from != s.from || n.to != s.to || n.size != s.size)
        return !1;
    }
    return !0;
  }
  draw(t, e) {
    return Y.replace({
      widget: new oo(this.size * (e ? t.scaleY : t.scaleX), e)
    }).range(this.from, this.to);
  }
}
class oo extends di {
  constructor(t, e) {
    super(), this.size = t, this.vertical = e;
  }
  eq(t) {
    return t.size == this.size && t.vertical == this.vertical;
  }
  toDOM() {
    let t = document.createElement("div");
    return this.vertical ? t.style.height = this.size + "px" : (t.style.width = this.size + "px", t.style.height = "2px", t.style.display = "inline-block"), t;
  }
  get estimatedHeight() {
    return this.vertical ? this.size : -1;
  }
}
class rn {
  constructor(t) {
    this.state = t, this.pixelViewport = { left: 0, right: window.innerWidth, top: 0, bottom: 0 }, this.inView = !0, this.paddingTop = 0, this.paddingBottom = 0, this.contentDOMWidth = 0, this.contentDOMHeight = 0, this.editorHeight = 0, this.editorWidth = 0, this.scrollTop = 0, this.scrolledToBottom = !1, this.scaleX = 1, this.scaleY = 1, this.scrollAnchorPos = 0, this.scrollAnchorHeight = -1, this.scaler = on, this.scrollTarget = null, this.printing = !1, this.mustMeasureContent = !0, this.defaultTextDirection = z.LTR, this.visibleRanges = [], this.mustEnforceCursorAssoc = !1;
    let e = t.facet(pi).some((i) => typeof i != "function" && i.class == "cm-lineWrapping");
    this.heightOracle = new Zr(e), this.stateDeco = t.facet(Jt).filter((i) => typeof i != "function"), this.heightMap = q.empty().applyChanges(this.stateDeco, A.empty, this.heightOracle.setDoc(t.doc), [new U(0, 0, 0, t.doc.length)]);
    for (let i = 0; i < 2 && (this.viewport = this.getViewport(0, null), !!this.updateForViewport()); i++)
      ;
    this.updateViewportLines(), this.lineGaps = this.ensureLineGaps([]), this.lineGapDeco = Y.set(this.lineGaps.map((i) => i.draw(this, !1))), this.computeVisibleRanges();
  }
  updateForViewport() {
    let t = [this.viewport], { main: e } = this.state.selection;
    for (let i = 0; i <= 1; i++) {
      let n = i ? e.head : e.anchor;
      if (!t.some(({ from: s, to: o }) => n >= s && n <= o)) {
        let { from: s, to: o } = this.lineBlockAt(n);
        t.push(new re(s, o));
      }
    }
    return this.viewports = t.sort((i, n) => i.from - n.from), this.updateScaler();
  }
  updateScaler() {
    let t = this.scaler;
    return this.scaler = this.heightMap.height <= 7e6 ? on : new wi(this.heightOracle, this.heightMap, this.viewports), t.eq(this.scaler) ? 0 : 2;
  }
  updateViewportLines() {
    this.viewportLines = [], this.heightMap.forEachLine(this.viewport.from, this.viewport.to, this.heightOracle.setDoc(this.state.doc), 0, 0, (t) => {
      this.viewportLines.push(Ht(t, this.scaler));
    });
  }
  update(t, e = null) {
    this.state = t.state;
    let i = this.stateDeco;
    this.stateDeco = this.state.facet(Jt).filter((a) => typeof a != "function");
    let n = t.changedRanges, s = U.extendWithRanges(n, io(i, this.stateDeco, t ? t.changes : L.empty(this.state.doc.length))), o = this.heightMap.height, l = this.scrolledToBottom ? null : this.scrollAnchorAt(this.scrollTop);
    nn(), this.heightMap = this.heightMap.applyChanges(this.stateDeco, t.startState.doc, this.heightOracle.setDoc(this.state.doc), s), (this.heightMap.height != o || Rt) && (t.flags |= 2), l ? (this.scrollAnchorPos = t.changes.mapPos(l.from, -1), this.scrollAnchorHeight = l.top) : (this.scrollAnchorPos = -1, this.scrollAnchorHeight = this.heightMap.height);
    let h = s.length ? this.mapViewport(this.viewport, t.changes) : this.viewport;
    (e && (e.range.head < h.from || e.range.head > h.to) || !this.viewportIsAppropriate(h)) && (h = this.getViewport(0, e));
    let f = h.from != this.viewport.from || h.to != this.viewport.to;
    this.viewport = h, t.flags |= this.updateForViewport(), (f || !t.changes.empty || t.flags & 2) && this.updateViewportLines(), (this.lineGaps.length || this.viewport.to - this.viewport.from > 4e3) && this.updateLineGaps(this.ensureLineGaps(this.mapLineGaps(this.lineGaps, t.changes))), t.flags |= this.computeVisibleRanges(), e && (this.scrollTarget = e), !this.mustEnforceCursorAssoc && t.selectionSet && t.view.lineWrapping && t.state.selection.main.empty && t.state.selection.main.assoc && !t.state.facet(mr) && (this.mustEnforceCursorAssoc = !0);
  }
  measure(t) {
    let e = t.contentDOM, i = window.getComputedStyle(e), n = this.heightOracle, s = i.whiteSpace;
    this.defaultTextDirection = i.direction == "rtl" ? z.RTL : z.LTR;
    let o = this.heightOracle.mustRefreshForWrapping(s), l = e.getBoundingClientRect(), h = o || this.mustMeasureContent || this.contentDOMHeight != l.height;
    this.contentDOMHeight = l.height, this.mustMeasureContent = !1;
    let f = 0, a = 0;
    if (l.width && l.height) {
      let { scaleX: x, scaleY: E } = Ln(e, l);
      (x > 5e-3 && Math.abs(this.scaleX - x) > 5e-3 || E > 5e-3 && Math.abs(this.scaleY - E) > 5e-3) && (this.scaleX = x, this.scaleY = E, f |= 8, o = h = !0);
    }
    let c = (parseInt(i.paddingTop) || 0) * this.scaleY, u = (parseInt(i.paddingBottom) || 0) * this.scaleY;
    (this.paddingTop != c || this.paddingBottom != u) && (this.paddingTop = c, this.paddingBottom = u, f |= 10), this.editorWidth != t.scrollDOM.clientWidth && (n.lineWrapping && (h = !0), this.editorWidth = t.scrollDOM.clientWidth, f |= 8);
    let d = t.scrollDOM.scrollTop * this.scaleY;
    this.scrollTop != d && (this.scrollAnchorHeight = -1, this.scrollTop = d), this.scrolledToBottom = Nn(t.scrollDOM);
    let g = (this.printing ? ro : so)(e, this.paddingTop), p = g.top - this.pixelViewport.top, m = g.bottom - this.pixelViewport.bottom;
    this.pixelViewport = g;
    let y = this.pixelViewport.bottom > this.pixelViewport.top && this.pixelViewport.right > this.pixelViewport.left;
    if (y != this.inView && (this.inView = y, y && (h = !0)), !this.inView && !this.scrollTarget)
      return 0;
    let S = l.width;
    if ((this.contentDOMWidth != S || this.editorHeight != t.scrollDOM.clientHeight) && (this.contentDOMWidth = l.width, this.editorHeight = t.scrollDOM.clientHeight, f |= 8), h) {
      let x = t.docView.measureVisibleLineHeights(this.viewport);
      if (n.mustRefreshForHeights(x) && (o = !0), o || n.lineWrapping && Math.abs(S - this.contentDOMWidth) > n.charWidth) {
        let { lineHeight: E, charWidth: G, textHeight: Ts } = t.docView.measureTextSize();
        o = E > 0 && n.refresh(s, E, G, Ts, S / G, x), o && (t.docView.minWidth = 0, f |= 8);
      }
      p > 0 && m > 0 ? a = Math.max(p, m) : p < 0 && m < 0 && (a = Math.min(p, m)), nn();
      for (let E of this.viewports) {
        let G = E.from == this.viewport.from ? x : t.docView.measureVisibleLineHeights(E);
        this.heightMap = (o ? q.empty().applyChanges(this.stateDeco, A.empty, this.heightOracle, [new U(0, 0, 0, t.state.doc.length)]) : this.heightMap).updateHeight(n, 0, o, new _r(E.from, G));
      }
      Rt && (f |= 2);
    }
    let k = !this.viewportIsAppropriate(this.viewport, a) || this.scrollTarget && (this.scrollTarget.range.head < this.viewport.from || this.scrollTarget.range.head > this.viewport.to);
    return k && (f & 2 && (f |= this.updateScaler()), this.viewport = this.getViewport(a, this.scrollTarget), f |= this.updateForViewport()), (f & 2 || k) && this.updateViewportLines(), (this.lineGaps.length || this.viewport.to - this.viewport.from > 4e3) && this.updateLineGaps(this.ensureLineGaps(o ? [] : this.lineGaps, t)), f |= this.computeVisibleRanges(), this.mustEnforceCursorAssoc && (this.mustEnforceCursorAssoc = !1, t.docView.enforceCursorAssoc()), f;
  }
  get visibleTop() {
    return this.scaler.fromDOM(this.pixelViewport.top);
  }
  get visibleBottom() {
    return this.scaler.fromDOM(this.pixelViewport.bottom);
  }
  getViewport(t, e) {
    let i = 0.5 - Math.max(-0.5, Math.min(0.5, t / 1e3 / 2)), n = this.heightMap, s = this.heightOracle, { visibleTop: o, visibleBottom: l } = this, h = new re(n.lineAt(o - i * 1e3, O.ByHeight, s, 0, 0).from, n.lineAt(l + (1 - i) * 1e3, O.ByHeight, s, 0, 0).to);
    if (e) {
      let { head: f } = e.range;
      if (f < h.from || f > h.to) {
        let a = Math.min(this.editorHeight, this.pixelViewport.bottom - this.pixelViewport.top), c = n.lineAt(f, O.ByPos, s, 0, 0), u;
        e.y == "center" ? u = (c.top + c.bottom) / 2 - a / 2 : e.y == "start" || e.y == "nearest" && f < h.from ? u = c.top : u = c.bottom - a, h = new re(n.lineAt(u - 1e3 / 2, O.ByHeight, s, 0, 0).from, n.lineAt(u + a + 1e3 / 2, O.ByHeight, s, 0, 0).to);
      }
    }
    return h;
  }
  mapViewport(t, e) {
    let i = e.mapPos(t.from, -1), n = e.mapPos(t.to, 1);
    return new re(this.heightMap.lineAt(i, O.ByPos, this.heightOracle, 0, 0).from, this.heightMap.lineAt(n, O.ByPos, this.heightOracle, 0, 0).to);
  }
  // Checks if a given viewport covers the visible part of the
  // document and not too much beyond that.
  viewportIsAppropriate({ from: t, to: e }, i = 0) {
    if (!this.inView)
      return !0;
    let { top: n } = this.heightMap.lineAt(t, O.ByPos, this.heightOracle, 0, 0), { bottom: s } = this.heightMap.lineAt(e, O.ByPos, this.heightOracle, 0, 0), { visibleTop: o, visibleBottom: l } = this;
    return (t == 0 || n <= o - Math.max(10, Math.min(
      -i,
      250
      /* VP.MaxCoverMargin */
    ))) && (e == this.state.doc.length || s >= l + Math.max(10, Math.min(
      i,
      250
      /* VP.MaxCoverMargin */
    ))) && n > o - 2 * 1e3 && s < l + 2 * 1e3;
  }
  mapLineGaps(t, e) {
    if (!t.length || e.empty)
      return t;
    let i = [];
    for (let n of t)
      e.touchesRange(n.from, n.to) || i.push(new Ee(e.mapPos(n.from), e.mapPos(n.to), n.size));
    return i;
  }
  // Computes positions in the viewport where the start or end of a
  // line should be hidden, trying to reuse existing line gaps when
  // appropriate to avoid unneccesary redraws.
  // Uses crude character-counting for the positioning and sizing,
  // since actual DOM coordinates aren't always available and
  // predictable. Relies on generous margins (see LG.Margin) to hide
  // the artifacts this might produce from the user.
  ensureLineGaps(t, e) {
    let i = this.heightOracle.lineWrapping, n = i ? 1e4 : 2e3, s = n >> 1, o = n << 1;
    if (this.defaultTextDirection != z.LTR && !i)
      return [];
    let l = [], h = (a, c, u, d) => {
      if (c - a < s)
        return;
      let g = this.state.selection.main, p = [g.from];
      g.empty || p.push(g.to);
      for (let y of p)
        if (y > a && y < c) {
          h(a, y - 10, u, d), h(y + 10, c, u, d);
          return;
        }
      let m = ho(t, (y) => y.from >= u.from && y.to <= u.to && Math.abs(y.from - a) < s && Math.abs(y.to - c) < s && !p.some((S) => y.from < S && y.to > S));
      if (!m) {
        if (c < u.to && e && i && e.visibleRanges.some((y) => y.from <= c && y.to >= c)) {
          let y = e.moveToLineBoundary(b.cursor(c), !1, !0).head;
          y > a && (c = y);
        }
        m = new Ee(a, c, this.gapSize(u, a, c, d));
      }
      l.push(m);
    }, f = (a) => {
      if (a.length < o || a.type != et.Text)
        return;
      let c = lo(a.from, a.to, this.stateDeco);
      if (c.total < o)
        return;
      let u = this.scrollTarget ? this.scrollTarget.range.head : null, d, g;
      if (i) {
        let p = n / this.heightOracle.lineLength * this.heightOracle.lineHeight, m, y;
        if (u != null) {
          let S = le(c, u), k = ((this.visibleBottom - this.visibleTop) / 2 + p) / a.height;
          m = S - k, y = S + k;
        } else
          m = (this.visibleTop - a.top - p) / a.height, y = (this.visibleBottom - a.top + p) / a.height;
        d = oe(c, m), g = oe(c, y);
      } else {
        let p = c.total * this.heightOracle.charWidth, m = n * this.heightOracle.charWidth, y, S;
        if (u != null) {
          let k = le(c, u), x = ((this.pixelViewport.right - this.pixelViewport.left) / 2 + m) / p;
          y = k - x, S = k + x;
        } else
          y = (this.pixelViewport.left - m) / p, S = (this.pixelViewport.right + m) / p;
        d = oe(c, y), g = oe(c, S);
      }
      d > a.from && h(a.from, d, a, c), g < a.to && h(g, a.to, a, c);
    };
    for (let a of this.viewportLines)
      Array.isArray(a.type) ? a.type.forEach(f) : f(a);
    return l;
  }
  gapSize(t, e, i, n) {
    let s = le(n, i) - le(n, e);
    return this.heightOracle.lineWrapping ? t.height * s : n.total * this.heightOracle.charWidth * s;
  }
  updateLineGaps(t) {
    Ee.same(t, this.lineGaps) || (this.lineGaps = t, this.lineGapDeco = Y.set(t.map((e) => e.draw(this, this.heightOracle.lineWrapping))));
  }
  computeVisibleRanges() {
    let t = this.stateDeco;
    this.lineGaps.length && (t = t.concat(this.lineGapDeco));
    let e = [];
    R.spans(t, this.viewport.from, this.viewport.to, {
      span(n, s) {
        e.push({ from: n, to: s });
      },
      point() {
      }
    }, 20);
    let i = e.length != this.visibleRanges.length || this.visibleRanges.some((n, s) => n.from != e[s].from || n.to != e[s].to);
    return this.visibleRanges = e, i ? 4 : 0;
  }
  lineBlockAt(t) {
    return t >= this.viewport.from && t <= this.viewport.to && this.viewportLines.find((e) => e.from <= t && e.to >= t) || Ht(this.heightMap.lineAt(t, O.ByPos, this.heightOracle, 0, 0), this.scaler);
  }
  lineBlockAtHeight(t) {
    return t >= this.viewportLines[0].top && t <= this.viewportLines[this.viewportLines.length - 1].bottom && this.viewportLines.find((e) => e.top <= t && e.bottom >= t) || Ht(this.heightMap.lineAt(this.scaler.fromDOM(t), O.ByHeight, this.heightOracle, 0, 0), this.scaler);
  }
  scrollAnchorAt(t) {
    let e = this.lineBlockAtHeight(t + 8);
    return e.from >= this.viewport.from || this.viewportLines[0].top - t > 200 ? e : this.viewportLines[0];
  }
  elementAtHeight(t) {
    return Ht(this.heightMap.blockAt(this.scaler.fromDOM(t), this.heightOracle, 0, 0), this.scaler);
  }
  get docHeight() {
    return this.scaler.toDOM(this.heightMap.height);
  }
  get contentHeight() {
    return this.docHeight + this.paddingTop + this.paddingBottom;
  }
}
class re {
  constructor(t, e) {
    this.from = t, this.to = e;
  }
}
function lo(r, t, e) {
  let i = [], n = r, s = 0;
  return R.spans(e, r, t, {
    span() {
    },
    point(o, l) {
      o > n && (i.push({ from: n, to: o }), s += o - n), n = l;
    }
  }, 20), n < t && (i.push({ from: n, to: t }), s += t - n), { total: s, ranges: i };
}
function oe({ total: r, ranges: t }, e) {
  if (e <= 0)
    return t[0].from;
  if (e >= 1)
    return t[t.length - 1].to;
  let i = Math.floor(r * e);
  for (let n = 0; ; n++) {
    let { from: s, to: o } = t[n], l = o - s;
    if (i <= l)
      return s + i;
    i -= l;
  }
}
function le(r, t) {
  let e = 0;
  for (let { from: i, to: n } of r.ranges) {
    if (t <= n) {
      e += t - i;
      break;
    }
    e += n - i;
  }
  return e / r.total;
}
function ho(r, t) {
  for (let e of r)
    if (t(e))
      return e;
}
const on = {
  toDOM(r) {
    return r;
  },
  fromDOM(r) {
    return r;
  },
  scale: 1,
  eq(r) {
    return r == this;
  }
};
class wi {
  constructor(t, e, i) {
    let n = 0, s = 0, o = 0;
    this.viewports = i.map(({ from: l, to: h }) => {
      let f = e.lineAt(l, O.ByPos, t, 0, 0).top, a = e.lineAt(h, O.ByPos, t, 0, 0).bottom;
      return n += a - f, { from: l, to: h, top: f, bottom: a, domTop: 0, domBottom: 0 };
    }), this.scale = (7e6 - n) / (e.height - n);
    for (let l of this.viewports)
      l.domTop = o + (l.top - s) * this.scale, o = l.domBottom = l.domTop + (l.bottom - l.top), s = l.bottom;
  }
  toDOM(t) {
    for (let e = 0, i = 0, n = 0; ; e++) {
      let s = e < this.viewports.length ? this.viewports[e] : null;
      if (!s || t < s.top)
        return n + (t - i) * this.scale;
      if (t <= s.bottom)
        return s.domTop + (t - s.top);
      i = s.bottom, n = s.domBottom;
    }
  }
  fromDOM(t) {
    for (let e = 0, i = 0, n = 0; ; e++) {
      let s = e < this.viewports.length ? this.viewports[e] : null;
      if (!s || t < s.domTop)
        return i + (t - n) / this.scale;
      if (t <= s.domBottom)
        return s.top + (t - s.domTop);
      i = s.bottom, n = s.domBottom;
    }
  }
  eq(t) {
    return t instanceof wi ? this.scale == t.scale && this.viewports.length == t.viewports.length && this.viewports.every((e, i) => e.from == t.viewports[i].from && e.to == t.viewports[i].to) : !1;
  }
}
function Ht(r, t) {
  if (t.scale == 1)
    return r;
  let e = t.toDOM(r.top), i = t.toDOM(r.bottom);
  return new tt(r.from, r.length, e, i - e, Array.isArray(r._content) ? r._content.map((n) => Ht(n, t)) : r._content);
}
const he = /* @__PURE__ */ v.define({ combine: (r) => r.join(" ") }), ri = /* @__PURE__ */ v.define({ combine: (r) => r.indexOf(!0) > -1 }), oi = /* @__PURE__ */ Mt.newName(), As = /* @__PURE__ */ Mt.newName(), Cs = /* @__PURE__ */ Mt.newName(), Ms = { "&light": "." + As, "&dark": "." + Cs };
function li(r, t, e) {
  return new Mt(t, {
    finish(i) {
      return /&/.test(i) ? i.replace(/&\w*/, (n) => {
        if (n == "&")
          return r;
        if (!e || !e[n])
          throw new RangeError(`Unsupported selector: ${n}`);
        return e[n];
      }) : r + " " + i;
    }
  });
}
const ao = /* @__PURE__ */ li("." + oi, {
  "&": {
    position: "relative !important",
    boxSizing: "border-box",
    "&.cm-focused": {
      // Provide a simple default outline to make sure a focused
      // editor is visually distinct. Can't leave the default behavior
      // because that will apply to the content element, which is
      // inside the scrollable container and doesn't include the
      // gutters. We also can't use an 'auto' outline, since those
      // are, for some reason, drawn behind the element content, which
      // will cause things like the active line background to cover
      // the outline (#297).
      outline: "1px dotted #212121"
    },
    display: "flex !important",
    flexDirection: "column"
  },
  ".cm-scroller": {
    display: "flex !important",
    alignItems: "flex-start !important",
    fontFamily: "monospace",
    lineHeight: 1.4,
    height: "100%",
    overflowX: "auto",
    position: "relative",
    zIndex: 0
  },
  ".cm-content": {
    margin: 0,
    flexGrow: 2,
    flexShrink: 0,
    display: "block",
    whiteSpace: "pre",
    wordWrap: "normal",
    // https://github.com/codemirror/dev/issues/456
    boxSizing: "border-box",
    minHeight: "100%",
    padding: "4px 0",
    outline: "none",
    "&[contenteditable=true]": {
      WebkitUserModify: "read-write-plaintext-only"
    }
  },
  ".cm-lineWrapping": {
    whiteSpace_fallback: "pre-wrap",
    // For IE
    whiteSpace: "break-spaces",
    wordBreak: "break-word",
    // For Safari, which doesn't support overflow-wrap: anywhere
    overflowWrap: "anywhere",
    flexShrink: 1
  },
  "&light .cm-content": { caretColor: "black" },
  "&dark .cm-content": { caretColor: "white" },
  ".cm-line": {
    display: "block",
    padding: "0 2px 0 6px"
  },
  ".cm-layer": {
    position: "absolute",
    left: 0,
    top: 0,
    contain: "size style",
    "& > *": {
      position: "absolute"
    }
  },
  "&light .cm-selectionBackground": {
    background: "#d9d9d9"
  },
  "&dark .cm-selectionBackground": {
    background: "#222"
  },
  "&light.cm-focused > .cm-scroller > .cm-selectionLayer .cm-selectionBackground": {
    background: "#d7d4f0"
  },
  "&dark.cm-focused > .cm-scroller > .cm-selectionLayer .cm-selectionBackground": {
    background: "#233"
  },
  ".cm-cursorLayer": {
    pointerEvents: "none"
  },
  "&.cm-focused > .cm-scroller > .cm-cursorLayer": {
    animation: "steps(1) cm-blink 1.2s infinite"
  },
  // Two animations defined so that we can switch between them to
  // restart the animation without forcing another style
  // recomputation.
  "@keyframes cm-blink": { "0%": {}, "50%": { opacity: 0 }, "100%": {} },
  "@keyframes cm-blink2": { "0%": {}, "50%": { opacity: 0 }, "100%": {} },
  ".cm-cursor, .cm-dropCursor": {
    borderLeft: "1.2px solid black",
    marginLeft: "-0.6px",
    pointerEvents: "none"
  },
  ".cm-cursor": {
    display: "none"
  },
  "&dark .cm-cursor": {
    borderLeftColor: "#444"
  },
  ".cm-dropCursor": {
    position: "absolute"
  },
  "&.cm-focused > .cm-scroller > .cm-cursorLayer .cm-cursor": {
    display: "block"
  },
  ".cm-iso": {
    unicodeBidi: "isolate"
  },
  ".cm-announced": {
    position: "fixed",
    top: "-10000px"
  },
  "@media print": {
    ".cm-announced": { display: "none" }
  },
  "&light .cm-activeLine": { backgroundColor: "#cceeff44" },
  "&dark .cm-activeLine": { backgroundColor: "#99eeff33" },
  "&light .cm-specialChar": { color: "red" },
  "&dark .cm-specialChar": { color: "#f78" },
  ".cm-gutters": {
    flexShrink: 0,
    display: "flex",
    height: "100%",
    boxSizing: "border-box",
    insetInlineStart: 0,
    zIndex: 200
  },
  "&light .cm-gutters": {
    backgroundColor: "#f5f5f5",
    color: "#6c6c6c",
    borderRight: "1px solid #ddd"
  },
  "&dark .cm-gutters": {
    backgroundColor: "#333338",
    color: "#ccc"
  },
  ".cm-gutter": {
    display: "flex !important",
    // Necessary -- prevents margin collapsing
    flexDirection: "column",
    flexShrink: 0,
    boxSizing: "border-box",
    minHeight: "100%",
    overflow: "hidden"
  },
  ".cm-gutterElement": {
    boxSizing: "border-box"
  },
  ".cm-lineNumbers .cm-gutterElement": {
    padding: "0 3px 0 5px",
    minWidth: "20px",
    textAlign: "right",
    whiteSpace: "nowrap"
  },
  "&light .cm-activeLineGutter": {
    backgroundColor: "#e2f2ff"
  },
  "&dark .cm-activeLineGutter": {
    backgroundColor: "#222227"
  },
  ".cm-panels": {
    boxSizing: "border-box",
    position: "sticky",
    left: 0,
    right: 0
  },
  "&light .cm-panels": {
    backgroundColor: "#f5f5f5",
    color: "black"
  },
  "&light .cm-panels-top": {
    borderBottom: "1px solid #ddd"
  },
  "&light .cm-panels-bottom": {
    borderTop: "1px solid #ddd"
  },
  "&dark .cm-panels": {
    backgroundColor: "#333338",
    color: "white"
  },
  ".cm-tab": {
    display: "inline-block",
    overflow: "hidden",
    verticalAlign: "bottom"
  },
  ".cm-widgetBuffer": {
    verticalAlign: "text-top",
    height: "1em",
    width: 0,
    display: "inline"
  },
  ".cm-placeholder": {
    color: "#888",
    display: "inline-block",
    verticalAlign: "top"
  },
  ".cm-highlightSpace:before": {
    content: "attr(data-display)",
    position: "absolute",
    pointerEvents: "none",
    color: "#888"
  },
  ".cm-highlightTab": {
    backgroundImage: `url('data:image/svg+xml,<svg xmlns="http://www.w3.org/2000/svg" width="200" height="20"><path stroke="%23888" stroke-width="1" fill="none" d="M1 10H196L190 5M190 15L196 10M197 4L197 16"/></svg>')`,
    backgroundSize: "auto 100%",
    backgroundPosition: "right 90%",
    backgroundRepeat: "no-repeat"
  },
  ".cm-trailingSpace": {
    backgroundColor: "#ff332255"
  },
  ".cm-button": {
    verticalAlign: "middle",
    color: "inherit",
    fontSize: "70%",
    padding: ".2em 1em",
    borderRadius: "1px"
  },
  "&light .cm-button": {
    backgroundImage: "linear-gradient(#eff1f5, #d9d9df)",
    border: "1px solid #888",
    "&:active": {
      backgroundImage: "linear-gradient(#b4b4b4, #d0d3d6)"
    }
  },
  "&dark .cm-button": {
    backgroundImage: "linear-gradient(#393939, #111)",
    border: "1px solid #888",
    "&:active": {
      backgroundImage: "linear-gradient(#111, #333)"
    }
  },
  ".cm-textfield": {
    verticalAlign: "middle",
    color: "inherit",
    fontSize: "70%",
    border: "1px solid silver",
    padding: ".2em .5em"
  },
  "&light .cm-textfield": {
    backgroundColor: "white"
  },
  "&dark .cm-textfield": {
    border: "1px solid #555",
    backgroundColor: "inherit"
  }
}, Ms), Vt = "￿";
class fo {
  constructor(t, e) {
    this.points = t, this.text = "", this.lineSeparator = e.facet(T.lineSeparator);
  }
  append(t) {
    this.text += t;
  }
  lineBreak() {
    this.text += Vt;
  }
  readRange(t, e) {
    if (!t)
      return this;
    let i = t.parentNode;
    for (let n = t; ; ) {
      this.findPointBefore(i, n);
      let s = this.text.length;
      this.readNode(n);
      let o = n.nextSibling;
      if (o == e)
        break;
      let l = M.get(n), h = M.get(o);
      (l && h ? l.breakAfter : (l ? l.breakAfter : ye(n)) || ye(o) && (n.nodeName != "BR" || n.cmIgnore) && this.text.length > s) && this.lineBreak(), n = o;
    }
    return this.findPointBefore(i, e), this;
  }
  readTextNode(t) {
    let e = t.nodeValue;
    for (let i of this.points)
      i.node == t && (i.pos = this.text.length + Math.min(i.offset, e.length));
    for (let i = 0, n = this.lineSeparator ? null : /\r\n?|\n/g; ; ) {
      let s = -1, o = 1, l;
      if (this.lineSeparator ? (s = e.indexOf(this.lineSeparator, i), o = this.lineSeparator.length) : (l = n.exec(e)) && (s = l.index, o = l[0].length), this.append(e.slice(i, s < 0 ? e.length : s)), s < 0)
        break;
      if (this.lineBreak(), o > 1)
        for (let h of this.points)
          h.node == t && h.pos > this.text.length && (h.pos -= o - 1);
      i = s + o;
    }
  }
  readNode(t) {
    if (t.cmIgnore)
      return;
    let e = M.get(t), i = e && e.overrideDOMText;
    if (i != null) {
      this.findPointInside(t, i.length);
      for (let n = i.iter(); !n.next().done; )
        n.lineBreak ? this.lineBreak() : this.append(n.value);
    } else t.nodeType == 3 ? this.readTextNode(t) : t.nodeName == "BR" ? t.nextSibling && this.lineBreak() : t.nodeType == 1 && this.readRange(t.firstChild, null);
  }
  findPointBefore(t, e) {
    for (let i of this.points)
      i.node == t && t.childNodes[i.offset] == e && (i.pos = this.text.length);
  }
  findPointInside(t, e) {
    for (let i of this.points)
      (t.nodeType == 3 ? i.node == t : t.contains(i.node)) && (i.pos = this.text.length + (co(t, i.node, i.offset) ? e : 0));
  }
}
function co(r, t, e) {
  for (; ; ) {
    if (!t || e < lt(t))
      return !1;
    if (t == r)
      return !0;
    e = mt(t) + 1, t = t.parentNode;
  }
}
class ln {
  constructor(t, e) {
    this.node = t, this.offset = e, this.pos = -1;
  }
}
class uo {
  constructor(t, e, i, n) {
    this.typeOver = n, this.bounds = null, this.text = "", this.domChanged = e > -1;
    let { impreciseHead: s, impreciseAnchor: o } = t.docView;
    if (t.state.readOnly && e > -1)
      this.newSel = null;
    else if (e > -1 && (this.bounds = t.docView.domBoundsAround(e, i, 0))) {
      let l = s || o ? [] : mo(t), h = new fo(l, t.state);
      h.readRange(this.bounds.startDOM, this.bounds.endDOM), this.text = h.text, this.newSel = yo(l, this.bounds.from);
    } else {
      let l = t.observer.selectionRange, h = s && s.node == l.focusNode && s.offset == l.focusOffset || !Ye(t.contentDOM, l.focusNode) ? t.state.selection.main.head : t.docView.posFromDOM(l.focusNode, l.focusOffset), f = o && o.node == l.anchorNode && o.offset == l.anchorOffset || !Ye(t.contentDOM, l.anchorNode) ? t.state.selection.main.anchor : t.docView.posFromDOM(l.anchorNode, l.anchorOffset), a = t.viewport;
      if ((w.ios || w.chrome) && t.state.selection.main.empty && h != f && (a.from > 0 || a.to < t.state.doc.length)) {
        let c = Math.min(h, f), u = Math.max(h, f), d = a.from - c, g = a.to - u;
        (d == 0 || d == 1 || c == 0) && (g == 0 || g == -1 || u == t.state.doc.length) && (h = 0, f = t.state.doc.length);
      }
      this.newSel = b.single(f, h);
    }
  }
}
function Ds(r, t) {
  let e, { newSel: i } = t, n = r.state.selection.main, s = r.inputState.lastKeyTime > Date.now() - 100 ? r.inputState.lastKeyCode : -1;
  if (t.bounds) {
    let { from: o, to: l } = t.bounds, h = n.from, f = null;
    (s === 8 || w.android && t.text.length < l - o) && (h = n.to, f = "end");
    let a = po(r.state.doc.sliceString(o, l, Vt), t.text, h - o, f);
    a && (w.chrome && s == 13 && a.toB == a.from + 2 && t.text.slice(a.from, a.toB) == Vt + Vt && a.toB--, e = {
      from: o + a.from,
      to: o + a.toA,
      insert: A.of(t.text.slice(a.from, a.toB).split(Vt))
    });
  } else i && (!r.hasFocus && r.state.facet(at) || i.main.eq(n)) && (i = null);
  if (!e && !i)
    return !1;
  if (!e && t.typeOver && !n.empty && i && i.main.empty ? e = { from: n.from, to: n.to, insert: r.state.doc.slice(n.from, n.to) } : e && e.from >= n.from && e.to <= n.to && (e.from != n.from || e.to != n.to) && n.to - n.from - (e.to - e.from) <= 4 ? e = {
    from: n.from,
    to: n.to,
    insert: r.state.doc.slice(n.from, e.from).append(e.insert).append(r.state.doc.slice(e.to, n.to))
  } : (w.mac || w.android) && e && e.from == e.to && e.from == n.head - 1 && /^\. ?$/.test(e.insert.toString()) && r.contentDOM.getAttribute("autocorrect") == "off" ? (i && e.insert.length == 2 && (i = b.single(i.main.anchor - 1, i.main.head - 1)), e = { from: n.from, to: n.to, insert: A.of([" "]) }) : w.chrome && e && e.from == e.to && e.from == n.head && e.insert.toString() == `
 ` && r.lineWrapping && (i && (i = b.single(i.main.anchor - 1, i.main.head - 1)), e = { from: n.from, to: n.to, insert: A.of([" "]) }), e)
    return Os(r, e, i, s);
  if (i && !i.main.eq(n)) {
    let o = !1, l = "select";
    return r.inputState.lastSelectionTime > Date.now() - 50 && (r.inputState.lastSelectionOrigin == "select" && (o = !0), l = r.inputState.lastSelectionOrigin), r.dispatch({ selection: i, scrollIntoView: o, userEvent: l }), !0;
  } else
    return !1;
}
function Os(r, t, e, i = -1) {
  if (w.ios && r.inputState.flushIOSKey(t))
    return !0;
  let n = r.state.selection.main;
  if (w.android && (t.to == n.to && // GBoard will sometimes remove a space it just inserted
  // after a completion when you press enter
  (t.from == n.from || t.from == n.from - 1 && r.state.sliceDoc(t.from, n.from) == " ") && t.insert.length == 1 && t.insert.lines == 2 && vt(r.contentDOM, "Enter", 13) || (t.from == n.from - 1 && t.to == n.to && t.insert.length == 0 || i == 8 && t.insert.length < t.to - t.from && t.to > n.head) && vt(r.contentDOM, "Backspace", 8) || t.from == n.from && t.to == n.to + 1 && t.insert.length == 0 && vt(r.contentDOM, "Delete", 46)))
    return !0;
  let s = t.insert.toString();
  r.inputState.composing >= 0 && r.inputState.composing++;
  let o, l = () => o || (o = go(r, t, e));
  return r.state.facet(ss).some((h) => h(r, t.from, t.to, s, l)) || r.dispatch(l()), !0;
}
function go(r, t, e) {
  let i, n = r.state, s = n.selection.main;
  if (t.from >= s.from && t.to <= s.to && t.to - t.from >= (s.to - s.from) / 3 && (!e || e.main.empty && e.main.from == t.from + t.insert.length) && r.inputState.composing < 0) {
    let l = s.from < t.from ? n.sliceDoc(s.from, t.from) : "", h = s.to > t.to ? n.sliceDoc(t.to, s.to) : "";
    i = n.replaceSelection(r.state.toText(l + t.insert.sliceString(0, void 0, r.state.lineBreak) + h));
  } else {
    let l = n.changes(t), h = e && e.main.to <= l.newLength ? e.main : void 0;
    if (n.selection.ranges.length > 1 && r.inputState.composing >= 0 && t.to <= s.to && t.to >= s.to - 10) {
      let f = r.state.sliceDoc(t.from, t.to), a, c = e && gs(r, e.main.head);
      if (c) {
        let g = t.insert.length - (t.to - t.from);
        a = { from: c.from, to: c.to - g };
      } else
        a = r.state.doc.lineAt(s.head);
      let u = s.to - t.to, d = s.to - s.from;
      i = n.changeByRange((g) => {
        if (g.from == s.from && g.to == s.to)
          return { changes: l, range: h || g.map(l) };
        let p = g.to - u, m = p - f.length;
        if (g.to - g.from != d || r.state.sliceDoc(m, p) != f || // Unfortunately, there's no way to make multiple
        // changes in the same node work without aborting
        // composition, so cursors in the composition range are
        // ignored.
        g.to >= a.from && g.from <= a.to)
          return { range: g };
        let y = n.changes({ from: m, to: p, insert: t.insert }), S = g.to - s.to;
        return {
          changes: y,
          range: h ? b.range(Math.max(0, h.anchor + S), Math.max(0, h.head + S)) : g.map(y)
        };
      });
    } else
      i = {
        changes: l,
        selection: h && n.selection.replaceRange(h)
      };
  }
  let o = "input.type";
  return (r.composing || r.inputState.compositionPendingChange && r.inputState.compositionEndedAt > Date.now() - 50) && (r.inputState.compositionPendingChange = !1, o += ".compose", r.inputState.compositionFirstChange && (o += ".start", r.inputState.compositionFirstChange = !1)), n.update(i, { userEvent: o, scrollIntoView: !0 });
}
function po(r, t, e, i) {
  let n = Math.min(r.length, t.length), s = 0;
  for (; s < n && r.charCodeAt(s) == t.charCodeAt(s); )
    s++;
  if (s == n && r.length == t.length)
    return null;
  let o = r.length, l = t.length;
  for (; o > 0 && l > 0 && r.charCodeAt(o - 1) == t.charCodeAt(l - 1); )
    o--, l--;
  if (i == "end") {
    let h = Math.max(0, s - Math.min(o, l));
    e -= o + h - s;
  }
  if (o < s && r.length < t.length) {
    let h = e <= s && e >= o ? s - e : 0;
    s -= h, l = s + (l - o), o = s;
  } else if (l < s) {
    let h = e <= s && e >= l ? s - e : 0;
    s -= h, o = s + (o - l), l = s;
  }
  return { from: s, toA: o, toB: l };
}
function mo(r) {
  let t = [];
  if (r.root.activeElement != r.contentDOM)
    return t;
  let { anchorNode: e, anchorOffset: i, focusNode: n, focusOffset: s } = r.observer.selectionRange;
  return e && (t.push(new ln(e, i)), (n != e || s != i) && t.push(new ln(n, s))), t;
}
function yo(r, t) {
  if (r.length == 0)
    return null;
  let e = r[0].pos, i = r.length == 2 ? r[1].pos : e;
  return e > -1 && i > -1 ? b.single(e + t, i + t) : null;
}
const wo = {
  childList: !0,
  characterData: !0,
  subtree: !0,
  attributes: !0,
  characterDataOldValue: !0
}, Pe = w.ie && w.ie_version <= 11;
class bo {
  constructor(t) {
    this.view = t, this.active = !1, this.editContext = null, this.selectionRange = new _s(), this.selectionChanged = !1, this.delayedFlush = -1, this.resizeTimeout = -1, this.queue = [], this.delayedAndroidKey = null, this.flushingAndroidKey = -1, this.lastChange = 0, this.scrollTargets = [], this.intersection = null, this.resizeScroll = null, this.intersecting = !1, this.gapIntersection = null, this.gaps = [], this.printQuery = null, this.parentCheck = -1, this.dom = t.contentDOM, this.observer = new MutationObserver((e) => {
      for (let i of e)
        this.queue.push(i);
      (w.ie && w.ie_version <= 11 || w.ios && t.composing) && e.some((i) => i.type == "childList" && i.removedNodes.length || i.type == "characterData" && i.oldValue.length > i.target.nodeValue.length) ? this.flushSoon() : this.flush();
    }), window.EditContext && t.constructor.EDIT_CONTEXT !== !1 && // Chrome <126 doesn't support inverted selections in edit context (#1392)
    !(w.chrome && w.chrome_version < 126) && (this.editContext = new xo(t), t.state.facet(at) && (t.contentDOM.editContext = this.editContext.editContext)), Pe && (this.onCharData = (e) => {
      this.queue.push({
        target: e.target,
        type: "characterData",
        oldValue: e.prevValue
      }), this.flushSoon();
    }), this.onSelectionChange = this.onSelectionChange.bind(this), this.onResize = this.onResize.bind(this), this.onPrint = this.onPrint.bind(this), this.onScroll = this.onScroll.bind(this), window.matchMedia && (this.printQuery = window.matchMedia("print")), typeof ResizeObserver == "function" && (this.resizeScroll = new ResizeObserver(() => {
      var e;
      ((e = this.view.docView) === null || e === void 0 ? void 0 : e.lastUpdate) < Date.now() - 75 && this.onResize();
    }), this.resizeScroll.observe(t.scrollDOM)), this.addWindowListeners(this.win = t.win), this.start(), typeof IntersectionObserver == "function" && (this.intersection = new IntersectionObserver((e) => {
      this.parentCheck < 0 && (this.parentCheck = setTimeout(this.listenForScroll.bind(this), 1e3)), e.length > 0 && e[e.length - 1].intersectionRatio > 0 != this.intersecting && (this.intersecting = !this.intersecting, this.intersecting != this.view.inView && this.onScrollChanged(document.createEvent("Event")));
    }, { threshold: [0, 1e-3] }), this.intersection.observe(this.dom), this.gapIntersection = new IntersectionObserver((e) => {
      e.length > 0 && e[e.length - 1].intersectionRatio > 0 && this.onScrollChanged(document.createEvent("Event"));
    }, {})), this.listenForScroll(), this.readSelectionRange();
  }
  onScrollChanged(t) {
    this.view.inputState.runHandlers("scroll", t), this.intersecting && this.view.measure();
  }
  onScroll(t) {
    this.intersecting && this.flush(!1), this.editContext && this.view.requestMeasure(this.editContext.measureReq), this.onScrollChanged(t);
  }
  onResize() {
    this.resizeTimeout < 0 && (this.resizeTimeout = setTimeout(() => {
      this.resizeTimeout = -1, this.view.requestMeasure();
    }, 50));
  }
  onPrint(t) {
    (t.type == "change" || !t.type) && !t.matches || (this.view.viewState.printing = !0, this.view.measure(), setTimeout(() => {
      this.view.viewState.printing = !1, this.view.requestMeasure();
    }, 500));
  }
  updateGaps(t) {
    if (this.gapIntersection && (t.length != this.gaps.length || this.gaps.some((e, i) => e != t[i]))) {
      this.gapIntersection.disconnect();
      for (let e of t)
        this.gapIntersection.observe(e);
      this.gaps = t;
    }
  }
  onSelectionChange(t) {
    let e = this.selectionChanged;
    if (!this.readSelectionRange() || this.delayedAndroidKey)
      return;
    let { view: i } = this, n = this.selectionRange;
    if (i.state.facet(at) ? i.root.activeElement != this.dom : !ce(i.dom, n))
      return;
    let s = n.anchorNode && i.docView.nearest(n.anchorNode);
    if (s && s.ignoreEvent(t)) {
      e || (this.selectionChanged = !1);
      return;
    }
    (w.ie && w.ie_version <= 11 || w.android && w.chrome) && !i.state.selection.main.empty && // (Selection.isCollapsed isn't reliable on IE)
    n.focusNode && qt(n.focusNode, n.focusOffset, n.anchorNode, n.anchorOffset) ? this.flushSoon() : this.flush(!1);
  }
  readSelectionRange() {
    let { view: t } = this, e = Gt(t.root);
    if (!e)
      return !1;
    let i = w.safari && t.root.nodeType == 11 && Xs(this.dom.ownerDocument) == this.dom && So(this.view, e) || e;
    if (!i || this.selectionRange.eq(i))
      return !1;
    let n = ce(this.dom, i);
    return n && !this.selectionChanged && t.inputState.lastFocusTime > Date.now() - 200 && t.inputState.lastTouchTime < Date.now() - 300 && er(this.dom, i) ? (this.view.inputState.lastFocusTime = 0, t.docView.updateSelection(), !1) : (this.selectionRange.setRange(i), n && (this.selectionChanged = !0), !0);
  }
  setSelectionRange(t, e) {
    this.selectionRange.set(t.node, t.offset, e.node, e.offset), this.selectionChanged = !1;
  }
  clearSelectionRange() {
    this.selectionRange.set(null, 0, null, 0);
  }
  listenForScroll() {
    this.parentCheck = -1;
    let t = 0, e = null;
    for (let i = this.dom; i; )
      if (i.nodeType == 1)
        !e && t < this.scrollTargets.length && this.scrollTargets[t] == i ? t++ : e || (e = this.scrollTargets.slice(0, t)), e && e.push(i), i = i.assignedSlot || i.parentNode;
      else if (i.nodeType == 11)
        i = i.host;
      else
        break;
    if (t < this.scrollTargets.length && !e && (e = this.scrollTargets.slice(0, t)), e) {
      for (let i of this.scrollTargets)
        i.removeEventListener("scroll", this.onScroll);
      for (let i of this.scrollTargets = e)
        i.addEventListener("scroll", this.onScroll);
    }
  }
  ignore(t) {
    if (!this.active)
      return t();
    try {
      return this.stop(), t();
    } finally {
      this.start(), this.clear();
    }
  }
  start() {
    this.active || (this.observer.observe(this.dom, wo), Pe && this.dom.addEventListener("DOMCharacterDataModified", this.onCharData), this.active = !0);
  }
  stop() {
    this.active && (this.active = !1, this.observer.disconnect(), Pe && this.dom.removeEventListener("DOMCharacterDataModified", this.onCharData));
  }
  // Throw away any pending changes
  clear() {
    this.processRecords(), this.queue.length = 0, this.selectionChanged = !1;
  }
  // Chrome Android, especially in combination with GBoard, not only
  // doesn't reliably fire regular key events, but also often
  // surrounds the effect of enter or backspace with a bunch of
  // composition events that, when interrupted, cause text duplication
  // or other kinds of corruption. This hack makes the editor back off
  // from handling DOM changes for a moment when such a key is
  // detected (via beforeinput or keydown), and then tries to flush
  // them or, if that has no effect, dispatches the given key.
  delayAndroidKey(t, e) {
    var i;
    if (!this.delayedAndroidKey) {
      let n = () => {
        let s = this.delayedAndroidKey;
        s && (this.clearDelayedAndroidKey(), this.view.inputState.lastKeyCode = s.keyCode, this.view.inputState.lastKeyTime = Date.now(), !this.flush() && s.force && vt(this.dom, s.key, s.keyCode));
      };
      this.flushingAndroidKey = this.view.win.requestAnimationFrame(n);
    }
    (!this.delayedAndroidKey || t == "Enter") && (this.delayedAndroidKey = {
      key: t,
      keyCode: e,
      // Only run the key handler when no changes are detected if
      // this isn't coming right after another change, in which case
      // it is probably part of a weird chain of updates, and should
      // be ignored if it returns the DOM to its previous state.
      force: this.lastChange < Date.now() - 50 || !!(!((i = this.delayedAndroidKey) === null || i === void 0) && i.force)
    });
  }
  clearDelayedAndroidKey() {
    this.win.cancelAnimationFrame(this.flushingAndroidKey), this.delayedAndroidKey = null, this.flushingAndroidKey = -1;
  }
  flushSoon() {
    this.delayedFlush < 0 && (this.delayedFlush = this.view.win.requestAnimationFrame(() => {
      this.delayedFlush = -1, this.flush();
    }));
  }
  forceFlush() {
    this.delayedFlush >= 0 && (this.view.win.cancelAnimationFrame(this.delayedFlush), this.delayedFlush = -1), this.flush();
  }
  pendingRecords() {
    for (let t of this.observer.takeRecords())
      this.queue.push(t);
    return this.queue;
  }
  processRecords() {
    let t = this.pendingRecords();
    t.length && (this.queue = []);
    let e = -1, i = -1, n = !1;
    for (let s of t) {
      let o = this.readMutation(s);
      o && (o.typeOver && (n = !0), e == -1 ? { from: e, to: i } = o : (e = Math.min(o.from, e), i = Math.max(o.to, i)));
    }
    return { from: e, to: i, typeOver: n };
  }
  readChange() {
    let { from: t, to: e, typeOver: i } = this.processRecords(), n = this.selectionChanged && ce(this.dom, this.selectionRange);
    if (t < 0 && !n)
      return null;
    t > -1 && (this.lastChange = Date.now()), this.view.inputState.lastFocusTime = 0, this.selectionChanged = !1;
    let s = new uo(this.view, t, e, i);
    return this.view.docView.domChanged = { newSel: s.newSel ? s.newSel.main : null }, s;
  }
  // Apply pending changes, if any
  flush(t = !0) {
    if (this.delayedFlush >= 0 || this.delayedAndroidKey)
      return !1;
    t && this.readSelectionRange();
    let e = this.readChange();
    if (!e)
      return this.view.requestMeasure(), !1;
    let i = this.view.state, n = Ds(this.view, e);
    return this.view.state == i && (e.domChanged || e.newSel && !e.newSel.main.eq(this.view.state.selection.main)) && this.view.update([]), n;
  }
  readMutation(t) {
    let e = this.view.docView.nearest(t.target);
    if (!e || e.ignoreMutation(t))
      return null;
    if (e.markDirty(t.type == "attributes"), t.type == "attributes" && (e.flags |= 4), t.type == "childList") {
      let i = hn(e, t.previousSibling || t.target.previousSibling, -1), n = hn(e, t.nextSibling || t.target.nextSibling, 1);
      return {
        from: i ? e.posAfter(i) : e.posAtStart,
        to: n ? e.posBefore(n) : e.posAtEnd,
        typeOver: !1
      };
    } else return t.type == "characterData" ? { from: e.posAtStart, to: e.posAtEnd, typeOver: t.target.nodeValue == t.oldValue } : null;
  }
  setWindow(t) {
    t != this.win && (this.removeWindowListeners(this.win), this.win = t, this.addWindowListeners(this.win));
  }
  addWindowListeners(t) {
    t.addEventListener("resize", this.onResize), this.printQuery ? this.printQuery.addEventListener ? this.printQuery.addEventListener("change", this.onPrint) : this.printQuery.addListener(this.onPrint) : t.addEventListener("beforeprint", this.onPrint), t.addEventListener("scroll", this.onScroll), t.document.addEventListener("selectionchange", this.onSelectionChange);
  }
  removeWindowListeners(t) {
    t.removeEventListener("scroll", this.onScroll), t.removeEventListener("resize", this.onResize), this.printQuery ? this.printQuery.removeEventListener ? this.printQuery.removeEventListener("change", this.onPrint) : this.printQuery.removeListener(this.onPrint) : t.removeEventListener("beforeprint", this.onPrint), t.document.removeEventListener("selectionchange", this.onSelectionChange);
  }
  update(t) {
    this.editContext && (this.editContext.update(t), t.startState.facet(at) != t.state.facet(at) && (t.view.contentDOM.editContext = t.state.facet(at) ? this.editContext.editContext : null));
  }
  destroy() {
    var t, e, i;
    this.stop(), (t = this.intersection) === null || t === void 0 || t.disconnect(), (e = this.gapIntersection) === null || e === void 0 || e.disconnect(), (i = this.resizeScroll) === null || i === void 0 || i.disconnect();
    for (let n of this.scrollTargets)
      n.removeEventListener("scroll", this.onScroll);
    this.removeWindowListeners(this.win), clearTimeout(this.parentCheck), clearTimeout(this.resizeTimeout), this.win.cancelAnimationFrame(this.delayedFlush), this.win.cancelAnimationFrame(this.flushingAndroidKey), this.editContext && (this.view.contentDOM.editContext = null, this.editContext.destroy());
  }
}
function hn(r, t, e) {
  for (; t; ) {
    let i = M.get(t);
    if (i && i.parent == r)
      return i;
    let n = t.parentNode;
    t = n != r.dom ? n : e > 0 ? t.nextSibling : t.previousSibling;
  }
  return null;
}
function an(r, t) {
  let e = t.startContainer, i = t.startOffset, n = t.endContainer, s = t.endOffset, o = r.docView.domAtPos(r.state.selection.main.anchor);
  return qt(o.node, o.offset, n, s) && ([e, i, n, s] = [n, s, e, i]), { anchorNode: e, anchorOffset: i, focusNode: n, focusOffset: s };
}
function So(r, t) {
  if (t.getComposedRanges) {
    let n = t.getComposedRanges(r.root)[0];
    if (n)
      return an(r, n);
  }
  let e = null;
  function i(n) {
    n.preventDefault(), n.stopImmediatePropagation(), e = n.getTargetRanges()[0];
  }
  return r.contentDOM.addEventListener("beforeinput", i, !0), r.dom.ownerDocument.execCommand("indent"), r.contentDOM.removeEventListener("beforeinput", i, !0), e ? an(r, e) : null;
}
class xo {
  constructor(t) {
    this.from = 0, this.to = 0, this.pendingContextChange = null, this.handlers = /* @__PURE__ */ Object.create(null), this.resetRange(t.state);
    let e = this.editContext = new window.EditContext({
      text: t.state.doc.sliceString(this.from, this.to),
      selectionStart: this.toContextPos(Math.max(this.from, Math.min(this.to, t.state.selection.main.anchor))),
      selectionEnd: this.toContextPos(t.state.selection.main.head)
    });
    this.handlers.textupdate = (i) => {
      let { anchor: n } = t.state.selection.main, s = {
        from: this.toEditorPos(i.updateRangeStart),
        to: this.toEditorPos(i.updateRangeEnd),
        insert: A.of(i.text.split(`
`))
      };
      s.from == this.from && n < this.from ? s.from = n : s.to == this.to && n > this.to && (s.to = n), !(s.from == s.to && !s.insert.length) && (this.pendingContextChange = s, t.state.readOnly || Os(t, s, b.single(this.toEditorPos(i.selectionStart), this.toEditorPos(i.selectionEnd))), this.pendingContextChange && (this.revertPending(t.state), this.setSelection(t.state)));
    }, this.handlers.characterboundsupdate = (i) => {
      let n = [], s = null;
      for (let o = this.toEditorPos(i.rangeStart), l = this.toEditorPos(i.rangeEnd); o < l; o++) {
        let h = t.coordsForChar(o);
        s = h && new DOMRect(h.left, h.top, h.right - h.left, h.bottom - h.top) || s || new DOMRect(), n.push(s);
      }
      e.updateCharacterBounds(i.rangeStart, n);
    }, this.handlers.textformatupdate = (i) => {
      let n = [];
      for (let s of i.getTextFormats()) {
        let o = s.underlineStyle, l = s.underlineThickness;
        if (o != "None" && l != "None") {
          let h = `text-decoration: underline ${o == "Dashed" ? "dashed " : o == "Squiggle" ? "wavy " : ""}${l == "Thin" ? 1 : 2}px`;
          n.push(Y.mark({ attributes: { style: h } }).range(this.toEditorPos(s.rangeStart), this.toEditorPos(s.rangeEnd)));
        }
      }
      t.dispatch({ effects: hs.of(Y.set(n)) });
    }, this.handlers.compositionstart = () => {
      t.inputState.composing < 0 && (t.inputState.composing = 0, t.inputState.compositionFirstChange = !0);
    }, this.handlers.compositionend = () => {
      t.inputState.composing = -1, t.inputState.compositionFirstChange = null;
    };
    for (let i in this.handlers)
      e.addEventListener(i, this.handlers[i]);
    this.measureReq = { read: (i) => {
      this.editContext.updateControlBounds(i.contentDOM.getBoundingClientRect());
      let n = Gt(i.root);
      n && n.rangeCount && this.editContext.updateSelectionBounds(n.getRangeAt(0).getBoundingClientRect());
    } };
  }
  applyEdits(t) {
    let e = 0, i = !1, n = this.pendingContextChange;
    return t.changes.iterChanges((s, o, l, h, f) => {
      if (i)
        return;
      let a = f.length - (o - s);
      if (n && o >= n.to)
        if (n.from == s && n.to == o && n.insert.eq(f)) {
          n = this.pendingContextChange = null, e += a, this.to += a;
          return;
        } else
          n = null, this.revertPending(t.state);
      if (s += e, o += e, o <= this.from)
        this.from += a, this.to += a;
      else if (s < this.to) {
        if (s < this.from || o > this.to || this.to - this.from + f.length > 3e4) {
          i = !0;
          return;
        }
        this.editContext.updateText(this.toContextPos(s), this.toContextPos(o), f.toString()), this.to += a;
      }
      e += a;
    }), n && !i && this.revertPending(t.state), !i;
  }
  update(t) {
    let e = this.pendingContextChange;
    !this.applyEdits(t) || !this.rangeIsValid(t.state) ? (this.pendingContextChange = null, this.resetRange(t.state), this.editContext.updateText(0, this.editContext.text.length, t.state.doc.sliceString(this.from, this.to)), this.setSelection(t.state)) : (t.docChanged || t.selectionSet || e) && this.setSelection(t.state), (t.geometryChanged || t.docChanged || t.selectionSet) && t.view.requestMeasure(this.measureReq);
  }
  resetRange(t) {
    let { head: e } = t.selection.main;
    this.from = Math.max(
      0,
      e - 1e4
      /* CxVp.Margin */
    ), this.to = Math.min(
      t.doc.length,
      e + 1e4
      /* CxVp.Margin */
    );
  }
  revertPending(t) {
    let e = this.pendingContextChange;
    this.pendingContextChange = null, this.editContext.updateText(this.toContextPos(e.from), this.toContextPos(e.from + e.insert.length), t.doc.sliceString(e.from, e.to));
  }
  setSelection(t) {
    let { main: e } = t.selection, i = this.toContextPos(Math.max(this.from, Math.min(this.to, e.anchor))), n = this.toContextPos(e.head);
    (this.editContext.selectionStart != i || this.editContext.selectionEnd != n) && this.editContext.updateSelection(i, n);
  }
  rangeIsValid(t) {
    let { head: e } = t.selection.main;
    return !(this.from > 0 && e - this.from < 500 || this.to < t.doc.length && this.to - e < 500 || this.to - this.from > 1e4 * 3);
  }
  toEditorPos(t) {
    return t + this.from;
  }
  toContextPos(t) {
    return t - this.from;
  }
  destroy() {
    for (let t in this.handlers)
      this.editContext.removeEventListener(t, this.handlers[t]);
  }
}
class D {
  /**
  The current editor state.
  */
  get state() {
    return this.viewState.state;
  }
  /**
  To be able to display large documents without consuming too much
  memory or overloading the browser, CodeMirror only draws the
  code that is visible (plus a margin around it) to the DOM. This
  property tells you the extent of the current drawn viewport, in
  document positions.
  */
  get viewport() {
    return this.viewState.viewport;
  }
  /**
  When there are, for example, large collapsed ranges in the
  viewport, its size can be a lot bigger than the actual visible
  content. Thus, if you are doing something like styling the
  content in the viewport, it is preferable to only do so for
  these ranges, which are the subset of the viewport that is
  actually drawn.
  */
  get visibleRanges() {
    return this.viewState.visibleRanges;
  }
  /**
  Returns false when the editor is entirely scrolled out of view
  or otherwise hidden.
  */
  get inView() {
    return this.viewState.inView;
  }
  /**
  Indicates whether the user is currently composing text via
  [IME](https://en.wikipedia.org/wiki/Input_method), and at least
  one change has been made in the current composition.
  */
  get composing() {
    return this.inputState.composing > 0;
  }
  /**
  Indicates whether the user is currently in composing state. Note
  that on some platforms, like Android, this will be the case a
  lot, since just putting the cursor on a word starts a
  composition there.
  */
  get compositionStarted() {
    return this.inputState.composing >= 0;
  }
  /**
  The document or shadow root that the view lives in.
  */
  get root() {
    return this._root;
  }
  /**
  @internal
  */
  get win() {
    return this.dom.ownerDocument.defaultView || window;
  }
  /**
  Construct a new view. You'll want to either provide a `parent`
  option, or put `view.dom` into your document after creating a
  view, so that the user can see the editor.
  */
  constructor(t = {}) {
    this.plugins = [], this.pluginMap = /* @__PURE__ */ new Map(), this.editorAttrs = {}, this.contentAttrs = {}, this.bidiCache = [], this.destroyed = !1, this.updateState = 2, this.measureScheduled = -1, this.measureRequests = [], this.contentDOM = document.createElement("div"), this.scrollDOM = document.createElement("div"), this.scrollDOM.tabIndex = -1, this.scrollDOM.className = "cm-scroller", this.scrollDOM.appendChild(this.contentDOM), this.announceDOM = document.createElement("div"), this.announceDOM.className = "cm-announced", this.announceDOM.setAttribute("aria-live", "polite"), this.dom = document.createElement("div"), this.dom.appendChild(this.announceDOM), this.dom.appendChild(this.scrollDOM), t.parent && t.parent.appendChild(this.dom);
    let { dispatch: e } = t;
    this.dispatchTransactions = t.dispatchTransactions || e && ((i) => i.forEach((n) => e(n, this))) || ((i) => this.update(i)), this.dispatch = this.dispatch.bind(this), this._root = t.root || tr(t.parent) || document, this.viewState = new rn(t.state || T.create(t)), t.scrollTo && t.scrollTo.is(ie) && (this.viewState.scrollTarget = t.scrollTo.value.clip(this.viewState.state)), this.plugins = this.state.facet(It).map((i) => new Oe(i));
    for (let i of this.plugins)
      i.update(this);
    this.observer = new bo(this), this.inputState = new Lr(this), this.inputState.ensureHandlers(this.plugins), this.docView = new Vi(this), this.mountStyles(), this.updateAttrs(), this.updateState = 0, this.requestMeasure();
  }
  dispatch(...t) {
    let e = t.length == 1 && t[0] instanceof W ? t : t.length == 1 && Array.isArray(t[0]) ? t[0] : [this.state.update(...t)];
    this.dispatchTransactions(e, this);
  }
  /**
  Update the view for the given array of transactions. This will
  update the visible document and selection to match the state
  produced by the transactions, and notify view plugins of the
  change. You should usually call
  [`dispatch`](https://codemirror.net/6/docs/ref/#view.EditorView.dispatch) instead, which uses this
  as a primitive.
  */
  update(t) {
    if (this.updateState != 0)
      throw new Error("Calls to EditorView.update are not allowed while an update is in progress");
    let e = !1, i = !1, n, s = this.state;
    for (let u of t) {
      if (u.startState != s)
        throw new RangeError("Trying to update state with a transaction that doesn't start from the previous state.");
      s = u.state;
    }
    if (this.destroyed) {
      this.viewState.state = s;
      return;
    }
    let o = this.hasFocus, l = 0, h = null;
    t.some((u) => u.annotation(Ss)) ? (this.inputState.notifiedFocused = o, l = 1) : o != this.inputState.notifiedFocused && (this.inputState.notifiedFocused = o, h = xs(s, o), h || (l = 1));
    let f = this.observer.delayedAndroidKey, a = null;
    if (f ? (this.observer.clearDelayedAndroidKey(), a = this.observer.readChange(), (a && !this.state.doc.eq(s.doc) || !this.state.selection.eq(s.selection)) && (a = null)) : this.observer.clear(), s.facet(T.phrases) != this.state.facet(T.phrases))
      return this.setState(s);
    n = be.create(this, s, t), n.flags |= l;
    let c = this.viewState.scrollTarget;
    try {
      this.updateState = 2;
      for (let u of t) {
        if (c && (c = c.map(u.changes)), u.scrollIntoView) {
          let { main: d } = u.state.selection;
          c = new kt(d.empty ? d : b.cursor(d.head, d.head > d.anchor ? -1 : 1));
        }
        for (let d of u.effects)
          d.is(ie) && (c = d.value.clip(this.state));
      }
      this.viewState.update(n, c), this.bidiCache = xe.update(this.bidiCache, n.changes), n.empty || (this.updatePlugins(n), this.inputState.update(n)), e = this.docView.update(n), this.state.facet(Nt) != this.styleModules && this.mountStyles(), i = this.updateAttrs(), this.showAnnouncements(t), this.docView.updateSelection(e, t.some((u) => u.isUserEvent("select.pointer")));
    } finally {
      this.updateState = 0;
    }
    if (n.startState.facet(he) != n.state.facet(he) && (this.viewState.mustMeasureContent = !0), (e || i || c || this.viewState.mustEnforceCursorAssoc || this.viewState.mustMeasureContent) && this.requestMeasure(), e && this.docViewUpdate(), !n.empty)
      for (let u of this.state.facet(ii))
        try {
          u(n);
        } catch (d) {
          st(this.state, d, "update listener");
        }
    (h || a) && Promise.resolve().then(() => {
      h && this.state == h.startState && this.dispatch(h), a && !Ds(this, a) && f.force && vt(this.contentDOM, f.key, f.keyCode);
    });
  }
  /**
  Reset the view to the given state. (This will cause the entire
  document to be redrawn and all view plugins to be reinitialized,
  so you should probably only use it when the new state isn't
  derived from the old state. Otherwise, use
  [`dispatch`](https://codemirror.net/6/docs/ref/#view.EditorView.dispatch) instead.)
  */
  setState(t) {
    if (this.updateState != 0)
      throw new Error("Calls to EditorView.setState are not allowed while an update is in progress");
    if (this.destroyed) {
      this.viewState.state = t;
      return;
    }
    this.updateState = 2;
    let e = this.hasFocus;
    try {
      for (let i of this.plugins)
        i.destroy(this);
      this.viewState = new rn(t), this.plugins = t.facet(It).map((i) => new Oe(i)), this.pluginMap.clear();
      for (let i of this.plugins)
        i.update(this);
      this.docView.destroy(), this.docView = new Vi(this), this.inputState.ensureHandlers(this.plugins), this.mountStyles(), this.updateAttrs(), this.bidiCache = [];
    } finally {
      this.updateState = 0;
    }
    e && this.focus(), this.requestMeasure();
  }
  updatePlugins(t) {
    let e = t.startState.facet(It), i = t.state.facet(It);
    if (e != i) {
      let n = [];
      for (let s of i) {
        let o = e.indexOf(s);
        if (o < 0)
          n.push(new Oe(s));
        else {
          let l = this.plugins[o];
          l.mustUpdate = t, n.push(l);
        }
      }
      for (let s of this.plugins)
        s.mustUpdate != t && s.destroy(this);
      this.plugins = n, this.pluginMap.clear();
    } else
      for (let n of this.plugins)
        n.mustUpdate = t;
    for (let n = 0; n < this.plugins.length; n++)
      this.plugins[n].update(this);
    e != i && this.inputState.ensureHandlers(this.plugins);
  }
  docViewUpdate() {
    for (let t of this.plugins) {
      let e = t.value;
      if (e && e.docViewUpdate)
        try {
          e.docViewUpdate(this);
        } catch (i) {
          st(this.state, i, "doc view update listener");
        }
    }
  }
  /**
  @internal
  */
  measure(t = !0) {
    if (this.destroyed)
      return;
    if (this.measureScheduled > -1 && this.win.cancelAnimationFrame(this.measureScheduled), this.observer.delayedAndroidKey) {
      this.measureScheduled = -1, this.requestMeasure();
      return;
    }
    this.measureScheduled = 0, t && this.observer.forceFlush();
    let e = null, i = this.scrollDOM, n = i.scrollTop * this.scaleY, { scrollAnchorPos: s, scrollAnchorHeight: o } = this.viewState;
    Math.abs(n - this.viewState.scrollTop) > 1 && (o = -1), this.viewState.scrollAnchorHeight = -1;
    try {
      for (let l = 0; ; l++) {
        if (o < 0)
          if (Nn(i))
            s = -1, o = this.viewState.heightMap.height;
          else {
            let d = this.viewState.scrollAnchorAt(n);
            s = d.from, o = d.top;
          }
        this.updateState = 1;
        let h = this.viewState.measure(this);
        if (!h && !this.measureRequests.length && this.viewState.scrollTarget == null)
          break;
        if (l > 5) {
          console.warn(this.measureRequests.length ? "Measure loop restarted more than 5 times" : "Viewport failed to stabilize");
          break;
        }
        let f = [];
        h & 4 || ([this.measureRequests, f] = [f, this.measureRequests]);
        let a = f.map((d) => {
          try {
            return d.read(this);
          } catch (g) {
            return st(this.state, g), fn;
          }
        }), c = be.create(this, this.state, []), u = !1;
        c.flags |= h, e ? e.flags |= h : e = c, this.updateState = 2, c.empty || (this.updatePlugins(c), this.inputState.update(c), this.updateAttrs(), u = this.docView.update(c), u && this.docViewUpdate());
        for (let d = 0; d < f.length; d++)
          if (a[d] != fn)
            try {
              let g = f[d];
              g.write && g.write(a[d], this);
            } catch (g) {
              st(this.state, g);
            }
        if (u && this.docView.updateSelection(!0), !c.viewportChanged && this.measureRequests.length == 0) {
          if (this.viewState.editorHeight)
            if (this.viewState.scrollTarget) {
              this.docView.scrollIntoView(this.viewState.scrollTarget), this.viewState.scrollTarget = null, o = -1;
              continue;
            } else {
              let g = (s < 0 ? this.viewState.heightMap.height : this.viewState.lineBlockAt(s).top) - o;
              if (g > 1 || g < -1) {
                n = n + g, i.scrollTop = n / this.scaleY, o = -1;
                continue;
              }
            }
          break;
        }
      }
    } finally {
      this.updateState = 0, this.measureScheduled = -1;
    }
    if (e && !e.empty)
      for (let l of this.state.facet(ii))
        l(e);
  }
  /**
  Get the CSS classes for the currently active editor themes.
  */
  get themeClasses() {
    return oi + " " + (this.state.facet(ri) ? Cs : As) + " " + this.state.facet(he);
  }
  updateAttrs() {
    let t = cn(this, as, {
      class: "cm-editor" + (this.hasFocus ? " cm-focused " : " ") + this.themeClasses
    }), e = {
      spellcheck: "false",
      autocorrect: "off",
      autocapitalize: "off",
      translate: "no",
      contenteditable: this.state.facet(at) ? "true" : "false",
      class: "cm-content",
      style: `${w.tabSize}: ${this.state.tabSize}`,
      role: "textbox",
      "aria-multiline": "true"
    };
    this.state.readOnly && (e["aria-readonly"] = "true"), cn(this, pi, e);
    let i = this.observer.ignore(() => {
      let n = Qe(this.contentDOM, this.contentAttrs, e), s = Qe(this.dom, this.editorAttrs, t);
      return n || s;
    });
    return this.editorAttrs = t, this.contentAttrs = e, i;
  }
  showAnnouncements(t) {
    let e = !0;
    for (let i of t)
      for (let n of i.effects)
        if (n.is(D.announce)) {
          e && (this.announceDOM.textContent = ""), e = !1;
          let s = this.announceDOM.appendChild(document.createElement("div"));
          s.textContent = n.value;
        }
  }
  mountStyles() {
    this.styleModules = this.state.facet(Nt);
    let t = this.state.facet(D.cspNonce);
    Mt.mount(this.root, this.styleModules.concat(ao).reverse(), t ? { nonce: t } : void 0);
  }
  readMeasured() {
    if (this.updateState == 2)
      throw new Error("Reading the editor layout isn't allowed during an update");
    this.updateState == 0 && this.measureScheduled > -1 && this.measure(!1);
  }
  /**
  Schedule a layout measurement, optionally providing callbacks to
  do custom DOM measuring followed by a DOM write phase. Using
  this is preferable reading DOM layout directly from, for
  example, an event handler, because it'll make sure measuring and
  drawing done by other components is synchronized, avoiding
  unnecessary DOM layout computations.
  */
  requestMeasure(t) {
    if (this.measureScheduled < 0 && (this.measureScheduled = this.win.requestAnimationFrame(() => this.measure())), t) {
      if (this.measureRequests.indexOf(t) > -1)
        return;
      if (t.key != null) {
        for (let e = 0; e < this.measureRequests.length; e++)
          if (this.measureRequests[e].key === t.key) {
            this.measureRequests[e] = t;
            return;
          }
      }
      this.measureRequests.push(t);
    }
  }
  /**
  Get the value of a specific plugin, if present. Note that
  plugins that crash can be dropped from a view, so even when you
  know you registered a given plugin, it is recommended to check
  the return value of this method.
  */
  plugin(t) {
    let e = this.pluginMap.get(t);
    return (e === void 0 || e && e.spec != t) && this.pluginMap.set(t, e = this.plugins.find((i) => i.spec == t) || null), e && e.update(this).value;
  }
  /**
  The top position of the document, in screen coordinates. This
  may be negative when the editor is scrolled down. Points
  directly to the top of the first line, not above the padding.
  */
  get documentTop() {
    return this.contentDOM.getBoundingClientRect().top + this.viewState.paddingTop;
  }
  /**
  Reports the padding above and below the document.
  */
  get documentPadding() {
    return { top: this.viewState.paddingTop, bottom: this.viewState.paddingBottom };
  }
  /**
  If the editor is transformed with CSS, this provides the scale
  along the X axis. Otherwise, it will just be 1. Note that
  transforms other than translation and scaling are not supported.
  */
  get scaleX() {
    return this.viewState.scaleX;
  }
  /**
  Provide the CSS transformed scale along the Y axis.
  */
  get scaleY() {
    return this.viewState.scaleY;
  }
  /**
  Find the text line or block widget at the given vertical
  position (which is interpreted as relative to the [top of the
  document](https://codemirror.net/6/docs/ref/#view.EditorView.documentTop)).
  */
  elementAtHeight(t) {
    return this.readMeasured(), this.viewState.elementAtHeight(t);
  }
  /**
  Find the line block (see
  [`lineBlockAt`](https://codemirror.net/6/docs/ref/#view.EditorView.lineBlockAt) at the given
  height, again interpreted relative to the [top of the
  document](https://codemirror.net/6/docs/ref/#view.EditorView.documentTop).
  */
  lineBlockAtHeight(t) {
    return this.readMeasured(), this.viewState.lineBlockAtHeight(t);
  }
  /**
  Get the extent and vertical position of all [line
  blocks](https://codemirror.net/6/docs/ref/#view.EditorView.lineBlockAt) in the viewport. Positions
  are relative to the [top of the
  document](https://codemirror.net/6/docs/ref/#view.EditorView.documentTop);
  */
  get viewportLineBlocks() {
    return this.viewState.viewportLines;
  }
  /**
  Find the line block around the given document position. A line
  block is a range delimited on both sides by either a
  non-[hidden](https://codemirror.net/6/docs/ref/#view.Decoration^replace) line breaks, or the
  start/end of the document. It will usually just hold a line of
  text, but may be broken into multiple textblocks by block
  widgets.
  */
  lineBlockAt(t) {
    return this.viewState.lineBlockAt(t);
  }
  /**
  The editor's total content height.
  */
  get contentHeight() {
    return this.viewState.contentHeight;
  }
  /**
  Move a cursor position by [grapheme
  cluster](https://codemirror.net/6/docs/ref/#state.findClusterBreak). `forward` determines whether
  the motion is away from the line start, or towards it. In
  bidirectional text, the line is traversed in visual order, using
  the editor's [text direction](https://codemirror.net/6/docs/ref/#view.EditorView.textDirection).
  When the start position was the last one on the line, the
  returned position will be across the line break. If there is no
  further line, the original position is returned.
  
  By default, this method moves over a single cluster. The
  optional `by` argument can be used to move across more. It will
  be called with the first cluster as argument, and should return
  a predicate that determines, for each subsequent cluster,
  whether it should also be moved over.
  */
  moveByChar(t, e, i) {
    return Re(this, t, ji(this, t, e, i));
  }
  /**
  Move a cursor position across the next group of either
  [letters](https://codemirror.net/6/docs/ref/#state.EditorState.charCategorizer) or non-letter
  non-whitespace characters.
  */
  moveByGroup(t, e) {
    return Re(this, t, ji(this, t, e, (i) => Pr(this, t.head, i)));
  }
  /**
  Get the cursor position visually at the start or end of a line.
  Note that this may differ from the _logical_ position at its
  start or end (which is simply at `line.from`/`line.to`) if text
  at the start or end goes against the line's base text direction.
  */
  visualLineSide(t, e) {
    let i = this.bidiSpans(t), n = this.textDirectionAt(t.from), s = i[e ? i.length - 1 : 0];
    return b.cursor(s.side(e, n) + t.from, s.forward(!e, n) ? 1 : -1);
  }
  /**
  Move to the next line boundary in the given direction. If
  `includeWrap` is true, line wrapping is on, and there is a
  further wrap point on the current line, the wrap point will be
  returned. Otherwise this function will return the start or end
  of the line.
  */
  moveToLineBoundary(t, e, i = !0) {
    return Er(this, t, e, i);
  }
  /**
  Move a cursor position vertically. When `distance` isn't given,
  it defaults to moving to the next line (including wrapped
  lines). Otherwise, `distance` should provide a positive distance
  in pixels.
  
  When `start` has a
  [`goalColumn`](https://codemirror.net/6/docs/ref/#state.SelectionRange.goalColumn), the vertical
  motion will use that as a target horizontal position. Otherwise,
  the cursor's own horizontal position is used. The returned
  cursor will have its goal column set to whichever column was
  used.
  */
  moveVertically(t, e, i) {
    return Re(this, t, Br(this, t, e, i));
  }
  /**
  Find the DOM parent node and offset (child offset if `node` is
  an element, character offset when it is a text node) at the
  given document position.
  
  Note that for positions that aren't currently in
  `visibleRanges`, the resulting DOM position isn't necessarily
  meaningful (it may just point before or after a placeholder
  element).
  */
  domAtPos(t) {
    return this.docView.domAtPos(t);
  }
  /**
  Find the document position at the given DOM node. Can be useful
  for associating positions with DOM events. Will raise an error
  when `node` isn't part of the editor content.
  */
  posAtDOM(t, e = 0) {
    return this.docView.posFromDOM(t, e);
  }
  posAtCoords(t, e = !0) {
    return this.readMeasured(), ps(this, t, e);
  }
  /**
  Get the screen coordinates at the given document position.
  `side` determines whether the coordinates are based on the
  element before (-1) or after (1) the position (if no element is
  available on the given side, the method will transparently use
  another strategy to get reasonable coordinates).
  */
  coordsAtPos(t, e = 1) {
    this.readMeasured();
    let i = this.docView.coordsAt(t, e);
    if (!i || i.left == i.right)
      return i;
    let n = this.state.doc.lineAt(t), s = this.bidiSpans(n), o = s[ct.find(s, t - n.from, -1, e)];
    return ci(i, o.dir == z.LTR == e > 0);
  }
  /**
  Return the rectangle around a given character. If `pos` does not
  point in front of a character that is in the viewport and
  rendered (i.e. not replaced, not a line break), this will return
  null. For space characters that are a line wrap point, this will
  return the position before the line break.
  */
  coordsForChar(t) {
    return this.readMeasured(), this.docView.coordsForChar(t);
  }
  /**
  The default width of a character in the editor. May not
  accurately reflect the width of all characters (given variable
  width fonts or styling of invididual ranges).
  */
  get defaultCharacterWidth() {
    return this.viewState.heightOracle.charWidth;
  }
  /**
  The default height of a line in the editor. May not be accurate
  for all lines.
  */
  get defaultLineHeight() {
    return this.viewState.heightOracle.lineHeight;
  }
  /**
  The text direction
  ([`direction`](https://developer.mozilla.org/en-US/docs/Web/CSS/direction)
  CSS property) of the editor's content element.
  */
  get textDirection() {
    return this.viewState.defaultTextDirection;
  }
  /**
  Find the text direction of the block at the given position, as
  assigned by CSS. If
  [`perLineTextDirection`](https://codemirror.net/6/docs/ref/#view.EditorView^perLineTextDirection)
  isn't enabled, or the given position is outside of the viewport,
  this will always return the same as
  [`textDirection`](https://codemirror.net/6/docs/ref/#view.EditorView.textDirection). Note that
  this may trigger a DOM layout.
  */
  textDirectionAt(t) {
    return !this.state.facet(os) || t < this.viewport.from || t > this.viewport.to ? this.textDirection : (this.readMeasured(), this.docView.textDirectionAt(t));
  }
  /**
  Whether this editor [wraps lines](https://codemirror.net/6/docs/ref/#view.EditorView.lineWrapping)
  (as determined by the
  [`white-space`](https://developer.mozilla.org/en-US/docs/Web/CSS/white-space)
  CSS property of its content element).
  */
  get lineWrapping() {
    return this.viewState.heightOracle.lineWrapping;
  }
  /**
  Returns the bidirectional text structure of the given line
  (which should be in the current document) as an array of span
  objects. The order of these spans matches the [text
  direction](https://codemirror.net/6/docs/ref/#view.EditorView.textDirection)—if that is
  left-to-right, the leftmost spans come first, otherwise the
  rightmost spans come first.
  */
  bidiSpans(t) {
    if (t.length > vo)
      return Zn(t.length);
    let e = this.textDirectionAt(t.from), i;
    for (let s of this.bidiCache)
      if (s.from == t.from && s.dir == e && (s.fresh || Qn(s.isolates, i = Hi(this, t))))
        return s.order;
    i || (i = Hi(this, t));
    let n = dr(t.text, e, i);
    return this.bidiCache.push(new xe(t.from, t.to, e, i, !0, n)), n;
  }
  /**
  Check whether the editor has focus.
  */
  get hasFocus() {
    var t;
    return (this.dom.ownerDocument.hasFocus() || w.safari && ((t = this.inputState) === null || t === void 0 ? void 0 : t.lastContextMenu) > Date.now() - 3e4) && this.root.activeElement == this.contentDOM;
  }
  /**
  Put focus on the editor.
  */
  focus() {
    this.observer.ignore(() => {
      Fn(this.contentDOM), this.docView.updateSelection();
    });
  }
  /**
  Update the [root](https://codemirror.net/6/docs/ref/##view.EditorViewConfig.root) in which the editor lives. This is only
  necessary when moving the editor's existing DOM to a new window or shadow root.
  */
  setRoot(t) {
    this._root != t && (this._root = t, this.observer.setWindow((t.nodeType == 9 ? t : t.ownerDocument).defaultView || window), this.mountStyles());
  }
  /**
  Clean up this editor view, removing its element from the
  document, unregistering event handlers, and notifying
  plugins. The view instance can no longer be used after
  calling this.
  */
  destroy() {
    this.root.activeElement == this.contentDOM && this.contentDOM.blur();
    for (let t of this.plugins)
      t.destroy(this);
    this.plugins = [], this.inputState.destroy(), this.docView.destroy(), this.dom.remove(), this.observer.destroy(), this.measureScheduled > -1 && this.win.cancelAnimationFrame(this.measureScheduled), this.destroyed = !0;
  }
  /**
  Returns an effect that can be
  [added](https://codemirror.net/6/docs/ref/#state.TransactionSpec.effects) to a transaction to
  cause it to scroll the given position or range into view.
  */
  static scrollIntoView(t, e = {}) {
    return ie.of(new kt(typeof t == "number" ? b.cursor(t) : t, e.y, e.x, e.yMargin, e.xMargin));
  }
  /**
  Return an effect that resets the editor to its current (at the
  time this method was called) scroll position. Note that this
  only affects the editor's own scrollable element, not parents.
  See also
  [`EditorViewConfig.scrollTo`](https://codemirror.net/6/docs/ref/#view.EditorViewConfig.scrollTo).
  
  The effect should be used with a document identical to the one
  it was created for. Failing to do so is not an error, but may
  not scroll to the expected position. You can
  [map](https://codemirror.net/6/docs/ref/#state.StateEffect.map) the effect to account for changes.
  */
  scrollSnapshot() {
    let { scrollTop: t, scrollLeft: e } = this.scrollDOM, i = this.viewState.scrollAnchorAt(t);
    return ie.of(new kt(b.cursor(i.from), "start", "start", i.top - t, e, !0));
  }
  /**
  Enable or disable tab-focus mode, which disables key bindings
  for Tab and Shift-Tab, letting the browser's default
  focus-changing behavior go through instead. This is useful to
  prevent trapping keyboard users in your editor.
  
  Without argument, this toggles the mode. With a boolean, it
  enables (true) or disables it (false). Given a number, it
  temporarily enables the mode until that number of milliseconds
  have passed or another non-Tab key is pressed.
  */
  setTabFocusMode(t) {
    t == null ? this.inputState.tabFocusMode = this.inputState.tabFocusMode < 0 ? 0 : -1 : typeof t == "boolean" ? this.inputState.tabFocusMode = t ? 0 : -1 : this.inputState.tabFocusMode != 0 && (this.inputState.tabFocusMode = Date.now() + t);
  }
  /**
  Returns an extension that can be used to add DOM event handlers.
  The value should be an object mapping event names to handler
  functions. For any given event, such functions are ordered by
  extension precedence, and the first handler to return true will
  be assumed to have handled that event, and no other handlers or
  built-in behavior will be activated for it. These are registered
  on the [content element](https://codemirror.net/6/docs/ref/#view.EditorView.contentDOM), except
  for `scroll` handlers, which will be called any time the
  editor's [scroll element](https://codemirror.net/6/docs/ref/#view.EditorView.scrollDOM) or one of
  its parent nodes is scrolled.
  */
  static domEventHandlers(t) {
    return Xt.define(() => ({}), { eventHandlers: t });
  }
  /**
  Create an extension that registers DOM event observers. Contrary
  to event [handlers](https://codemirror.net/6/docs/ref/#view.EditorView^domEventHandlers),
  observers can't be prevented from running by a higher-precedence
  handler returning true. They also don't prevent other handlers
  and observers from running when they return true, and should not
  call `preventDefault`.
  */
  static domEventObservers(t) {
    return Xt.define(() => ({}), { eventObservers: t });
  }
  /**
  Create a theme extension. The first argument can be a
  [`style-mod`](https://github.com/marijnh/style-mod#documentation)
  style spec providing the styles for the theme. These will be
  prefixed with a generated class for the style.
  
  Because the selectors will be prefixed with a scope class, rule
  that directly match the editor's [wrapper
  element](https://codemirror.net/6/docs/ref/#view.EditorView.dom)—to which the scope class will be
  added—need to be explicitly differentiated by adding an `&` to
  the selector for that element—for example
  `&.cm-focused`.
  
  When `dark` is set to true, the theme will be marked as dark,
  which will cause the `&dark` rules from [base
  themes](https://codemirror.net/6/docs/ref/#view.EditorView^baseTheme) to be used (as opposed to
  `&light` when a light theme is active).
  */
  static theme(t, e) {
    let i = Mt.newName(), n = [he.of(i), Nt.of(li(`.${i}`, t))];
    return e && e.dark && n.push(ri.of(!0)), n;
  }
  /**
  Create an extension that adds styles to the base theme. Like
  with [`theme`](https://codemirror.net/6/docs/ref/#view.EditorView^theme), use `&` to indicate the
  place of the editor wrapper element when directly targeting
  that. You can also use `&dark` or `&light` instead to only
  target editors with a dark or light theme.
  */
  static baseTheme(t) {
    return Fs.lowest(Nt.of(li("." + oi, t, Ms)));
  }
  /**
  Retrieve an editor view instance from the view's DOM
  representation.
  */
  static findFromDOM(t) {
    var e;
    let i = t.querySelector(".cm-content"), n = i && M.get(i) || M.get(t);
    return ((e = n == null ? void 0 : n.rootView) === null || e === void 0 ? void 0 : e.view) || null;
  }
}
D.styleModule = Nt;
D.inputHandler = ss;
D.scrollHandler = ls;
D.focusChangeEffect = rs;
D.perLineTextDirection = os;
D.exceptionSink = ns;
D.updateListener = ii;
D.editable = at;
D.mouseSelectionStyle = is;
D.dragMovesSelection = es;
D.clickAddsSelectionRange = ts;
D.decorations = Jt;
D.outerDecorations = fs;
D.atomicRanges = mi;
D.bidiIsolatedRanges = cs;
D.scrollMargins = us;
D.darkTheme = ri;
D.cspNonce = /* @__PURE__ */ v.define({ combine: (r) => r.length ? r[0] : "" });
D.contentAttributes = pi;
D.editorAttributes = as;
D.lineWrapping = /* @__PURE__ */ D.contentAttributes.of({ class: "cm-lineWrapping" });
D.announce = /* @__PURE__ */ F.define();
const vo = 4096, fn = {};
class xe {
  constructor(t, e, i, n, s, o) {
    this.from = t, this.to = e, this.dir = i, this.isolates = n, this.fresh = s, this.order = o;
  }
  static update(t, e) {
    if (e.empty && !t.some((s) => s.fresh))
      return t;
    let i = [], n = t.length ? t[t.length - 1].dir : z.LTR;
    for (let s = Math.max(0, t.length - 10); s < t.length; s++) {
      let o = t[s];
      o.dir == n && !e.touchesRange(o.from, o.to) && i.push(new xe(e.mapPos(o.from, 1), e.mapPos(o.to, -1), o.dir, o.isolates, !1, o.order));
    }
    return i;
  }
}
function cn(r, t, e) {
  for (let i = r.state.facet(t), n = i.length - 1; n >= 0; n--) {
    let s = i[n], o = typeof s == "function" ? s(r) : s;
    o && Je(o, e);
  }
  return e;
}
const ko = !w.ios, un = {
  ".cm-line": {
    "& ::selection, &::selection": { backgroundColor: "transparent !important" }
  },
  ".cm-content": {
    "& :focus": {
      caretColor: "initial !important",
      "&::selection, & ::selection": {
        backgroundColor: "Highlight !important"
      }
    }
  }
};
ko && (un[".cm-line"].caretColor = un[".cm-content"].caretColor = "transparent !important");
class Et extends Ct {
  /**
  @internal
  */
  compare(t) {
    return this == t || this.constructor == t.constructor && this.eq(t);
  }
  /**
  Compare this marker to another marker of the same type.
  */
  eq(t) {
    return !1;
  }
  /**
  Called if the marker has a `toDOM` method and its representation
  was removed from a gutter.
  */
  destroy(t) {
  }
}
Et.prototype.elementClass = "";
Et.prototype.toDOM = void 0;
Et.prototype.mapMode = j.TrackBefore;
Et.prototype.startSide = Et.prototype.endSide = -1;
Et.prototype.point = !0;
const dn = (r) => `![loading...](${r.name})`, gn = (r) => /^http(s)?:\/\/([\w-]+\.)+[\w-]+(\/[\w- ./?%&=]*)?$/.test(r), pn = async (r, t) => {
  if (t.upload)
    return await t.upload(r) || "上传失败";
  if (typeof t.storageServer == "string" && t.callback instanceof Function) {
    const n = `${(/* @__PURE__ */ new Date()).getTime()}.${r.name.split(".").pop()}`;
    r = new File([r], n, { type: r.type });
    const s = new FormData();
    s.append("file", r);
    const o = await fetch("/api/v1/document/page/imagePut", {
      method: "POST",
      body: s
    }).then((h) => h.json());
    let l = t.callback(o);
    return (typeof l != "string" || l === "") && console.error("callback function should return a not empty string"), l || "上传失败";
  }
  return console.error("upload function or storageServer is required");
};
let mn = null;
const Co = (r) => D.domEventHandlers({
  paste(t, e) {
    const i = t.clipboardData;
    if (!i)
      return console.error("clipboardData is empty");
    for (let n = 0; n < i.items.length; n++) {
      const s = i.items[n];
      if (s.kind === "file" && s.type.includes("image")) {
        const o = s.getAsFile(), l = dn(o), { from: h, to: f } = e.state.selection.ranges[0];
        e.dispatch({
          changes: {
            from: h,
            to: f,
            insert: l
          }
        });
        const a = f + l.length;
        pn(o, r).then((c) => {
          if (!c || !gn(c))
            return console.error(`url is not valid: ${c}`);
          const u = `![${o.name}](${c})`;
          e.dispatch({
            changes: {
              from: h,
              to: a,
              insert: u
            },
            selection: {
              anchor: h + u.length,
              head: h + u.length
            }
          });
        });
      }
    }
  },
  drop(t, e) {
    var o, l;
    t.preventDefault();
    const i = (l = (o = t.dataTransfer) == null ? void 0 : o.files) == null ? void 0 : l[0];
    if (!i)
      return console.error("file is required");
    let n = mn, s = dn(i);
    n || (n = e.posAtCoords({ x: t.clientX, y: t.clientY })), n && (e.dispatch({
      changes: {
        from: n,
        to: n,
        insert: s
      }
    }), pn(i, r).then((h) => {
      if (!h || !gn(h))
        return console.error(`url is not valid: ${h}`);
      const f = `![${i.name}](${h})`;
      e.dispatch({
        changes: {
          from: n,
          to: n + s.length,
          insert: f
        },
        selection: {
          anchor: n + f.length,
          head: n + f.length
        }
      });
    }));
  },
  dragover(t, e) {
    const { clientX: i, clientY: n } = t;
    mn = e.posAtCoords({ x: i, y: n });
  }
});
export {
  Co as PicUploadExt
};
